/*
 * $QNXLicenseC:
 * Copyright 2007, 2008, QNX Software Systems.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You
 * may not reproduce, modify or distribute this software except in
 * compliance with the License. You may obtain a copy of the License
 * at: http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as
 * contributors under the License or as licensors under other terms.
 * Please review this entire file for other proprietary rights or license
 * notices, as well as the QNX Development Suite License Guide at
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */


// Module Description:  MMC command processing

#include <sim_mmc.h>

//#define MMCSD_DEBUG

#ifndef CAM_ALIGN_MASK
#define CAM_ALIGN_MASK( _val, _mask )	( ( (_val) + (_mask) ) & ~(_mask) )
#endif

#ifndef CAM_ALIGN
#define CAM_ALIGN( _val, _align )		CAM_ALIGN_MASK( _val, ( typeof( _val ) )(_align) - 1 )
#endif

static const uint32_t mmcsd_ts_exp[] = { 100, 1000, 10000, 100000, 0, 0, 0, 0 };
static const uint32_t mmcsd_ts_mul[] = { 0, 1000, 1200, 1300, 1500, 2000, 2500, 3000,
						3500, 4000, 4500, 5000, 5500, 6000, 7000, 8000 };


char *mmc_idents[] = {	"MMCSD_IDENT_NONE", "MMCSD_IDENT_START",
						"MMCSD_IDENT_GO_IDLE", "MMCSD_IDENT_CARD_VERSION",
						"MMCSD_IDENT_SD_POWERUP", " ", "MMCSD_IDENT_MMC_POWERUP",
						" "," ", " ",
						"MMCSD_IDENT_ALL_SEND_CID", "MMCSD_IDENT_SET_ADDRESS",
						"MMCSD_IDENT_SEND_CID", "MMCSD_IDENT_SEND_CSD",
						"MMCSD_IDENT_SEND_SCR", "MMCSD_IDENT_PARSE_SCR",
						"MMCSD_IDENT_CHECK_MODE", "MMCSD_IDENT_SWITCH_MODE",
						"MMCSD_IDENT_SEL_DES_CARD", "MMCSD_IDENT_CHECK_STS",
						"MMCSD_IDENT_SET_BW_APP", "MMCSD_IDENT_SET_BW",
						"MMCSD_IDENT_EXT_CSD","MMCSD_IDENT_MMC_SWITCH_MODE_HS","MMCSD_IDENT_MMC_SWITCH_MODE_BUSWIDTH",
						" "," "," "," "," ",
						"MMCSD_IDENT_FINAL" };

char *mmc_cmds[] = {	"MMC_GO_IDLE_STATE", "MMC_SEND_OP_COND",
						"MMC_ALL_SEND_CID", "MMC_SET_RELATIVE_ADDR",
						"MMC_SET_DSR", "MMC_SLEEP_AWAKE", "MMC_SWITCH",
						"MMC_SEL_DES_CARD", "MMC_IF_COND", "MMC_SEND_CSD",
						"MMC_SEND_CID", "MMC_READ_DAT_UNTIL_STOP",
						"MMC_STOP_TRANSMISSION", "MMC_SEND_STATUS", "",
						"MMC_GO_INACTIVE_STATE", "MMC_SET_BLOCKLEN",
						"MMC_READ_SINGLE_BLOCK", "MMC_READ_MULTIPLE_BLOCK",
						"", "MMC_WRITE_DAT_UNTIL_STOP", "", "", "",
						"MMC_WRITE_BLOCK", "MMC_WRITE_MULTIPLE_BLOCK",
						"MMC_PROGRAM_CID", "MMC_PROGRAM_CSD",
						"MMC_SET_WRITE_PROT", "MMC_CLR_WRITE_PROT",
						"MMC_SEND_WRITE_PROT", "", "MMC_TAG_SECTOR_START",
						"MMC_TAG_SECTOR_END", "MMC_UNTAG_SECTOR",
						"MMC_TAG_ERASE_GROUP_START", "MMC_TAG_ERASE_GROUP_END",
						"MMC_UNTAG_ERASE_GROUP", "MMC_ERASE", "MMC_FAST_IO",
						"MMC_GO_IRQ_STATE", "SD_SEND_OP_COND", "MMC_LOCK_UNLOCK",
						"", "", "", "", "", "", "", "", "SD_SEND_SCR", "", "",
						"", "MMC_APP_CMD", "MMC_GEN_CMD", "", "MMC_READ_OCR",
						"MMC_CRC_ON_OFF" };


mmcsd_device_errata_t mmc_errata[] = {
		// device will discard data when AC12 is used
	{ "MAG2GA", 0x15, 0x0, 0x0, 0x6, DEV_ERRATA_ACMD12 },
		// device will discard data when AC12 is used
	{ "MAG4GA", 0x15, 0x0, 0x0, 0x6, DEV_ERRATA_ACMD12 },
	{ NULL, 0, 0, 0, 0 }
};

mmcsd_device_errata_t *mmcsd_device_errata( SIM_HBA *hba, mmcsd_device_errata_t *erratas )
{
	SIM_MMC_EXT				*ext;
	mmc_cid_t				*cid;
	mmcsd_device_errata_t	*errata;

	ext = (SIM_MMC_EXT *)hba->ext;
	cid = &ext->cid.mmc_cid;

	for( errata = erratas; errata->errata; errata++ ) {
		if( !strcmp( (char *)cid->pnm, errata->pnm ) &&
				cid->mid == errata->mid && cid->oid == errata->oid &&
				cid->fwr >= errata->prv_s && cid->fwr <= errata->prv_e ) {
			return( errata );
		}
	}

	return( NULL );
}

int mmcsd_reconcile_errata( SIM_HBA *hba, mmcsd_device_errata_t *errata )
{
	SIM_MMC_EXT	*ext;

	ext = (SIM_MMC_EXT *)hba->ext;

	if( ( errata->errata & DEV_ERRATA_ACMD12 ) ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: disabling ACMD12", __FUNCTION__ );
		ext->hccap &= ~MMC_HCCAP_ACMD12;
	}

	return( EOK );
}

void mmc_dump( char *buffer, long cnt )
{
	char	*ptr, output[80];
	int		res, i, j, k, n;
	long	l;
	char	ref[80];

	l = 0L;
	res = cnt;
	ptr = buffer;
	while( res ) {
#define BL 16
#define AL BL/2*5+4
		sprintf( output, "%08ld:  %-*.*s\"%-*.*s\"", l, AL, AL, "  ", BL, BL, "  " );
//		sprintf( output, "%08lx:  %-*.*s\"%-*.*s\"", l, AL, AL, "  ", BL, BL, "  " );
		k = strlen( output ) - (BL + 1);
		j = l % BL;
		i = 12 + (j << 1) + (j >> 1) + (j >= (BL / 2) ? 2 : 0);
		for( ; j < BL && res; j++, res--, ptr++, l++ ) {
			n = ((int) *ptr >> 4) & 0xf;
			output [i++] = (n < 10 ? (char) n + '0': (char) n + ('a' - 10));
			n = (int) *ptr & 0xf;
			output [i++] = (n < 10 ? (char) n + '0': (char) n + ('a' - 10));
			if (j & 1)
				i++;
			if (j == BL / 2 - 1)
				i += 2;
#define UC unsigned char
			output [j + k] = ((UC) *ptr < ' ' || (UC) *ptr > '~') ? '.' : *ptr;
#undef UC
		}
		mmc_slogf( NULL, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s", output );
		memcpy( ref, output, 80 );
#undef BL
#undef AL
	}
}

int _mmc_sendcmd(SIM_HBA *hba, uint32_t cmdflg, uint32_t argument)
{
	SIM_MMC_EXT		*ext;
	mmc_cmd_t		*cmd;

	ext = (SIM_MMC_EXT *)hba->ext;
	cmd = &ext->cmd;

	/* Put the card in idle state */
	cmd->argument = argument;
	cmd->opcode   = cmdflg & 0xFF;
	cmd->resp     = (cmdflg >> 8) & 0xFF;
	cmd->eflags   = (cmdflg >> 16) & 0xFFFF;
	if (ext->eflags & MMC_EFLAG_PPL)
		cmd->eflags  |= MMC_CMD_PPL;

	ext->mmc_resp[0] = ext->mmc_resp[1] = ext->mmc_resp[2] = ext->mmc_resp[3] = 0;
	ext->resp_type = cmd->resp;

#ifdef MMCSD_DEBUG
	if( !( cmd->opcode == MMC_WRITE_BLOCK || cmd->opcode == MMC_WRITE_MULTIPLE_BLOCK
			|| cmd->opcode == MMC_READ_SINGLE_BLOCK || cmd->opcode == MMC_READ_MULTIPLE_BLOCK ) ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: %s (%d), arg 0x%x, resp 0x%x, eflags 0x%x", __FUNCTION__, mmc_cmds[cmd->opcode], cmd->opcode, cmd->argument, cmd->resp, cmd->eflags );
	}
#endif

	if(ext->cmdlog){
		if(ext->cmdlog>=5){ //more readable form
			slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMCSD: %s: %d, arg 0x%x, resp 0x%x, eflags 0x%x", mmc_cmds[cmd->opcode], cmd->opcode, cmd->argument, cmd->resp, cmd->eflags );
		}else if(ext->cmdlog>3){ //log all commands
			slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMCSD: cmd %d, arg 0x%x, resp 0x%x", cmd->opcode, cmd->argument, cmd->resp);
		}else if(ext->cmdlog>2){ //log all commands, read/write command will be loged in send_rw call
			if(!(cmd->opcode == MMC_WRITE_BLOCK || cmd->opcode == MMC_WRITE_MULTIPLE_BLOCK
				|| cmd->opcode == MMC_READ_SINGLE_BLOCK || cmd->opcode == MMC_READ_MULTIPLE_BLOCK)){
				slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMCSD: cmd %d, arg 0x%x, resp 0x%x", cmd->opcode, cmd->argument, cmd->resp);
			}
		}else if(ext->cmdlog>1){ //log all commands except send status, read/write command will be loged in send_rw call
			if(!(cmd->opcode == MMC_WRITE_BLOCK || cmd->opcode == MMC_WRITE_MULTIPLE_BLOCK
				|| cmd->opcode == MMC_READ_SINGLE_BLOCK || cmd->opcode == MMC_READ_MULTIPLE_BLOCK
				|| cmd->opcode == MMC_SEND_STATUS)){
				slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMCSD: cmd %d, arg 0x%x, resp 0x%x", cmd->opcode, cmd->argument, cmd->resp);
			}
		}
	}

	return (ext->command(hba, cmd));
}

int mmc_get_card_status( SIM_HBA *hba )
{
	SIM_MMC_EXT	*ext;

	ext = (SIM_MMC_EXT *)hba->ext;

	if( mmc_sendcmd( hba, MMC_SEND_STATUS, MMC_RSP_R1, MMC_CMD_INTR, ext->rca << 16 ) != MMC_SUCCESS ) {
		return( MMC_FAILURE );
	}

	return( mmc_wait_command( hba, MMC_TIME_DEFAULT ) );
}

int mmc_switch( SIM_HBA *hba, int flgs, uint32_t cmdset, uint32_t mode, uint32_t index, uint32_t value )
{
	SIM_MMC_EXT		*ext;
	int		status;

	ext		= (SIM_MMC_EXT *)hba->ext;
	if( ( status = mmc_sendcmd( hba, MMC_SWITCH, MMC_RSP_R1B, MMC_CMD_INTR, ( mode << 24 ) | ( index << 16 ) | ( value << 8 ) | cmdset ) ) == MMC_SUCCESS && flgs ) {
		status = mmc_wait_command( hba, MMC_TIME_DEFAULT );
		if( ( status = mmc_get_card_status( hba ) ) == MMC_SUCCESS ) {
			if( ( ext->mmc_resp[0] & MMC_SWITCH_ERROR ) ) {
				mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: idx %d, val %x, card status resp %x", __FUNCTION__, index, value, ext->mmc_resp[0] );
				status = MMC_ERROR;
			}
		}
	}
	return( status );
}

int mmc_send_cid( SIM_HBA *hba, uint32_t *cid )
{
	SIM_MMC_EXT		*ext;
	int				status;

	ext		= (SIM_MMC_EXT *)hba->ext;
	status	= MMC_SUCCESS;

	if( ( status = mmc_sendcmd( hba, MMC_SEND_CID, MMC_RSP_R2, MMC_CMD_INTR, ext->rca << 16 ) ) == MMC_SUCCESS ) {
		status = mmc_wait_command( hba, MMC_TIME_DEFAULT );
	}

	cid[0] = ext->mmc_resp[0];
	cid[1] = ext->mmc_resp[1];
	cid[2] = ext->mmc_resp[2];
	cid[3] = ext->mmc_resp[3];

	return( status );
}

int mmc_send_csd( SIM_HBA *hba, uint32_t *csd )
{
	SIM_MMC_EXT		*ext;
	int				status;

	ext		= (SIM_MMC_EXT *)hba->ext;
	status	= MMC_SUCCESS;

	if( ( status = mmc_sendcmd( hba, MMC_SEND_CSD, MMC_RSP_R2, MMC_CMD_INTR, ext->rca << 16 ) ) == MMC_SUCCESS ) {
		if( ( status = mmc_wait_command( hba, MMC_TIME_DEFAULT ) ) == MMC_SUCCESS ) {
			csd[0] = ext->mmc_resp[0];
			csd[1] = ext->mmc_resp[1];
			csd[2] = ext->mmc_resp[2];
			csd[3] = ext->mmc_resp[3];
		}
	}

	return( status );
}

int mmc_send_ext_csd( SIM_HBA *hba, uint8_t *csd )
{
	SIM_MMC_EXT		*ext;
	int				status;

	ext			= (SIM_MMC_EXT *)hba->ext;
	status		= MMC_SUCCESS;

	ext->dir	= MMC_DIR_IN;
	ext->set_blksz( hba, MMC_DFLT_BLKSIZE );
	ext->setup_pio( hba, EXT_CSD_SIZE, MMC_DIR_IN );
	if( ( status = mmc_sendcmd( hba, MMC_SEND_EXT_CSD, MMC_RSP_R1, MMC_CMD_DATA_IN | MMC_CMD_DATA, 0 ) ) == MMC_SUCCESS ) {
		if( ( status = mmc_wait_command( hba, MMC_TIME_DEFAULT ) ) == MMC_SUCCESS ) {
			ext->pio_done( hba, (char *)csd, EXT_CSD_SIZE, MMC_DIR_IN );
		}
	}

	return( status );
}

int mmc_select_deselect_card( SIM_HBA *hba, int rca )
{
	int				status;

	status	= MMC_SUCCESS;

	if( ( status = mmc_sendcmd( hba, MMC_SEL_DES_CARD, rca ? MMC_RSP_R1: MMC_RSP_R1B, MMC_CMD_INTR, rca << 16 ) ) == MMC_SUCCESS ) {
		status = mmc_wait_command( hba, MMC_TIME_DEFAULT );
	}

	return( status );
}

int mmc_sleep_awake( SIM_HBA *hba, int flgs )
{
	SIM_MMC_EXT		*ext;
	int				status;
	uint32_t		arg;

	ext		= (SIM_MMC_EXT *)hba->ext;
	status	= MMC_SUCCESS;
	arg		= ext->rca << 16;

	if( flgs ) {
		arg |= MMC_SA_SLEEP;
	}

	if( ( status = mmc_sendcmd( hba, MMC_SLEEP_AWAKE, MMC_RSP_R1B, MMC_CMD_INTR, arg ) ) == MMC_SUCCESS ) {
		status = mmc_wait_command( hba, MMC_TIME_DEFAULT );
	}

	return( status );
}

int mmc_wait_card_status( SIM_HBA *hba, uint32_t mask, uint32_t val, uint32_t msec )
{
	SIM_MMC_EXT			*ext;
	int					cnt;
	int					stat;
	uint32_t			iter;

	ext		= (SIM_MMC_EXT *)hba->ext;
	stat	= MMC_FAILURE;

		// fast poll for 10ms - 1ms intervals
	for( cnt = min( msec, 10 ); cnt; cnt-- ) {
		if( mmc_get_card_status( hba ) != MMC_SUCCESS ) {
			stat = MMC_FAILURE;
			break;
		}

		if( ( ext->mmc_resp[0] & mask ) == val ) {
			stat = MMC_SUCCESS;
			break;
		}

		if( ( ext->mmc_resp[0] & MMC_ERROR ) ) {
			stat = MMC_FAILURE;
			break;
		}

		nanospin_ns( 1000000L );
	}

	if( msec > 10 && stat ) {
		iter	= msec / 10L;
		for( cnt = iter; cnt; cnt-- ) {
			if( mmc_get_card_status( hba ) != MMC_SUCCESS ) {
				stat = MMC_FAILURE;
				break;
			}

			if( ( ext->mmc_resp[0] & mask ) == val ) {
				stat = MMC_SUCCESS;
				break;
			}

			if( ( ext->mmc_resp[0] & MMC_ERROR ) ) {
				stat = MMC_FAILURE;
				break;
			}

			delay( 10 );
		}
	}

	if( !cnt ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s:  mask %x, val %x, card status %x", __FUNCTION__, mask, val, ext->mmc_resp[0] );
	}

	return( stat );
}

int mmc_erase( SIM_HBA *hba, int partition, int flgs, uint64_t lba, int nlba )
{
	SIM_MMC_EXT		*ext;
	int				status;
	uint8_t			*ecsd;
	uint32_t		tmult;
	uint32_t		timeout;
	uint64_t		addr;
	uint64_t		erase_grp_size;
#ifdef MMCSD_DEBUG
	uint64_t		wp_grp_size;
	wp_grp_size		= mmc_wp_grp_size( hba ) / 512;
#endif

	ext				= (SIM_MMC_EXT *)hba->ext;
	status			= MMC_SUCCESS;
	ecsd			= ext->extended_csd;
	tmult			= MMC_ERASE_TIMEOUT;
	
	erase_grp_size	= mmc_erase_grp_size( hba ) / 512;

#ifdef MMCSD_DEBUG
	mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: WP_GRP_SIZE %lld, lba %lld, nblks %d", __FUNCTION__, wp_grp_size, lba, nlba );
#endif

	if( mmc_set_partition( hba, partition ) == MMC_FAILURE ) {
		return( MMC_FAILURE );
	}

	if( ecsd[EXT_CSD_ERASE_GRP_DEF] ) {
		switch( flgs ) {
			case MMC_ERASE_NORM:
				tmult = MMC_ERASE_TIMEOUT * ecsd[EXT_CSD_ERASE_MULT];
				break;

			case MMC_ERASE_TRIM:
				tmult = MMC_ERASE_TIMEOUT * ecsd[EXT_CSD_TRIM_MULT];
				break;

			case MMC_ERASE_SECURE_TRIM:
				tmult = MMC_ERASE_TIMEOUT * ecsd[EXT_CSD_ERASE_MULT] * ecsd[EXT_CSD_SEC_TRIM_MULT];
				break;

			case MMC_ERASE_SECURE:
				tmult = MMC_ERASE_TIMEOUT * ecsd[EXT_CSD_ERASE_MULT] * ecsd[EXT_CSD_SEC_ERASE_MULT];
				break;

			default:
				tmult = MMC_ERASE_TIMEOUT * ecsd[EXT_CSD_ERASE_MULT];
				break;
		}
	}

	while( nlba ) {
		addr = ( ext->eflags & MMC_EFLAG_HC ) ? lba : ( lba * ext->mmc_blksize );
		if( ( status = mmc_sendcmd( hba, MMC_TAG_ERASE_GROUP_START, MMC_RSP_R1, MMC_CMD_INTR, addr ) ) == MMC_SUCCESS ) {
			status = mmc_wait_command( hba, MMC_TIME_DEFAULT );
		}

#if 1
//		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: lba %lld to %lld, timeout %d", __FUNCTION__, lba, lba + nlba - 1, tmult * ( nlba / erase_grp_size ) );
		timeout = tmult * ( nlba / erase_grp_size );
		lba		+= nlba;
		nlba	-= nlba;
#else
//		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: lba %lld to %lld, timeout %d", __FUNCTION__, lba, lba + min( nlba, wp_grp_size ) - 1, tmult * ( wp_grp_size / erase_grp_size ) );
		timeout = tmult * ( wp_grp_size / erase_grp_size );
		lba		+= min( nlba, wp_grp_size );
		nlba	-= min( nlba, wp_grp_size );
#endif
		lba--;
		addr = ( ext->eflags & MMC_EFLAG_HC ) ? lba : ( lba * ext->mmc_blksize );
		if( !status && ( status = mmc_sendcmd( hba, MMC_TAG_ERASE_GROUP_END, MMC_RSP_R1, MMC_CMD_INTR, addr ) ) == MMC_SUCCESS ) {
			status = mmc_wait_command( hba, MMC_TIME_DEFAULT );
		}

		// Set to R1, we will poll for card status
//		if( !status && ( status = mmc_sendcmd( hba, MMC_ERASE, MMC_RSP_R1B, MMC_CMD_INTR, flgs ) ) == MMC_SUCCESS ) {
		if( !status && ( status = mmc_sendcmd( hba, MMC_ERASE, MMC_RSP_R1, MMC_CMD_INTR, flgs ) ) == MMC_SUCCESS ) {
			status = mmc_wait_command( hba, MMC_TIME_INFINITY );
		}

		if( !status && ( status = mmc_wait_card_status( hba, MMC_READY_FOR_DATA, MMC_READY_FOR_DATA, timeout ) ) ) {

		}

		if( status ) {
			break;
		}
	}

	return( status );
}

int mmc_write_protect( SIM_HBA *hba, int op, int ptype, int mode, uint32_t lba, uint32_t nlba )
{
	SIM_MMC_EXT		*ext;
	int				status;
	uint64_t		addr;
	uint64_t		wp_grp_size;

	ext			= (SIM_MMC_EXT *)hba->ext;
	wp_grp_size = mmc_wp_grp_size( hba ) / 512;

#ifdef MMCSD_DEBUG
	mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: WP_GRP_SIZE %lld, lba %d, nblks %d, mode %x", __FUNCTION__, wp_grp_size, lba, nlba, mode );
#endif

	if( op ) {
		switch( ptype ) {
			case MMC_PTYPE_USER:
				if( ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_USER_WP, mode ) ) == MMC_FAILURE ) {
					mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_user_wp", __FUNCTION__ );
					return( status );
				}
				break;

			case MMC_PTYPE_BOOT:
				if( ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_BOOT_WP, mode ) ) == MMC_FAILURE ) {
					mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_boot_wp", __FUNCTION__ );
					return( status );
				}
				break;

			case MMC_PTYPE_GP:
			default:
				break;
		}
	}

	while( nlba ) {
		addr = ( ext->eflags & MMC_EFLAG_HC ) ? lba : ( lba * ext->mmc_blksize );
		if( ( status = mmc_sendcmd( hba, op ? MMC_SET_WRITE_PROT : MMC_CLR_WRITE_PROT,
				MMC_RSP_R1B, MMC_CMD_INTR, addr ) ) == MMC_SUCCESS ) {
			status = mmc_wait_command( hba, MMC_TIME_DEFAULT );
		}

		if( status ) {
			return( status );
		}

		lba		+= wp_grp_size;
		nlba	-= wp_grp_size;
	}

	return( MMC_SUCCESS );
}

int mmc_rpmb_rw_frame( SIM_HBA *hba, MMCSD_RPMB_FRAME *rpmb_frame, int partition, int dir, int reliable )
{
	SIM_MMC_EXT	*ext;
	int			cmd;
	int			flag;

	ext = (SIM_MMC_EXT *) hba->ext;
	flag = MMC_CMD_DATA | MMC_CMD_DATA_MULTI;
	cmd = dir == MMC_DIR_IN ? MMC_READ_MULTIPLE_BLOCK : MMC_WRITE_MULTIPLE_BLOCK;

	if( dir == MMC_DIR_IN ) {
		flag |= MMC_CMD_DATA_IN;
	}

	if( mmc_wait_card_ready( hba ) != MMC_SUCCESS ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: wait for card ready failed!", __FUNCTION__ );
		return( MMC_FAILURE );
	}

// mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: tid %d, lun %d, pact %d", __FUNCTION__, ext->nexus->cam_ch.cam_target_id, ext->nexus->cam_ch.cam_target_lun, ext->pactive );

	// switch partition to RPMB if necessary. CMD6
	if( mmc_set_partition( hba, partition ) == MMC_FAILURE ) {
		return( MMC_FAILURE );
	}

	// set block count. CMD23
	if( mmc_sendcmd( hba, MMC_SET_BLOCK_COUNT, MMC_RSP_R1, MMC_CMD_INTR, reliable ? 0x80000001 : 0x1 )
			!= MMC_SUCCESS ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: send set blk cnt cmd failed!", __FUNCTION__ );
		return( MMC_FAILURE );
	}

	if( mmc_wait_command( hba, MMC_TIME_DEFAULT ) != MMC_SUCCESS ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: wait for set blk cnt cmd failed!", __FUNCTION__ );
		return( MMC_FAILURE );
	}

	ext->setup_pio( hba, 512, dir );

	// multi block read/write. CMD18/CMD25
	if( mmc_sendcmd( hba, cmd, MMC_RSP_R1, flag, 0 ) != MMC_SUCCESS ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: send read/write cmd failed!", __FUNCTION__ );
		return( MMC_FAILURE );
	}

	if( mmc_wait_command( hba, MMC_TIME_DEFAULT ) != MMC_SUCCESS ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: wait for read/write cmd failed!", __FUNCTION__ );
		return( MMC_FAILURE );
	}

	ext->pio_done( hba, (char *)rpmb_frame, 512, dir );
	if( dir == MMC_DIR_OUT && mmc_wait_command( hba, MMC_TIME_DEFAULT ) != MMC_SUCCESS ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: wait for PIO done failed!", __FUNCTION__ );
		return( MMC_FAILURE );
	}

	return( MMC_SUCCESS );
}

int mmc_send_rwcmd(SIM_HBA *hba, int nbytes, int flag)
{
	SIM_MMC_EXT	*ext;
	uint32_t	cmd;

	ext = (SIM_MMC_EXT *)hba->ext;

	if (nbytes == ext->mmc_blksize) {
		cmd = ext->dir == MMC_DIR_IN ? MMC_READ_SINGLE_BLOCK : MMC_WRITE_BLOCK;
		ext->stop = MMCSD_STOP_NONE;
	}
	else {
		cmd = ext->dir == MMC_DIR_IN ? MMC_READ_MULTIPLE_BLOCK : MMC_WRITE_MULTIPLE_BLOCK;
		if (!(ext->hccap & MMC_HCCAP_ACMD12))
			ext->stop = MMCSD_STOP_PEND;
		else
			ext->stop = MMCSD_STOP_AUTO;
		flag |= MMC_CMD_DATA_MULTI;
	}

	if (ext->dir == MMC_DIR_IN)
		flag |= MMC_CMD_DATA_IN;

	// Data is expected, DON'T set MMC_CMD_INTR flag
	flag |= MMC_CMD_DATA;

#ifdef MMCSD_DEBUG
	mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: %s (%d), lba 0x%x, nblks 0x%x", __FUNCTION__, mmc_cmds[cmd], cmd, ( ext->eflags & MMC_EFLAG_HC ) ? ext->blkno : ( ext->blkno * ext->mmc_blksize ), nbytes / ext->mmc_blksize );
#endif

	if(ext->cmdlog){
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMCSD: cmd %d, lba 0x%x, nblks 0x%x", cmd, (ext->eflags & MMC_EFLAG_HC) ? ext->blkno : (ext->blkno * ext->mmc_blksize), nbytes/ext->mmc_blksize);
	}

	if (mmc_sendcmd(hba, cmd, MMC_RSP_R1, flag,	
		(ext->eflags & MMC_EFLAG_HC) ? ext->blkno : (ext->blkno * ext->mmc_blksize)) != MMC_SUCCESS)
		return 0;

	return nbytes;
}

int mmc_dump_ext_csd( SIM_HBA *hba )
{
	uint8_t			csd[EXT_CSD_SIZE];
	int				status;

	InterruptUnmask( hba->cfg.IRQRegisters[0], hba->iid );
	status = mmc_send_ext_csd( hba, csd );
	InterruptMask( hba->cfg.IRQRegisters[0], hba->iid );
	mmc_dump( (char *)csd, 512 );

	return( status );
}

static void mmc_disp_mmc_cid(uint8_t mmc_prot, mmc_cid_t *cid)
{
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMC CID info:");
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " MID (Manufacturer ID)       : %d", cid->mid);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " PNM (Product name)          : %s", cid->pnm);
	if (mmc_prot < 2) {
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " HWR (Hardware revision)     : %d", cid->hwr);
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " FWR (Hardware revision)     : %d", cid->fwr);
	} else
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " OID (OEM ID)                : %d", cid->oid);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " PSN (Product serial number) : %08x", cid->psn);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " MCD (Month code)            : %d", cid->mcd);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " YCD (Year code)             : %d", cid->ycd);
}

/*
 * CSD has to be already parsed
 */
void mmc_parse_mmc_cid(SIM_HBA *hba, mmc_cid_t *cid, uint32_t *resp)
{
	SIM_MMC_EXT				*ext;
	mmcsd_device_errata_t	*errata;

	ext = (SIM_MMC_EXT *)hba->ext;

	cid->pnm[0] = resp[3];
	cid->pnm[1] = resp[2] >> 24;
	cid->pnm[2] = resp[2] >> 16;
	cid->pnm[3] = resp[2] >> 8;
	cid->pnm[4] = resp[2];
	cid->pnm[5] = resp[1] >> 24;

	if (ext->csd.mmc_csd.mmc_prot < 2) {
		cid->mid    = resp[3] >> 8;
		cid->pnm[6] = resp[1] >> 16;
		cid->pnm[7] = 0;
		cid->hwr    = (resp[1] >> 12) & 0x0F;
		cid->fwr    = (resp[1] >> 8) & 0x0F;
	} else {
		cid->mid    = resp[3] >> 24;
		cid->oid    = (resp[3] >> 8) & 0xFF;
		cid->fwr    = (resp[1] >> 16) & 0xFF;
		cid->pnm[6] = 0;
	}

	cid->psn    = ((resp[1] & 0xFFFF) << 16) | (resp[0] >> 16);
	cid->mcd    = (resp[0] >> 12) & 0x0F;
	cid->ycd    = ((resp[0] >> 8) & 0x0F) + 1997;

	if( !strncmp( (char *)cid->pnm, "SEM", 3 ) ) {
			// Sandisk iNand: SEM02G, SEM04G, SEM08G, SEM16G,
		ext->devid = MMC_SANDISK_INAND;
	}

	if (hba->verbosity > 1)
		mmc_disp_mmc_cid(ext->csd.mmc_csd.mmc_prot, cid);

	if( ( errata = mmcsd_device_errata( hba, mmc_errata ) ) ) {
		mmcsd_reconcile_errata( hba, errata );
	}
}

static void mmc_disp_sd_cid(sd_cid_t *cid)
{
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "SD CID info:");
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " MID (Manufacturer ID)       : %d", cid->mid);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " OID (OEM/Application ID)    : %s", cid->oid);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " PNM (Product name)          : %s", cid->pnm);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " PRV (Product revision)      : %d.%d", cid->prv >> 4, cid->prv & 0x0F);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " PSN (Product serial number) : %08x", cid->psn);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " MDT (Manufacturing date)    : %d.%d", ((cid->mdt >> 4) & 0xFF) + 2000, cid->mdt & 0x0F);
}

void mmc_parse_sd_cid(SIM_HBA *hba, sd_cid_t *cid, uint32_t *resp)
{
	cid->mid    = resp[3] >> 24;
	cid->oid[0] = (resp[3] >> 8) & 0xFF;
	cid->oid[1] = (resp[3] >> 16) & 0xFFFF;
	cid->oid[2] = 0;
	cid->pnm[0] = resp[3];
	cid->pnm[1] = resp[2] >> 24;
	cid->pnm[2] = resp[2] >> 16;
	cid->pnm[3] = resp[2] >> 8;
	cid->pnm[4] = resp[2];
	cid->pnm[5] = 0;
	cid->prv    = resp[1] >> 24;
	cid->psn    = (resp[1] << 8) | (resp[0] >> 24);
	cid->mdt    = (resp[0] >> 8) & 0xFFFF;

	if (hba->verbosity > 1)
		mmc_disp_sd_cid(cid);
}

static void mmc_disp_mmc_csd(mmc_csd_t *csd)
{
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMC CSD info:");
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " CSD_STRUCTURE      : %x", csd->csd_structure);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " MMC_PROT           : %x", csd->mmc_prot);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " TAAC               : %x", csd->taac);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " NSAC               : %x", csd->nsac);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " TRAN_SPEED         : %x", csd->tran_speed);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " CCC                : %x", csd->ccc);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " READ_BL_LEN        : %x", csd->read_bl_len);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " READ_BL_PARTIAL    : %x", csd->read_bl_partial);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " WRITE_BLK_MISALIGN : %x", csd->write_blk_misalign);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " READ_BLK_MISALIGN  : %x", csd->read_blk_misalign);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " DSR_IMP            : %x", csd->dsr_imp);
	if (csd->csd_structure <= CSD_VERSION_11) {
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " SECTOR_SIZE        : %x", csd->erase.mmc_v22.sector_size);
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " ERASE_GRP_SIZE     : %x", csd->erase.mmc_v22.erase_grp_size);
	} else  {
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " ERASE_GRP_SIZE     : %x", csd->erase.mmc_v31.erase_grp_size);
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " ERASE_GRP_MULT     : %x", csd->erase.mmc_v31.erase_grp_mult);
	}
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " WP_GRP_SIZE        : %x", csd->wp_grp_size);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " R2W_FACTOR         : %x", csd->r2w_factor);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " WRITE_BL_LEN       : %x", csd->write_bl_len);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " WRITE_BL_PARTIAL   : %x", csd->write_bl_partial);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " COPY               : %x", csd->copy);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " PERM_WRITE_PROTECT : %x", csd->perm_write_protect);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " TMP_WRITE_PROTECT  : %x", csd->tmp_write_protect);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " ECC                : %x", csd->ecc);
}

void mmc_parse_mmc_csd(SIM_HBA *hba, mmc_csd_t *csd, uint32_t *resp)
{
	SIM_MMC_EXT		*ext;
	MMC_PARTITION	*part;
	uint32_t		csize, csizem, bsize;

	ext = (SIM_MMC_EXT *)hba->ext;

	part					= &ext->targets[0].partitions[0];
	csd->csd_structure      = resp[3] >> 30;
	csd->mmc_prot			= (resp[3] >> 26) & 0x0F;
	csd->taac               = resp[3] >> 16;
	csd->nsac               = resp[3] >> 8;
	csd->tran_speed         = resp[3];
	csd->ccc                = resp[2] >> 20;
	csd->read_bl_len        = (resp[2] >> 16) & 0x0F;
	csd->read_bl_partial    = (resp[2] >> 15) & 1;
	csd->write_blk_misalign = (resp[2] >> 14) & 1;
	csd->read_blk_misalign  = (resp[2] >> 13) & 1;
	csd->dsr_imp            = (resp[2] >> 12) & 1;
	csd->c_size				= ((resp[2] & 0x3FF) << 2) | (resp[1] >> 30);
	csd->vdd_r_curr_min		= (resp[1] >> 27) & 0x07;
	csd->vdd_r_curr_max		= (resp[1] >> 24) & 0x07;
	csd->vdd_w_curr_min		= (resp[1] >> 21) & 0x07;
	csd->vdd_w_curr_max		= (resp[1] >> 18) & 0x07;
	csd->c_size_mult		= (resp[1] >> 15) & 0x07;

	if (csd->csd_structure <= CSD_VERSION_11) {
		csd->erase.mmc_v22.sector_size		= (resp[1] >> 10) & 0x1F;
		csd->erase.mmc_v22.erase_grp_size	= (resp[1] >> 5) & 0x1F;
	} else {
		csd->erase.mmc_v31.erase_grp_size	= (resp[1] >> 10) & 0x1F;
		csd->erase.mmc_v31.erase_grp_mult	= (resp[1] >> 5) & 0x1F;
	}
	csd->wp_grp_size        = (resp[1] >> 0) & 0x1F;
	csd->wp_grp_enable      = (resp[0] >> 31);
	csd->r2w_factor         = (resp[0] >> 26) & 0x07;
	csd->write_bl_len       = (resp[0] >> 22) & 0x0F;
	csd->write_bl_partial   = (resp[0] >> 21) & 1;
	csd->copy               = (resp[0] >> 14) & 1;
	csd->perm_write_protect = (resp[0] >> 13) & 1;
	csd->tmp_write_protect  = (resp[0] >> 12) & 1;
	csd->ecc				= (resp[0] >>  8) & 3;

	if (csd->perm_write_protect || csd->tmp_write_protect) {
		part->pflags |= MMC_PFLAG_WP;
	}

	bsize  = 1 << csd->read_bl_len;
	csize  = csd->c_size + 1;
	csizem = 1 << (csd->c_size_mult + 2);

	// force to 512 byte block
	if (bsize > MMC_DFLT_BLKSIZE && (bsize % MMC_DFLT_BLKSIZE) == 0) {
		uint32_t ts = bsize / MMC_DFLT_BLKSIZE;
		csize = csize * ts;
		bsize = bsize / ts;
	}

	ext->mmc_blksize	= bsize;
	part->ptype			= MMC_PTYPE_USER;
	part->slba			= 0;
	part->nlba			= csize * csizem;
	part->elba			= part->nlba - 1;

	if (hba->verbosity > 1)
		mmc_disp_mmc_csd(csd);
}

static void mmc_disp_sd_csd(sd_csd_t *csd)
{
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "SD CSD info:");
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " CSD_STRUCTURE      : %x", csd->csd_structure);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " TAAC               : %x", csd->taac);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " NSAC               : %x", csd->nsac);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " TRAN_SPEED         : %x", csd->tran_speed);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " CCC                : %x", csd->ccc);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " READ_BL_LEN        : %x", csd->read_bl_len);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " READ_BL_PARTIAL    : %x", csd->read_bl_partial);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " WRITE_BLK_MISALIGN : %x", csd->write_blk_misalign);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " READ_BLK_MISALIGN  : %x", csd->read_blk_misalign);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " DSR_IMP            : %x", csd->dsr_imp);
	if (csd->csd_structure == 0) {
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " C_SIZE             : %x", csd->csd.csd_ver1.c_size);
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " VDD_R_CURR_MIN     : %x", csd->csd.csd_ver1.vdd_r_curr_min);
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " VDD_R_CURR_MAX     : %x", csd->csd.csd_ver1.vdd_r_curr_max);
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " VDD_W_CURR_MIN     : %x", csd->csd.csd_ver1.vdd_w_curr_min);
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " VDD_W_CURR_MAX     : %x", csd->csd.csd_ver1.vdd_w_curr_max);
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " C_SIZE_MULT        : %x", csd->csd.csd_ver1.c_size_mult);
	} else
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " C_SIZE             : %x", csd->csd.csd_ver2.c_size);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " ERASE_BLK_EN       : %x", csd->erase_blk_en);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " SECTOR_SIZE        : %x", csd->sector_size);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " WP_GRP_SIZE        : %x", csd->wp_grp_size);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " R2W_FACTOR         : %x", csd->r2w_factor);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " WRITE_BL_LEN       : %x", csd->write_bl_len);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " WRITE_BL_PARTIAL   : %x", csd->write_bl_partial);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " FILE_FORMAT_GRP    : %x", csd->file_format_grp);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " COPY               : %x", csd->copy);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " PERM_WRITE_PROTECT : %x", csd->perm_write_protect);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " TMP_WRITE_PROTECT  : %x", csd->tmp_write_protect);
	slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " FILE_FORMAT        : %x", csd->file_format);
}

void mmc_parse_sd_csd(SIM_HBA *hba, sd_csd_t *csd, uint32_t *resp)
{
	SIM_MMC_EXT		*ext;
	MMC_PARTITION	*part;
	uint32_t		csize, csizem, bsize;

	ext = (SIM_MMC_EXT *)hba->ext;

	part					= &ext->targets[0].partitions[0];
	csd->csd_structure      = resp[3] >> 30;
	csd->taac               = resp[3] >> 16;
	csd->nsac               = resp[3] >> 8;
	csd->tran_speed         = resp[3];
	csd->ccc                = resp[2] >> 20;
	csd->read_bl_len        = (resp[2] >> 16) & 0x0F;
	csd->read_bl_partial    = (resp[2] >> 15) & 1;
	csd->write_blk_misalign = (resp[2] >> 14) & 1;
	csd->read_blk_misalign  = (resp[2] >> 13) & 1;
	csd->dsr_imp            = (resp[2] >> 12) & 1;

	if (csd->csd_structure == 0) {
		csd->csd.csd_ver1.c_size = ((resp[2] & 0x3FF) << 2) | (resp[1] >> 30);
		csd->csd.csd_ver1.vdd_r_curr_min = (resp[1] >> 27) & 0x07;
		csd->csd.csd_ver1.vdd_r_curr_max = (resp[1] >> 24) & 0x07;
		csd->csd.csd_ver1.vdd_w_curr_min = (resp[1] >> 21) & 0x07;
		csd->csd.csd_ver1.vdd_w_curr_max = (resp[1] >> 18) & 0x07;
		csd->csd.csd_ver1.c_size_mult    = (resp[1] >> 15) & 0x07;
	} else {
		csd->csd.csd_ver2.c_size = ((resp[2] & 0x3F) << 16) | (resp[1] >> 16);
	}
	csd->erase_blk_en       = (resp[1] >> 14) & 1;
	csd->sector_size        = (resp[1] >> 7) & 0x7F;
	csd->wp_grp_size        = (resp[1] >> 0) & 0x7F;
	csd->wp_grp_enable      = (resp[0] >> 31);
	csd->r2w_factor         = (resp[0] >> 26) & 0x07;
	csd->write_bl_len       = (resp[0] >> 22) & 0x0F;
	csd->write_bl_partial   = (resp[0] >> 21) & 1;
	csd->file_format_grp    = (resp[0] >> 15) & 1;
	csd->copy               = (resp[0] >> 14) & 1;
	csd->perm_write_protect = (resp[0] >> 13) & 1;
	csd->tmp_write_protect  = (resp[0] >> 12) & 1;
	csd->file_format        = (resp[0] >> 10) & 3;

	if (csd->perm_write_protect || csd->tmp_write_protect) {
		part->pflags |= MMC_PFLAG_WP;
	}

	if (csd->csd_structure == 0) {
		bsize  = 1 << csd->read_bl_len;
		csize  = csd->csd.csd_ver1.c_size + 1;
		csizem = 1 << (csd->csd.csd_ver1.c_size_mult + 2);
	} else {
		bsize  = MMC_DFLT_BLKSIZE;
		csize  = csd->csd.csd_ver2.c_size + 1;
		csizem = 1024;
	}

	// force to 512 byte block
	if (bsize > MMC_DFLT_BLKSIZE && (bsize % MMC_DFLT_BLKSIZE) == 0) {
		uint32_t ts = bsize / MMC_DFLT_BLKSIZE;
		csize = csize * ts;
		bsize = bsize / ts;
	}

	part->ptype		= MMC_PTYPE_USER;
	part->slba		= 0;
	part->nlba		= csize * csizem;
	part->elba		= part->nlba - 1;

	if (hba->verbosity > 1)
		mmc_disp_sd_csd(csd);
}

static uint32_t mmcsd_tran_speed(uint8_t ts)
{
	return mmcsd_ts_exp[(ts & 0x7)] * mmcsd_ts_mul[(ts & 0x78) >> 3];
}

static int mmcsd_mmc_config_buswidth(SIM_HBA *hba)
{
	SIM_MMC_EXT		*ext;
	unsigned		ext_csd_bit;

	ext			= (SIM_MMC_EXT *)hba->ext;
	ext_csd_bit	= EXT_CSD_BUS_WIDTH_1;

	if( ext->eflags & MMC_EFLAG_FORCEBW ) {
		if( ext->bus_width == 8 )
			ext_csd_bit = EXT_CSD_BUS_WIDTH_8;
		else if( ext->bus_width == 4 )
			ext_csd_bit = EXT_CSD_BUS_WIDTH_4;
	}
	else if( ext->hccap & MMC_HCCAP_BW8 ) {
		ext_csd_bit = EXT_CSD_BUS_WIDTH_8;
		ext->bus_width = 8;
	}
	else if( ext->hccap & MMC_HCCAP_BW4 ) {
		ext_csd_bit = EXT_CSD_BUS_WIDTH_4;
		ext->bus_width = 4;
	}
	else {
		ext->bus_width = 1;
	}

	if(mmc_wait_card_ready(hba)!=MMC_SUCCESS)
		return (CAM_FAILURE);

	if( ext_csd_bit && ( ext->eflags & MMC_EFLAG_HS ) &&
			( ext->hccap & MMC_HCCAP_DDR50 ) && ( ext->dcap & MMC_DCAP_DDR ) ) {
		ext->eflags |= MMC_EFLAG_DDR;
		ext_csd_bit += 4;
	}

	ext->cfg_bus(hba, ext->bus_width, 0);
	
	ext->ident = MMCSD_IDENT_MMC_SWITCH_MODE_BUSWIDTH;

	return mmc_sendcmd(hba, SD_SET_BUS_WIDTH, MMC_RSP_R1B, MMC_CMD_INTR, (0x3 << 24) |
	  (EXT_CSD_BUS_WIDTH << 16) | (ext_csd_bit << 8) | 0x1);
}

static int mmc_ident_sd_clock(SIM_HBA *hba)
{
	SIM_MMC_EXT		*ext = (SIM_MMC_EXT *)hba->ext;
	int				clock, hs = 0;

	if ( ext->eflags & MMC_EFLAG_HS ) {
		clock = 50000000;
		hs = 1;
	}else{
		clock = mmcsd_tran_speed(ext->csd.sd_csd.tran_speed);
	}

	if ( (ext->eflags & MMC_EFLAG_FORCECLK) && ext->nclock < clock ){
		clock = ext->nclock;
		if( clock <= 25000000 ){
			hs = 0;
		}
	}

	if (hba->verbosity)
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "SD : config clock as %d", clock);
	ext->set_clock(hba, &clock, hs);
	ext->speed = clock;

	if (hba->verbosity)
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "SD : actual operation clock %d", clock);

	ext->set_blksz(hba, MMC_DFLT_BLKSIZE);	// block size, 512, every card should support

	ext->ident = MMCSD_IDENT_FINAL;

	return mmc_sendcmd(hba, MMC_SET_BLOCKLEN, MMC_RSP_R1, MMC_CMD_INTR, MMC_DFLT_BLKSIZE);
}

static int mmc_ident_mmc_clock(SIM_HBA *hba)
{
	SIM_MMC_EXT		*ext = (SIM_MMC_EXT *)hba->ext;
	int				clock;

	clock = mmcsd_tran_speed(ext->csd.mmc_csd.tran_speed);
	if (ext->eflags & MMC_EFLAG_FORCECLK){
		clock = (ext->nclock < clock)? ext->nclock : clock;    // MMC operation clock
	}
	
	if (hba->verbosity)
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO,
			"MMC config: clock %d, bus width %d, block size %d", clock, 1, MMC_DFLT_BLKSIZE);
	
	ext->cfg_bus(hba, 1, 1);		// 1 bit bus, MMC card
	ext->set_blksz(hba, MMC_DFLT_BLKSIZE);	// block size, 512, every card should support
	ext->set_clock(hba, &clock, 0); // no high speed support
	ext->speed = clock;
	
	if (hba->verbosity)
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMC : actual operation clock %d", clock);
	
	ext->ident = MMCSD_IDENT_FINAL;
	
	/*
	 * Set bus width?
	 * We need MMC specification
	 */
	return mmc_sendcmd(hba, MMC_SET_BLOCKLEN, MMC_RSP_R1, MMC_CMD_INTR, MMC_DFLT_BLKSIZE);
}

static int mmc_ident_switch_buswidth(SIM_HBA *hba, int intr)
{
	SIM_MMC_EXT		*ext = (SIM_MMC_EXT *)hba->ext;

	if (intr & MMC_INTR_ERROR){
		ext->eflags &= ~MMC_EFLAG_DDR;
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMC : Switch to bus width %d bits failed", ext->bus_width);
		return (CAM_FAILURE);
	}

	if (hba->verbosity)
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMC : Switch to bus width %dbit %ssuccessful", ext->bus_width,
				( ext->eflags & MMC_EFLAG_DDR ) ? "(DDR) " : "" );

	ext->set_blksz(hba, MMC_DFLT_BLKSIZE);	// block size, 512, every card should support

	if( ext->eflags & MMC_EFLAG_DDR ) {
			// MMC_SET_BLOCKLEN is invalid when we are in DDR mode.
		ext->ident = MMCSD_IDENT_NONE;
		return (CAM_SUCCESS);
	}

	ext->ident = MMCSD_IDENT_FINAL;
	
	if(mmc_wait_card_ready(hba)!=MMC_SUCCESS)
		return (CAM_FAILURE);

	return mmc_sendcmd(hba, MMC_SET_BLOCKLEN, MMC_RSP_R1, MMC_CMD_INTR, MMC_DFLT_BLKSIZE);
}

static int mmc_ident_switch_mode_hs(SIM_HBA *hba, int intr)
{
	SIM_MMC_EXT		*ext;
	int				clock, hs = 1;
	ext = (SIM_MMC_EXT *)hba->ext;

	if (intr & MMC_INTR_ERROR){
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMC : Switch to hih speed mode failed");
		return (CAM_FAILURE);
	}

	/*we are in HS now*/
	ext->eflags |= MMC_EFLAG_HS;

	clock = ext->csd.mmc_csd.ext_csd.hs_max_dtr;
	if ( (ext->eflags & MMC_EFLAG_FORCECLK) && ext->nclock < clock ){
		clock = ext->nclock;
		if(clock <=25000000){
			hs = 0;
		}
	}

	if (hba->verbosity)
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMC : config clock as %d", clock);

	ext->set_clock(hba, &clock, hs);
	ext->speed = clock;
	
	if (hba->verbosity)
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMC : actual operation clock %d", clock);

	/*
	 * Activate wide bus (if supported).
	 */
	if ((ext->csd.mmc_csd.mmc_prot >=CSD_VERSION_14) &&
	    (ext->hccap & (MMC_HCCAP_BW8 | MMC_HCCAP_BW4))) {
		return mmcsd_mmc_config_buswidth(hba);
	}else{
		return mmc_ident_mmc_clock(hba);
	}
	return (CAM_FAILURE);
}

#if 0
static void mmc_partition_config_rsvd( SIM_HBA *hba )
{
	SIM_MMC_EXT		*ext;
	MMC_PARTITION	*part;
	MMC_PARTITION	*rpart;
	int				i;
	int				nluns;

	ext		= (SIM_MMC_EXT *)hba->ext;
	part	= ext->partitions;

	for( i = 0, nluns = ext->nluns + 1; i < nluns && i < MAX_LUN; part++, i++ ) {
		if( part->pflags & MMC_PFLAG_PART_RSV ) {
			if( part->pflags & MMC_PFLAG_PART_RSV_REMAP ) {
				rpart			= &ext->partitions[++ext->nluns];
				rpart->ptype	= part->ptype;
				rpart->pflags	= part->pflags;
				rpart->config	= part->config;
				rpart->nlba		= part->slba;
				rpart->slba		= 0;
				rpart->elba		= rpart->nlba - 1;
			}
			part->nlba	-= part->slba;
			part->elba	-= part->slba;
		}
	}
}
#endif

#ifndef BS_PARTITION_CONFIG
int mmc_partition_config( SIM_HBA *hba, uint8_t *csd )
{
	SIM_MMC_EXT		*ext;
	MMC_PARTITION	*part;
	int				pidx;
	int				pconf;
	uint32_t		nlba;
	uint64_t		gp_size;
	uint64_t		enh_size;
	uint64_t		wp_grp_size;
	
	ext				= (SIM_MMC_EXT *)hba->ext;
	pconf			= csd[EXT_CSD_PART_CONFIG];
	ext->pactive	= pconf & EXT_CSD_PC_ACCESS_MSK;
	pconf			&= ~EXT_CSD_PC_ACCESS_MSK;
	part			= &ext->targets[0].partitions[0];
	part->config	= pconf;

	wp_grp_size = (uint64_t )csd[EXT_CSD_HC_WP_GRP_SIZE ] *
				(uint64_t )csd[EXT_CSD_ERASE_GRP_SIZE ] * MMC_ERASE_GRP_512K;

		// verify partitioning support/validity
#if 1
	if( !( csd[EXT_CSD_PARTITIONING_SUP] & EXT_CSD_PS_EN ) || 
			!( ext->eflags & MMC_EFLAG_PARTITIONS ) ) {
		return( MMC_FAILURE );
	}
#else
	if( !( csd[EXT_CSD_PARTITIONING_SUP] & EXT_CSD_PS_EN ) ||
			!( csd[EXT_CSD_PARTITION_SETTING] & EXT_CSD_PS_CMP ) ) {
		return( MMC_FAILURE );
	}
#endif

	enh_size = (uint64_t)csd[EXT_CSD_MAX_ENH_SIZE_MULT + 0];
	enh_size |= (uint64_t)csd[EXT_CSD_MAX_ENH_SIZE_MULT + 1] << 8;
	enh_size |= (uint64_t)csd[EXT_CSD_MAX_ENH_SIZE_MULT + 2] << 16;
	mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: HC_WP_GRP_SZ %d, ERASE_GRP_SZ %d, ENH_SZ_MULT %d, SWITCH_TIME %dms, pw %x, pr %x", __FUNCTION__, csd[EXT_CSD_HC_WP_GRP_SIZE], csd[EXT_CSD_ERASE_GRP_SIZE], enh_size, csd[EXT_CSD_SWITCH_TIME] * 10, csd[235], csd[234] );

	part++;

		// boot area partition 1
	part->ptype		= MMC_PTYPE_BOOT;
	part->config	= pconf | 1;
	part->slba		= 0;
	part->nlba		= (uint64_t)csd[EXT_CSD_BOOT_SIZE_MULT] * MMC_BOOT_SIZE_MULT / 512;
	part->elba		= part->nlba - 1;
	part++; ext->ntargs++;

		// boot area partition 2
	part->ptype		= MMC_PTYPE_BOOT;
	part->config	= pconf | 2;
	part->slba		= 0;
	part->nlba		= (uint64_t)csd[EXT_CSD_BOOT_SIZE_MULT] * MMC_BOOT_SIZE_MULT / 512;
	part->elba		= part->nlba - 1;
	part++; ext->ntargs++;

		// RPMB area
	part->ptype	= MMC_PTYPE_RPMB;
	part->config	= pconf | 3;
	part->slba	= 0;
	part->nlba	= (uint64_t)csd[EXT_CSD_RPMB_SIZE_MULT] * MMC_RPMB_SIZE_MULT / 512;
	part->elba	= part->slba + part->nlba - 1;
	part++; ext->ntargs++;

	for( pidx = 0; pidx < MMC_GP_PART_MAX; pidx++, part++ ) {
		gp_size = (uint64_t)csd[EXT_CSD_GP_SIZE + 0 + ( pidx * 3 ) ];
		gp_size |= (uint64_t)csd[EXT_CSD_GP_SIZE + 1 + ( pidx * 3 ) ] << 8;
		gp_size |= (uint64_t)csd[EXT_CSD_GP_SIZE + 2 + ( pidx * 3 ) ] << 16;
		nlba = ( gp_size * wp_grp_size ) / 512;

		ext->ntargs++;
		part->ptype		= MMC_PTYPE_GP;
		part->config	= pconf | ( 4 + pidx );
		part->slba		= 0;
		part->nlba		= nlba;
		part->elba		= part->nlba - 1;
	}


	return( MMC_SUCCESS );
}
#endif

#if 0
int mmc_create_partitions( SIM_HBA *hba, uint8_t *csd )
{
	SIM_MMC_EXT		*ext;
	int				status;
	uint64_t		part_size;
	uint64_t		wp_grp_size;

	ext		= (SIM_MMC_EXT *)hba->ext;
		
		// verify partitioning support/validity
	if( !( csd[EXT_CSD_PARTITIONING_SUP] & EXT_CSD_PS_EN ) ||
			( csd[EXT_CSD_PARTITION_SETTING] & EXT_CSD_PS_CMP ) ) {
		return( MMC_SUCCESS );
	}

	InterruptUnmask( hba->cfg.IRQRegisters[0], hba->iid );

		// Enable Write Reliability
	if( ( csd[EXT_CSD_WR_REL_PARAM] & EXT_CSD_WRP_WR_DATA_REL ) ) {
		if( mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_WR_REL_SET, EXT_CSD_WRS_USR | EXT_CSD_WRS_1 | EXT_CSD_WRS_2 | EXT_CSD_WRS_3 ) == MMC_FAILURE ) {
			mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_wr_rel_param", __FUNCTION__ );
		}
	}

	wp_grp_size = (uint64_t )csd[EXT_CSD_HC_WP_GRP_SIZE ] *
				(uint64_t )csd[EXT_CSD_ERASE_GRP_SIZE ] * MMC_ERASE_GRP_512K;

		// set partition 1 to 32MB/64MB (depends on wp_grp_size)
	part_size = max( 0x2000000 / wp_grp_size, 1 );
	if( ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_GP1_SIZE + 0, part_size & 0xff ) ) ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_gp1_size 0", __FUNCTION__ );
	}

	if( !status && ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_GP1_SIZE + 1, ( part_size >> 8 ) & 0xff ) ) ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_gp1_size 1", __FUNCTION__ );
	}

	if( !status && ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_GP1_SIZE + 2, ( part_size >> 16 ) & 0xff ) ) ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_gp1_size 2", __FUNCTION__ );
	}

		// set partition 2 to 800MB
	part_size = max( 0x30000000 / wp_grp_size, 1 );
	if( !status && ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_GP2_SIZE + 0, part_size & 0xff ) ) ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_gp2_size 0", __FUNCTION__ );
	}

	if( !status && ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_GP2_SIZE + 1, ( part_size >> 8 ) & 0xff ) ) ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_gp2_size 1", __FUNCTION__ );
	}

	if( !status && ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_GP2_SIZE + 2, ( part_size >> 16 ) & 0xff ) ) ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_gp2_size 2", __FUNCTION__ );
	}

		// set partition 3 to 512MB
	part_size = max( 0x20000000 / wp_grp_size, 1 );
	if( !status && ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_GP3_SIZE + 0, part_size & 0xff ) ) ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_gp3_size 0", __FUNCTION__ );
	}

	if( !status && ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_GP3_SIZE + 1, ( part_size >> 8 ) & 0xff ) ) ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_gp3_size 1", __FUNCTION__ );
	}

	if( !status && ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_GP3_SIZE + 2, ( part_size >> 16 ) & 0xff ) ) ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_gp3_size 2", __FUNCTION__ );
	}

#if 0
		// set enhanced start address
	enh_addr = 0;
	if( !status && ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_ENH_START_ADDR + 0, enh_addr & 0xff ) ) ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_enh_start_addr 0", __FUNCTION__ );
	}

	if( !status && ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_ENH_START_ADDR + 1, ( enh_addr >> 8 ) & 0xff ) ) ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_enh_start_addr 1", __FUNCTION__ );
	}

	if( !status && ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_ENH_START_ADDR + 2, ( enh_addr >> 16 ) & 0xff ) ) ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_enh_start_addr 2", __FUNCTION__ );
	}

	if( !status && ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_ENH_START_ADDR + 3, ( enh_addr >> 24 ) & 0xff ) ) ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_enh_start_addr 3", __FUNCTION__ );
	}

		// set enhanced size 2MB to 64MB (1 * wp_grp_siz) 
	part_size = 1;
	if( !status && ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_ENH_SIZE_MULT + 0, part_size & 0xff ) ) ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_enh_size_mult 0", __FUNCTION__ );
	}

	if( !status && ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_ENH_SIZE_MULT + 1, ( part_size >> 8 ) & 0xff ) ) ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_enh_size_mult 1", __FUNCTION__ );
	}

	if( !status && ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_ENH_SIZE_MULT + 2, ( part_size >> 16 ) & 0xff ) ) ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_enh_size_mult 2", __FUNCTION__ );
	}

		// set enhanced attributes on USER, GP 1/2/3
	if( !status && ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_PARTITIONS_ATTR, EXT_CSD_PA_ENH_USR | EXT_CSD_PA_ENH_1 | EXT_CSD_PA_ENH_2 | EXT_CSD_PA_ENH_3 ) ) ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_partitions_attr", __FUNCTION__ );
	}
#else
		// set enhanced attributes on GP 1/2/3
	if( !status && ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_PARTITIONS_ATTR, EXT_CSD_PA_ENH_1 | EXT_CSD_PA_ENH_2 | EXT_CSD_PA_ENH_3 ) ) ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_partitions_attr", __FUNCTION__ );
	}
#endif

		// mark as complete
	if( !status && ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_PARTITION_SETTING, EXT_CSD_PS_CMP ) ) ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_partition_setting", __FUNCTION__ );
	}

	InterruptMask( hba->cfg.IRQRegisters[0], hba->iid );
	exit( 1 );
	return( MMC_SUCCESS );
}
#endif

int mmc_erase_grp_config( SIM_HBA *hba, int hc )
{
	SIM_MMC_EXT		*ext;
	int				status;

	ext	= (SIM_MMC_EXT *)hba->ext;

	InterruptUnmask( hba->cfg.IRQRegisters[0], hba->iid );
	if( ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_ERASE_GRP_DEF, hc ? EXT_CSD_EGD_EN : 0 ) ) == MMC_SUCCESS ) {
		ext->extended_csd[EXT_CSD_ERASE_GRP_DEF] = 1;
		if(mmc_wait_card_ready(hba)!=MMC_SUCCESS)
			return (CAM_FAILURE);
	}
	else {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_erase_grp_def", __FUNCTION__ );
	}

	InterruptMask( hba->cfg.IRQRegisters[0], hba->iid );

	return( status );
}

// WARNING:  BKOPS enable is permanent (not in spec).
// 			 Firmware will not do them even if
// 		     required to continue functioning.
#ifdef MMC_BKOPS_SUP
int mmc_bkops_config( SIM_HBA *hba )
{
	SIM_MMC_EXT		*ext;
	int				status;

	ext	= (SIM_MMC_EXT *)hba->ext;

	if( !( ext->dcap & MMC_DCAP_BKOPS ) || !( ext->eflags & MMC_EFLAG_BKOPS ) ) {
		return( MMC_SUCCESS );
	}

	InterruptUnmask( hba->cfg.IRQRegisters[0], hba->iid );
	if( ( status = mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_BKOPS_EN, EXT_CSD_BKOPS_ENABLE ) ) ) {
		ext->eflags &= ~MMC_EFLAG_BKOPS;
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: switch ext_csd_bkops_en", __FUNCTION__ );
	}
	InterruptMask( hba->cfg.IRQRegisters[0], hba->iid );

	return( status );
}
#endif

/*
 * Read and decode extended CSD.
 */
static int mmc_ident_read_ext_csd(SIM_HBA *hba, int intr)
{
	SIM_MMC_EXT		*ext;
	mmc_ext_csd_t	*ext_csd;
	uint8_t 		*raw_ext_csd;
	int 			ext_csd_struct;
	int				clock;

	ext			= (SIM_MMC_EXT *)hba->ext;
	ext_csd		= &ext->csd.mmc_csd.ext_csd;
	raw_ext_csd	= ext->extended_csd;

	if (intr & MMC_INTR_ERROR){
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMC :SEND_EXT_CSD command failed");
		return (CAM_FAILURE);
	}

	if (ext->pio_done(hba, (char *)raw_ext_csd, EXT_CSD_SIZE, MMC_DIR_IN) != EXT_CSD_SIZE){
			slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMC : pio read EXT_CSD failed");
		return (CAM_FAILURE);
	}

	if( hba->verbosity > 3 ) {
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: EXT_CDD", __FUNCTION__ );
		mmc_dump( (char *)raw_ext_csd, EXT_CSD_SIZE );
	}

	ext_csd_struct = raw_ext_csd[EXT_CSD_REV];
	if (ext_csd_struct >= 2) {
		if( raw_ext_csd[EXT_CSD_SEC_FEATURE_SUPPORT] & EXT_CSD_SEC_GB_CL_EN ) {
			ext->dcap |= MMC_DCAP_TRIM;
		}

		if( raw_ext_csd[EXT_CSD_BKOPS_SUPPORTED] & EXT_CSD_BKOPS_SUP ) {
			ext->dcap |= MMC_DCAP_BKOPS;
			if( raw_ext_csd[EXT_CSD_BKOPS_EN] & EXT_CSD_BKOPS_ENABLE ) {
				// BKOPS enable is permanent (not in spec).
				// Note:  Firmware will not do them even if
				//        required to continue functioning,
				//        so we enable.
				ext->eflags |= MMC_EFLAG_BKOPS;
			}
		}

		ext_csd->sectors =
			raw_ext_csd[EXT_CSD_SEC_CNT + 0] << 0 |
			raw_ext_csd[EXT_CSD_SEC_CNT + 1] << 8 |
			raw_ext_csd[EXT_CSD_SEC_CNT + 2] << 16 |
			raw_ext_csd[EXT_CSD_SEC_CNT + 3] << 24;
	}

	// Needed for write protection
	ext->csd.mmc_csd.ext_csd.erase_grp_def = raw_ext_csd[EXT_CSD_ERASE_GRP_DEF];
	ext->csd.mmc_csd.ext_csd.hc_erase_group_size = raw_ext_csd[EXT_CSD_ERASE_GRP_SIZE];
	ext->csd.mmc_csd.ext_csd.hc_wp_grp_size = raw_ext_csd[EXT_CSD_HC_WP_GRP_SIZE];
	ext->csd.mmc_csd.ext_csd.user_wp = raw_ext_csd[EXT_CSD_USER_WP];

	if( hba->verbosity ) {
        slogf( _SLOGC_SIM_MMC, _SLOG_INFO, "MMC card : EXT_CSD structure: " );
        slogf( _SLOGC_SIM_MMC, _SLOG_INFO, " EXT_CSD_REV: %x", ext_csd_struct );
        slogf( _SLOGC_SIM_MMC, _SLOG_INFO, " Bus Width: %x", raw_ext_csd[EXT_CSD_BUS_WIDTH] );
        slogf( _SLOGC_SIM_MMC, _SLOG_INFO, " Card Type: %x", raw_ext_csd[EXT_CSD_CARD_TYPE] );
        slogf( _SLOGC_SIM_MMC, _SLOG_INFO, " SECTORS: %d", ext_csd->sectors );
    }

	switch( raw_ext_csd[EXT_CSD_CARD_TYPE] ) {
		case 0x57:
		case EXT_CSD_CARD_TYPE_200 | EXT_CSD_CARD_TYPE_DDR_52 | EXT_CSD_CARD_TYPE_52 | EXT_CSD_CARD_TYPE_26:
		case EXT_CSD_CARD_TYPE_DDR_52_LV | EXT_CSD_CARD_TYPE_DDR_52 | EXT_CSD_CARD_TYPE_52 | EXT_CSD_CARD_TYPE_26:
		case EXT_CSD_CARD_TYPE_DDR_52 | EXT_CSD_CARD_TYPE_52 | EXT_CSD_CARD_TYPE_26:
			ext->dcap |= MMC_DCAP_DDR;
		case EXT_CSD_CARD_TYPE_52 | EXT_CSD_CARD_TYPE_26:
			ext_csd->hs_max_dtr = 52000000;
			break;
		case EXT_CSD_CARD_TYPE_26:
			ext_csd->hs_max_dtr = 26000000;
			break;
		default:
			/* MMC v4 spec says this cannot happen */
			slogf( _SLOGC_SIM_MMC, _SLOG_INFO, "%s: card is mmc v4 but doesn't "
				"support any high-speed modes.\n", __func__ );
			return (CAM_FAILURE);
	}

		// update capacity for HC
	if( ( ext->csd.mmc_csd.mmc_prot >= CSD_VERSION_14 ) && ext_csd->sectors ) {
		ext->targets[0].partitions[0].nlba = ext_csd->sectors;
		ext->targets[0].partitions[0].elba = ext_csd->sectors - 1;
	}

	mmc_erase_grp_config( hba, raw_ext_csd[EXT_CSD_ERASE_GRP_SIZE] );

		// Needed for write protection
	ext_csd->erase_grp_def			= raw_ext_csd[EXT_CSD_ERASE_GRP_DEF];
	ext_csd->hc_erase_group_size	= raw_ext_csd[EXT_CSD_ERASE_GRP_SIZE];
	ext_csd->hc_wp_grp_size			= raw_ext_csd[EXT_CSD_HC_WP_GRP_SIZE];
	ext_csd->user_wp				= raw_ext_csd[EXT_CSD_USER_WP];

#ifdef MMC_BKOPS_SUP
	mmc_bkops_config( hba );
#endif
	mmc_partition_config( hba, raw_ext_csd );
//	mmc_partition_config_rsvd( hba );
//	mmc_create_partitions( hba, raw_ext_csd );

		// Activate high speed (if supported)
	if( ( ext_csd->hs_max_dtr != 0 ) && ( ext->hccap & MMC_HCCAP_HS ) ) {
		if( hba->verbosity )
			slogf( _SLOGC_SIM_MMC, _SLOG_INFO, "MMC: Switch to high speed mode." );

		ext->ident = MMCSD_IDENT_MMC_SWITCH_MODE_HS;

		return( mmc_sendcmd(hba, SD_SET_BUS_WIDTH, MMC_RSP_R1B, MMC_CMD_INTR,
				(0x3 << 24) | ( EXT_CSD_HS_TIMING << 16 ) | ( 1 << 8) | 0x1 ) );
		/* Activate wide bus (if supported). */
	}
	else if( ( ext->csd.mmc_csd.mmc_prot >= CSD_VERSION_14 ) &&
			( ext->hccap & ( MMC_HCCAP_BW8 | MMC_HCCAP_BW4 ) ) ) {

		clock = mmcsd_tran_speed(ext->csd.mmc_csd.tran_speed);
		if ( (ext->eflags & MMC_EFLAG_FORCECLK) && ext->nclock < clock ){
			clock = ext->nclock;		// MMC operation clock
		}
		ext->set_clock( hba, &clock, 0 ); // no high speed support
		ext->speed = clock;
		if( hba->verbosity )
			slogf( _SLOGC_SIM_MMC, _SLOG_INFO, "MMC : actual operation clock %d", clock );

		return( mmcsd_mmc_config_buswidth( hba ) );
	}
	else {
		return( mmc_ident_mmc_clock( hba ) );
	}

	return( CAM_FAILURE );
}

static int mmc_ident_status_done(SIM_HBA *hba, int intr)
{
	SIM_MMC_EXT		*ext;
	int				bus_width;

	ext = (SIM_MMC_EXT *)hba->ext;

	if (intr & MMC_INTR_ERROR)
		return (CAM_FAILURE);

	if (ext->version >= MMC_VERSION_1) {	// MMC card
		if(ext->csd.mmc_csd.mmc_prot >=CSD_VERSION_14){
			/* MMC V4 need to read ext_csd*/
			ext->cfg_bus(hba, 1, 1);		
			ext->set_blksz(hba, EXT_CSD_SIZE);	
			if (ext->setup_pio(hba, EXT_CSD_SIZE, MMC_DIR_IN) != EXT_CSD_SIZE)
				return (CAM_FAILURE);
			ext->ident = MMCSD_IDENT_EXT_CSD;

			return mmc_sendcmd(hba, MMC_SEND_EXT_CSD, MMC_RSP_R1, MMC_CMD_DATA_IN | MMC_CMD_DATA, 0);
		}else{
			return mmc_ident_mmc_clock(hba);
		}
	}
 
	/*
	 * The host controller should support at least 1 of them
	 */
	if (ext->eflags & MMC_EFLAG_FORCEBW)
		bus_width = ext->bus_width;
	else if (ext->hccap & MMC_HCCAP_BW4)
		bus_width = 4;		// 4 bit bus
	else
		bus_width = 1;		// 1 bit bus

	ext->cfg_bus(hba, bus_width, 0);

	if (bus_width == 4) {
		ext->ident = MMCSD_IDENT_SET_BW_APP;

		return mmc_sendcmd(hba, MMC_APP_CMD, MMC_RSP_R1, MMC_CMD_INTR, ext->rca << 16);
	}
	// Read SCR
	ext->set_blksz(hba, 8);
	ext->ident = MMCSD_IDENT_SEND_SCR;

	return mmc_sendcmd(hba, MMC_APP_CMD, MMC_RSP_R1, MMC_CMD_INTR, ext->rca << 16);
}

static int mmc_ident_start(SIM_HBA *hba)
{
	SIM_MMC_EXT		*ext;
	static int		count = 0;

	ext = (SIM_MMC_EXT *)hba->ext;

	if (count == 0) {
		ext->targets[0].partitions[0].pflags &= ~MMC_PFLAG_WP;
		atomic_clr(&ext->eflags, MMC_EFLAG_PPL | MMC_EFLAG_HS | MMC_EFLAG_HC | MMC_EFLAG_DDR);
		memset( ext->extended_csd, 0, sizeof( ext->extended_csd ) );
        memset(&ext->cid, 0, max(sizeof(sd_cid_t), sizeof(mmc_cid_t)));
        memset(&ext->csd, 0, max(sizeof(sd_csd_t), sizeof(mmc_csd_t)));
	}

	if (ext->detect(hba) != MMC_SUCCESS)
		return -1;

	ext->powerdown(hba);

	if (ext->pwr_delay)
		delay(ext->pwr_delay);

	if (ext->powerup(hba) != MMC_SUCCESS)
		return (CAM_FAILURE);

	if (ext->pwr_delay)
		delay(ext->pwr_delay);

	if (++count > ext->pwr_retry) {
		ext->ident = MMCSD_IDENT_GO_IDLE;
		count = 0;
	}

	return mmc_sendcmd(hba, MMC_GO_IDLE_STATE, MMC_RSP_NONE, MMC_CMD_INIT | MMC_CMD_INTR, 0);
}

static int mmc_ident_mmc_card_powerup(SIM_HBA *hba, int intr)
{
	SIM_MMC_EXT		*ext;
	int				ocr=0;

	ext = (SIM_MMC_EXT *)hba->ext;

	if(hba->verbosity>1) {
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMC card powerup: istage %d intr 0x%x %s resp01 0x%x",
		       ext->istage, intr, (intr&MMC_INTR_ERROR)?"ERR":"OK ", ext->mmc_resp[0]);
	}

	// 3 seconds should be enough
	if (ext->pwr_delay*ext->istage > 3000 || ext->istage>3000){
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMC card : powerup timeout");
		return (MMC_FAILURE);
	}

	ext->istage++;

	if (ext->pwr_delay)
		delay(ext->pwr_delay);

//	if (ext->istage > 1) {
	if (ext->istage > 2) {
		if (intr & MMC_INTR_ERROR)
			return (MMC_FAILURE);

		/* High capacity card ? */
		if (ext->mmc_resp[0] & 0x40000000) {
			atomic_set(&ext->eflags, MMC_EFLAG_HC);
		}

		/* Power up ? */
		if (ext->mmc_resp[0] & 0x80000000) {
			ext->ident = MMCSD_IDENT_ALL_SEND_CID;	// card is powered up, go next
			ext->istage = 0;
			ext->version = MMC_VERSION_1;
			if (hba->verbosity)
				slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMC card detected");
			if ((ext->eflags & MMC_EFLAG_HC) && hba->verbosity)
				slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMC card : high capacity");

			return mmc_sendcmd(hba, MMC_ALL_SEND_CID, MMC_RSP_R2, MMC_CMD_INTR, 0);
		}
	}

	//tell the card that we support High Capacity
	if(ext->istage > 1){
		ocr = 0x40000000;
		if(ext->hccap & MMC_HCCAP_18V)
			ocr |= 0x80;
		if(ext->hccap & (MMC_HCCAP_30V | MMC_HCCAP_33V))
			ocr |= 0x00FF8000; // high voltage support
	}
	else ocr = 0;

	return mmc_sendcmd(hba, MMC_SEND_OP_COND, MMC_RSP_R3, MMC_CMD_INTR, ocr);
}

static int mmc_ident_sd_card_powerup(SIM_HBA *hba, int intr)
{
	SIM_MMC_EXT		*ext;
	int				ocr = 0;

	ext = (SIM_MMC_EXT *)hba->ext;

	if(hba->verbosity>1) {
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "SD card : powerup");
	}

	if (ext->istage > 1000) {
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMC card power up failed");
		return (MMC_FAILURE);
	}

	if ((ext->istage % 3) == 0) {
		delay(5);
		ext->istage++;

		if (ext->istage == 1)
			return mmc_sendcmd(hba, MMC_APP_CMD, MMC_RSP_R1, MMC_CMD_INTR, 0);
	}

	if ((ext->hccap & (MMC_HCCAP_18V | MMC_HCCAP_30V | MMC_HCCAP_33V)) == (MMC_HCCAP_18V | MMC_HCCAP_30V | MMC_HCCAP_33V))
		ocr = 0x00ff8000;
	else {
		if (ext->hccap & MMC_HCCAP_18V)
			ocr |= 1 << 24;
		else if (ext->hccap & MMC_HCCAP_30V)
			ocr |= 1 << 18;
		else
			ocr |= 1 << 21;
	}

	if (ext->istage == 1) {
		if (ext->version == SD_VERSION_1 && (intr & MMC_ERR_CMD_TO)) {
			// timeout, might be a MMC card
			ext->ident = MMCSD_IDENT_MMC_POWERUP;
			ext->istage = 0;
			ext->eflags &= ~MMC_EFLAG_PPL;
#if 1
			return mmc_sendcmd(hba, MMC_GO_IDLE_STATE, MMC_RSP_NONE, MMC_CMD_INIT | MMC_CMD_INTR, 0);
#else
			return mmc_ident_mmc_card_powerup(hba, intr);
#endif
		} else if (intr & MMC_INTR_ERROR) {
			return (MMC_FAILURE);
		}
		ocr = 0;
	}

	switch (ext->istage % 3) {
		case 1:		// wait for MMC_APP_CMD complete
			if (intr & MMC_INTR_ERROR)
				return (MMC_FAILURE);
			if (ext->version == SD_VERSION_2)
				ocr |= 1 << 30;
			ext->istage++;
			return mmc_sendcmd(hba, SD_SEND_OP_COND, MMC_RSP_R3, MMC_CMD_INTR, ocr);

		case 2:		// wait for SD_SEND_OP_COND complete
			if (intr & MMC_INTR_ERROR)
				return (MMC_FAILURE);

			/* High capacity card ? */
			if (ext->mmc_resp[0] & 0x40000000) {
				atomic_set(&ext->eflags, MMC_EFLAG_HC);
			}

			/* Power up ? */
			if (ext->mmc_resp[0] & 0x80000000) {
				ext->ident = MMCSD_IDENT_ALL_SEND_CID;	// card is powered up, go next
				ext->istage = 0;

				if ((ext->eflags & MMC_EFLAG_HC) && hba->verbosity)
					slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "SD card : high capacity");

				if (hba->verbosity)
					slogf(_SLOGC_SIM_MMC, _SLOG_INFO,
						"SD card version = %d", ext->version);


				return mmc_sendcmd(hba, MMC_ALL_SEND_CID, MMC_RSP_R2, MMC_CMD_INTR, 0);
			} else
				ext->istage++;

			return mmc_sendcmd(hba, MMC_APP_CMD, MMC_RSP_R1, MMC_CMD_INTR, 0);
	}

	return (MMC_FAILURE);
}

static int mmc_ident_go_idle(SIM_HBA *hba, int intr)
{
	SIM_MMC_EXT		*ext;

	ext = (SIM_MMC_EXT *)hba->ext;

	if (intr & MMC_INTR_ERROR)
		return (MMC_FAILURE);

	if (intr & MMC_INTR_COMMAND) {
		ext->ident = MMCSD_IDENT_CARD_VERSION;
		ext->istage = 0;

		if (ext->pwr_delay)
			delay(ext->pwr_delay);

		return mmc_sendcmd(hba, MMC_IF_COND, MMC_RSP_R7, MMC_CMD_INTR, 0x1AA);
	}

	return (MMC_SUCCESS);
}

static int mmc_ident_card_version(SIM_HBA *hba, int intr)
{
	SIM_MMC_EXT		*ext = (SIM_MMC_EXT *)hba->ext;

	if (intr & MMC_ERR_CMD_TO)			// timeout
		ext->version = SD_VERSION_1;
	else if (!(intr & MMC_INTR_ERROR) && (ext->mmc_resp[0] & 0x1FFF) == 0x1AA)
		ext->version = SD_VERSION_2;	// Version 2.0
	else
		return (MMC_FAILURE);

	if (ext->pwr_delay)
		delay(ext->pwr_delay);

	ext->ident = MMCSD_IDENT_SD_POWERUP;		// goto the next stage
	ext->istage = 0;

	return mmc_ident_sd_card_powerup(hba, intr);
}

static int mmc_ident_all_send_cid(SIM_HBA *hba, int intr)
{
	SIM_MMC_EXT		*ext = (SIM_MMC_EXT *)hba->ext;

	if (intr & MMC_INTR_ERROR)
		return (CAM_FAILURE);
	ext->ident = MMCSD_IDENT_SET_ADDRESS;
	if (ext->version < MMC_VERSION_1)
		ext->rca = 0;
	else
		ext->rca = 2;
	return mmc_sendcmd(hba, MMC_SET_RELATIVE_ADDR, MMC_RSP_R1, MMC_CMD_INTR, ext->rca << 16);
}

static int mmc_ident_set_address(SIM_HBA *hba, int intr)
{
	SIM_MMC_EXT		*ext = (SIM_MMC_EXT *)hba->ext;

	if (intr & MMC_INTR_ERROR)
		return (CAM_FAILURE);

	ext->eflags |= MMC_EFLAG_PPL;
	if (ext->version < MMC_VERSION_1)
		ext->rca = ext->mmc_resp[0] >> 16;
	ext->ident   = MMCSD_IDENT_SEND_CID;

	/* Ask card to send CID */
	return mmc_sendcmd(hba, MMC_SEND_CID, MMC_RSP_R2, MMC_CMD_INTR, ext->rca << 16);
}

static int mmc_ident_send_cid(SIM_HBA *hba, int intr)
{
	SIM_MMC_EXT		*ext = (SIM_MMC_EXT *)hba->ext;

	if (intr & MMC_INTR_ERROR)
		return (CAM_FAILURE);
	ext->raw_cid[0] = ext->mmc_resp[0];
	ext->raw_cid[1] = ext->mmc_resp[1];
	ext->raw_cid[2] = ext->mmc_resp[2];
	ext->raw_cid[3] = ext->mmc_resp[3];
	ext->ident = MMCSD_IDENT_SEND_CSD;

	return mmc_sendcmd(hba, MMC_SEND_CSD, MMC_RSP_R2, MMC_CMD_INTR, ext->rca << 16);
}

static int mmc_ident_send_csd(SIM_HBA *hba, int intr)
{
	SIM_MMC_EXT		*ext = (SIM_MMC_EXT *)hba->ext;

	if (intr & MMC_INTR_ERROR)
		return (CAM_FAILURE);
	ext->raw_csd[0] = ext->mmc_resp[0];
	ext->raw_csd[1] = ext->mmc_resp[1];
	ext->raw_csd[2] = ext->mmc_resp[2];
	ext->raw_csd[3] = ext->mmc_resp[3];
	if (hba->verbosity) {
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "CSD : %08X:%08X:%08X:%08X", 
			ext->raw_csd[0], ext->raw_csd[1], ext->raw_csd[2], ext->raw_csd[3]);
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "CID : %08X:%08X:%08X:%08X", 
			ext->raw_cid[0], ext->raw_cid[1], ext->raw_cid[2], ext->raw_cid[3]);
	}
	if (ext->version < MMC_VERSION_1) {
		mmc_parse_sd_csd(hba, &ext->csd.sd_csd, ext->raw_csd);
		mmc_parse_sd_cid(hba, &ext->cid.sd_cid, ext->raw_cid);
	} else {
		mmc_parse_mmc_csd(hba, &ext->csd.mmc_csd, ext->raw_csd);
		mmc_parse_mmc_cid(hba, &ext->cid.mmc_cid, ext->raw_cid);
	}

	ext->ident = MMCSD_IDENT_SEL_DES_CARD;
	return mmc_sendcmd(hba, MMC_SEL_DES_CARD, MMC_RSP_R1, MMC_CMD_INTR, ext->rca << 16);
}

static int mmc_ident_send_scr(SIM_HBA *hba, int intr)
{
	SIM_MMC_EXT		*ext = (SIM_MMC_EXT *)hba->ext;

	if (intr & MMC_INTR_ERROR)
		return (CAM_FAILURE);

	if (ext->setup_pio(hba, SD_SCR_SIZE, MMC_DIR_IN) != 8)
		return (CAM_FAILURE);

	ext->ident = MMCSD_IDENT_PARSE_SCR;

	return mmc_sendcmd(hba, SD_SEND_SCR, MMC_RSP_R1, MMC_CMD_DATA_IN | MMC_CMD_DATA, 0);
}

static int mmc_ident_parse_scr(SIM_HBA *hba, int intr)
{
	SIM_MMC_EXT		*ext = (SIM_MMC_EXT *)hba->ext;
	uint8_t			*pscr = &ext->raw_scr[0];
	int				st;

	if (intr & MMC_INTR_ERROR) {
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "SD: read SCR failed, skip high speed probe.");
		return (CAM_FAILURE);
	}

	if ((st=ext->pio_done(hba, (char *)pscr, SD_SCR_SIZE, MMC_DIR_IN)) != 8) {
		slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "SD: pio_done(8) failed, st=%d", st);
		return (CAM_FAILURE);
	}

	if (hba->verbosity)
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "SCR : %02X:%02X:%02X:%02X:%02X:%02X:%02X:%02X",
			pscr[0], pscr[1], pscr[2], pscr[3], pscr[4], pscr[5], pscr[6], pscr[7]);

	ext->sd_scr.scr_structure         = pscr[0] >> 4;
	ext->sd_scr.sd_spec               = pscr[0] & 0x0F;
	ext->sd_scr.data_stat_after_erase = pscr[1] >> 7;
	ext->sd_scr.sd_security           = (pscr[1] >> 4) & 0x0F;
	ext->sd_scr.sd_bus_width          = pscr[1] & 0x0F;

	if (hba->verbosity) {
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "SD SCR info:");
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " SCR_STRUCTURE         : %x", ext->sd_scr.scr_structure);
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " SD_SPEC               : %x", ext->sd_scr.sd_spec);
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " DATA_STAT_AFTER_ERASE : %x", ext->sd_scr.data_stat_after_erase);
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " SD_SECURITY           : %x", ext->sd_scr.sd_security);
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, " SD_BUS_WIDTH          : %x", ext->sd_scr.sd_bus_width);
	}

	if (ext->sd_scr.sd_spec < 1 || ext->set_blksz(hba, 64) != MMC_SUCCESS)
		return mmc_ident_sd_clock(hba);

	if (ext->setup_pio(hba, 64, MMC_DIR_IN) != 64)
		return mmc_ident_sd_clock(hba);

	ext->ident = MMCSD_IDENT_CHECK_MODE;

	return mmc_sendcmd(hba, SD_SET_BUS_WIDTH, MMC_RSP_R1, MMC_CMD_DATA_IN | MMC_CMD_DATA, 0x00ffff01);
}

static int mmc_ident_check_mode(SIM_HBA *hba, int intr)
{
	SIM_MMC_EXT		*ext = (SIM_MMC_EXT *)hba->ext;
	uint8_t			tmp[64];

	if (intr & MMC_INTR_ERROR) {
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "SD: check mode failed, skip high speed probe.");
		return mmc_ident_sd_clock(hba);
	}

	if (ext->pio_done(hba, (char *)tmp, 64, MMC_DIR_IN) != 64){
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "SD: check mode pio read failed, skip high speed probe.");
		return (CAM_FAILURE);
	}

	if ((tmp[13] & 0x02) == 0)
		goto nohs;

	if (hba->verbosity)
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "SD: Card support high speed mode.");

	if (!(ext->csd.sd_csd.ccc & (1 << 10)))	// Switch command supported?
		goto nohs;

	if (hba->verbosity)
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "SD: Card support switch command.");

	if (!(ext->hccap & MMC_HCCAP_HS))	// Host support HS?
		goto nohs;

	ext->max_dtr = 50000000;

	if (ext->setup_pio(hba, 64, MMC_DIR_IN) != 64)
		return (CAM_FAILURE);

	if (hba->verbosity)
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "SD: Switch to high speed mode.");

	ext->ident = MMCSD_IDENT_SWITCH_MODE;

	return mmc_sendcmd(hba, SD_SET_BUS_WIDTH, MMC_RSP_R1, MMC_CMD_DATA_IN | MMC_CMD_DATA, 0x80ffff01);

nohs:
	return mmc_ident_sd_clock(hba);
}

static int mmc_ident_switch_mode(SIM_HBA *hba, int intr)
{
	SIM_MMC_EXT		*ext = (SIM_MMC_EXT *)hba->ext;
	uint8_t			tmp[64];

	if (intr & MMC_INTR_ERROR)
		return (CAM_FAILURE);

	if (ext->pio_done(hba, (char *)tmp, 64, MMC_DIR_IN) != 64)
		return (CAM_FAILURE);

	if ((tmp[16] & 0x0F) == 1){
		ext->eflags |= MMC_EFLAG_HS;
		if (hba->verbosity)
			slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "SD: Switch to high speed mode successful");
	}
	return mmc_ident_sd_clock(hba);
}

static int mmc_ident_sel_des_card(SIM_HBA *hba, int intr)
{
	SIM_MMC_EXT		*ext = (SIM_MMC_EXT *)hba->ext;

	if (intr & MMC_INTR_ERROR)
		return (CAM_FAILURE);

	ext->ident = MMCSD_IDENT_CHECK_STS;

	return mmc_sendcmd(hba, MMC_SEND_STATUS, MMC_RSP_R1, MMC_CMD_INTR, ext->rca << 16);
}

static int mmc_ident_setbw_app(SIM_HBA *hba, int intr)
{
	SIM_MMC_EXT		*ext = (SIM_MMC_EXT *)hba->ext;

	if (intr & MMC_INTR_ERROR)
		return (CAM_FAILURE);

	ext->ident = MMCSD_IDENT_SET_BW;

	return mmc_sendcmd(hba, SD_SET_BUS_WIDTH, MMC_RSP_R1, MMC_CMD_INTR, 2);
}

static int mmc_ident_setbw(SIM_HBA *hba, int intr)
{
	SIM_MMC_EXT		*ext = (SIM_MMC_EXT *)hba->ext;

	if (intr & MMC_INTR_ERROR)
		return (CAM_FAILURE);

	if ((ext->eflags & MMC_EFLAG_NOHS) || !ext->setup_pio || ext->set_blksz(hba, 8) != MMC_SUCCESS)
		return mmc_ident_sd_clock(hba);

	// Read SCR
	ext->ident = MMCSD_IDENT_SEND_SCR;

	return mmc_sendcmd(hba, MMC_APP_CMD, MMC_RSP_R1, MMC_CMD_INTR, ext->rca << 16);
}

int mmc_card_ident(SIM_HBA *hba, uint32_t intr)
{
	SIM_MMC_EXT		*ext;

	ext = (SIM_MMC_EXT *)hba->ext;

#ifdef MMCSD_DEBUG
	mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: ident %s intr 0x%x", __FUNCTION__, mmc_idents[ext->ident], intr );
#endif

	switch (ext->ident) {
		case MMCSD_IDENT_START:
			return mmc_ident_start(hba);

		case MMCSD_IDENT_GO_IDLE:
			return mmc_ident_go_idle(hba, intr);

		case MMCSD_IDENT_CARD_VERSION:
			return mmc_ident_card_version(hba, intr);

		case MMCSD_IDENT_SD_POWERUP:
			return mmc_ident_sd_card_powerup(hba, intr);

		case MMCSD_IDENT_MMC_POWERUP:
			return mmc_ident_mmc_card_powerup(hba, intr);

		case MMCSD_IDENT_ALL_SEND_CID:
			return mmc_ident_all_send_cid(hba, intr);

		case MMCSD_IDENT_SET_ADDRESS:
			return mmc_ident_set_address(hba, intr);

		case MMCSD_IDENT_SEND_CID:
			return mmc_ident_send_cid(hba, intr);

		case MMCSD_IDENT_SEND_CSD:
			return mmc_ident_send_csd(hba, intr);

		case MMCSD_IDENT_SEL_DES_CARD:
			return mmc_ident_sel_des_card(hba, intr);

		case MMCSD_IDENT_SEND_SCR:
			return mmc_ident_send_scr(hba, intr);

		case MMCSD_IDENT_PARSE_SCR:
			return mmc_ident_parse_scr(hba, intr);

		case MMCSD_IDENT_CHECK_MODE:
			return mmc_ident_check_mode(hba, intr);

		case MMCSD_IDENT_SWITCH_MODE:
			return mmc_ident_switch_mode(hba, intr);

		case MMCSD_IDENT_CHECK_STS:
			return mmc_ident_status_done(hba, intr);

		case MMCSD_IDENT_SET_BW_APP:
			return mmc_ident_setbw_app(hba, intr);

		case MMCSD_IDENT_SET_BW:
			return mmc_ident_setbw(hba, intr);

		case MMCSD_IDENT_EXT_CSD:
			return mmc_ident_read_ext_csd(hba, intr);

		case MMCSD_IDENT_MMC_SWITCH_MODE_HS:
			return mmc_ident_switch_mode_hs(hba, intr);

		case MMCSD_IDENT_MMC_SWITCH_MODE_BUSWIDTH:
			return mmc_ident_switch_buswidth(hba, intr);

		case MMCSD_IDENT_FINAL:
			ext->ident = MMCSD_IDENT_NONE;
			if (intr & MMC_INTR_ERROR)
				return (CAM_FAILURE);
			break;
		default:
			return (CAM_FAILURE);
	}

	return (CAM_SUCCESS);
}

#if defined(__QNXNTO__) && defined(__USESRCVERSION)
#include <sys/srcversion.h>
__SRCVERSION("$URL: http://svn/product/branches/6.6.0/trunk/hardware/devb/mmcsd/mmcsd.c $ $Rev: 736700 $")
#endif
