/*
 * $QNXLicenseC:
 * Copyright 2007, 2008, QNX Software Systems.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You
 * may not reproduce, modify or distribute this software except in
 * compliance with the License. You may obtain a copy of the License
 * at: http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as
 * contributors under the License or as licensors under other terms.
 * Please review this entire file for other proprietary rights or license
 * notices, as well as the QNX Development Suite License Guide at
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

// Module Description:  MMC command processing

#include <sim_mmc.h>

static void mmc_media_check(SIM_HBA *hba);

#ifdef MMC_TRACE
void mmc_trace_start( void )
{
	TraceEvent( _NTO_TRACE_DELALLCLASSES );
	TraceEvent( _NTO_TRACE_CLRCLASSPID, _NTO_TRACE_KERCALL );
	TraceEvent( _NTO_TRACE_CLRCLASSTID, _NTO_TRACE_KERCALL );
	TraceEvent( _NTO_TRACE_CLRCLASSPID, _NTO_TRACE_THREAD );
	TraceEvent( _NTO_TRACE_CLRCLASSTID, _NTO_TRACE_THREAD );

#if 1
	TraceEvent( _NTO_TRACE_ADDALLCLASSES );
#else
	TraceEvent( _NTO_TRACE_ADDCLASS, _NTO_TRACE_CONTROL );
	TraceEvent( _NTO_TRACE_ADDCLASS, _NTO_TRACE_INT );
	TraceEvent( _NTO_TRACE_ADDCLASS, _NTO_TRACE_KERCALL );
	TraceEvent( _NTO_TRACE_ADDCLASS, _NTO_TRACE_PROCESS );
	TraceEvent( _NTO_TRACE_ADDCLASS, _NTO_TRACE_THREAD );
#endif

	TraceEvent( _NTO_TRACE_START );
}

void mmc_trace_stop( void )
{
	TraceEvent( _NTO_TRACE_STOP );
}

void mmc_trace_event( int event, const char *fmt, ... )
{
	char		buf[255];
	va_list		arglist;

	va_start( arglist, fmt );
	vsnprintf( buf, 254, fmt, arglist );
	va_end( arglist );
	TraceEvent( _NTO_TRACE_INSERTUSRSTREVENT, event, buf ); 
}
#endif

ssize_t mmc_slogf(SIM_HBA *hba, int opcode, int severity, int vlevel, const char *fmt, ...)
{
	ssize_t		ret;
	va_list		arglist;
	int			verbosity;

	ret			= 0;
	verbosity	= hba ? hba->verbosity : mmc_ctrl.verbosity;
	
	if (verbosity > 5) {
		va_start(arglist, fmt);
		vfprintf(stderr, fmt, arglist);
		va_end(arglist);
		fprintf(stderr, "\n");
	}

	if (vlevel <= 4 || verbosity >= vlevel) {
		va_start(arglist, fmt);
		ret = vslogf(opcode, severity, fmt, arglist);
		va_end(arglist);
	}
	return (ret);
}

int mmc_pm( SIM_HBA *hba, int op )
{
	SIM_MMC_EXT		*ext;

	ext = (SIM_MMC_EXT *)hba->ext;

//	if( ext->pwrmgnt == NULL || !ext->normval ) {
	if( ext->pwrmgnt == NULL ) {
		return( CAM_SUCCESS );
	}

	if( op ) {
		if( !( ext->eflags & MMC_EFLAG_ACTIVE ) ) {
			ext->eflags |= MMC_EFLAG_ACTIVE;
			ext->pwrmgnt( hba, op );
			simq_timer_settime( hba->simq, SIMQ_TIMER_ENABLE );
#ifdef MMCSD_PWR_MGNT_DEBUG
			mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: active", __FUNCTION__ );
#endif
#if 0
			if( ext->state != MMCSD_STATE_IDENT ) {
				if( mmc_sleep_awake( hba, 0 ) == MMC_SUCCESS ) {
					mmc_select_deselect_card( hba, ext->rca );
				}
				else {
					mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: awake fail", __FUNCTION__ );
				}
			}
#endif
		}
	}
	else {
		if( ( ext->eflags & MMC_EFLAG_ACTIVE ) && ext->state == MMCSD_STATE_IDLE ) {
			ext->eflags &= ~MMC_EFLAG_ACTIVE;
#ifdef MMCSD_PWR_MGNT_DEBUG
			mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: sleep", __FUNCTION__ );
#endif
#if 0 
			if( ext->state != MMCSD_STATE_IDENT ) {
				mmc_select_deselect_card( hba, 0 );
				if( mmc_sleep_awake( hba, 1 ) == MMC_SUCCESS ) {
				}
				else {
					mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: sleep fail", __FUNCTION__ );
				}
			}
#endif
			simq_timer_settime( hba->simq, SIMQ_TIMER_DISABLE );
			ext->pwrmgnt( hba, op );
		}
	}

	return( CAM_SUCCESS );
}

int mmc_post_ccb( SIM_HBA *hba, CCB_SCSIIO *ccb )
{
	SIM_MMC_EXT	*ext;

	ext = (SIM_MMC_EXT *)hba->ext;

	ext->nexus = NULL;
	simq_post_ccb( hba->simq, ccb );

	if( ext->eflags & MMC_EFLAG_PWRDN ) {
#ifdef MMCSD_PWR_MGNT_DEBUG
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: idle immed", __FUNCTION__ );
#endif
		mmc_pm( hba, MMC_PM_IDLE );
	}

	return( CAM_SUCCESS );
}

int mmc_sense(SIM_HBA *hba, int sense, int asc, int ascq)
{
	SIM_MMC_EXT	*ext = (SIM_MMC_EXT *)hba->ext;
	CCB_SCSIIO	*ccb = ext->nexus;
	SCSI_SENSE	*sptr;

	if (ccb) {
		ccb->cam_scsi_status	= SCS_CHECK;
		ccb->cam_ch.cam_status	= CAM_REQ_CMP_ERR;
		sptr					= (SCSI_SENSE *)ccb->cam_sense_ptr;

		if ((ccb->cam_ch.cam_flags & CAM_DIS_AUTOSENSE) || sptr == NULL) {
			return( ccb->cam_ch.cam_status );
		}

		ccb->cam_ch.cam_status |= CAM_AUTOSNS_VALID;

		memset(sptr, 0, sizeof(*sptr));
		sptr->error	= 0x70;		// Error code
		sptr->sense = sense;	// Sense key
		sptr->asc	= asc;		// Additional sense code (Invalid field in CDB)
		sptr->ascq	= ascq;		// Additional sense code qualifier
	}
	return( ccb->cam_ch.cam_status );
}

static void mmc_reset(SIM_HBA *hba)
{

}

int mmc_wait_command( SIM_HBA *hba, int timeo )
{
	SIM_MMC_EXT		*ext;
	struct _pulse	pulse;
	int				rid;
	int				intr;
	uint32_t		timeout;
	struct timespec	time;

	ext   = (SIM_MMC_EXT *)hba->ext;
	
	clock_gettime( CLOCK_MONOTONIC, &time );
	timeout = time.tv_sec + timeo;

	while( 1 ) {
		intr = 0;

		if( ( rid = MsgReceivePulse( hba->chid, &pulse, sizeof( pulse ), NULL ) ) == -1 ) {
			continue;
		}

		switch( pulse.code ) {
			case SIM_INTERRUPT:
				intr = ext->interrupt( hba, hba->cfg.IRQRegisters[0], ext->resp_type, ext->mmc_resp );
				InterruptUnmask( hba->cfg.IRQRegisters[0], hba->iid );
				break;

			case SIM_DMA_INTERRUPT:
				intr = ext->interrupt( hba, hba->cfg.IRQRegisters[0], ext->resp_type, ext->mmc_resp );
				if( hba->cfg.NumIRQs > 1 ) {
					InterruptUnmask( hba->cfg.IRQRegisters[1], ext->iid );
				}
				break;

			case SIM_TIMER:
					// check media if card detection interrupt is not supported
				if( !( ext->hccap & MMC_HCCAP_CD_INTR ) ) {
					mmc_media_check( hba );
				}

				if( !( ext->eflags & MMC_EFLAG_INSERTED ) ) {
					return( MMC_FAILURE );
				}
				break;

			default:
				break;
		}

#ifdef MMCSD_DEBUG
		mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: intr %x", __FUNCTION__, intr );
#endif

		if( intr & MMC_INTR_CARD ) {
			mmc_media_check( hba );
			return( MMC_FAILURE );
		}
		
		if( ( intr & MMC_INTR_ERROR ) ) {
			return( MMC_FAILURE );
		}

		if( ( intr & ( MMC_INTR_COMMAND | MMC_INTR_DATA | MMC_INTR_RBRDY | MMC_INTR_WBRDY ) ) ) {
			return( MMC_SUCCESS );
		}
		
		if( timeo != MMC_TIME_INFINITY ) {
			clock_gettime( CLOCK_MONOTONIC, &time );
			if( timeout < time.tv_sec ) {
#ifdef MMCSD_DEBUG
				mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: timeout", __FUNCTION__ );
#endif
				break;
			}
		}
	}
	return( MMC_FAILURE );
}

int mmc_wait_card_ready(SIM_HBA *hba)
{
	SIM_MMC_EXT	*ext;
	uint32_t	timeout;
	struct timespec time;

	ext = (SIM_MMC_EXT *)hba->ext;

	if(!(ext->hccap & MMC_HCCAP_NOCD_BUSY)){
		return (MMC_SUCCESS);
	}
	
	clock_gettime( CLOCK_MONOTONIC, &time );
	timeout = time.tv_sec + (ext->busy_retry/1000);
	while(1){
		if(mmc_get_card_status(hba)!=MMC_SUCCESS){
			slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "%s: failed %x", __func__, ext->mmc_resp[0]);
			break;
		}
		if( ext->mmc_resp[0] & 0x100){
			return MMC_SUCCESS;
		}
		//TOD. what about error status (not in programming state)
		clock_gettime( CLOCK_MONOTONIC, &time );
		if( timeout < time.tv_sec ) {
			slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "%s: timeout,  card status %x", __func__, ext->mmc_resp[0]);
			break;
		}
	}

	return MMC_FAILURE;
}

static void mmc_error(SIM_HBA *hba, int mmc_status)
{
	SIM_MMC_EXT	*ext = (SIM_MMC_EXT *)hba->ext;
	CCB_SCSIIO	*ccb = ext->nexus;

	if (ccb) {
		switch (mmc_status) {
			case MMC_DATA_OVERRUN:
				ccb->cam_ch.cam_status = CAM_DATA_RUN_ERR;
				break;
			case MMC_NOT_PRESENT:
				mmc_sense(hba, SK_NOT_RDY, ASC_MEDIA_NOT_PRESENT, 0);
				break;
			case MMC_TIMEOUT:
			case MMC_COMMAND_FAILURE:
				mmc_reset(hba);
				ccb->cam_ch.cam_status = CAM_CMD_TIMEOUT;
				break;
			case MMC_READ_ERROR:			// CRC errors
			case MMC_WRITE_ERROR:
			default:
				ccb->cam_ch.cam_status = CAM_CMD_TIMEOUT;
				break;
		}
	}
}

static void mmc_msense(SIM_HBA *hba)
{
	SIM_MMC_EXT			*ext;
	CCB_SCSIIO			*ccb;
	MODE_PARM_HEADER10	*hdr;

	ext = (SIM_MMC_EXT *)hba->ext;
	ccb = ext->nexus;

	if (!(ext->eflags & MMC_EFLAG_READY)) {
		mmc_sense(hba, SK_NOT_RDY, ASC_MEDIA_NOT_PRESENT, 0);
		return;
	}

	switch (ccb->cam_cdb_io.cam_cdb_bytes[2]) {
		case MP_RW_ERR:
			hdr = (MODE_PARM_HEADER10 *)ccb->cam_data.cam_data_ptr;
			memset(hdr, 0, sizeof(*hdr));

			if( ext->targets[ccb->cam_ch.cam_target_id].partitions[ccb->cam_ch.cam_target_lun].pflags & MMC_PFLAG_WP ) {
				hdr->device_specific |= MP_DS_WP;
			}

			ccb->cam_ch.cam_status = CAM_REQ_CMP;
			ccb->cam_scsi_status   = SCS_GOOD;
			break;
		default:
			mmc_sense(hba, SK_ILLEGAL, ASC_INVALID_FIELD, 0);
			break;
	}
}

static void mmc_unit_ready(SIM_HBA *hba)
{
	SIM_MMC_EXT	*ext = (SIM_MMC_EXT *)hba->ext;
	CCB_SCSIIO	*ccb = ext->nexus;

	if (ext->eflags & MMC_EFLAG_MEDIA_CHANGED) {
		mmc_sense(hba, SK_UNIT_ATN, ASC_MEDIUM_CHANGED, 0);
		atomic_clr(&ext->eflags, MMC_EFLAG_MEDIA_CHANGED);
	} else if (ext->eflags & MMC_EFLAG_READY) {
		if (ext->detect(hba) != MMC_SUCCESS) {
			mmc_media_check(hba);
			mmc_sense(hba, SK_NOT_RDY, ASC_MEDIA_NOT_PRESENT, 0);
		} else
			ccb->cam_ch.cam_status = CAM_REQ_CMP;
	} else
		mmc_sense(hba, SK_NOT_RDY, ASC_MEDIA_NOT_PRESENT, 0);
}

int mmc_evpd_inquiry( SIM_HBA *hba, CCB_SCSIIO *ccb )
{
	SIM_MMC_EXT						*ext;
	VPD_HEADER						*vh;
	BLOCK_LIMITS					*bl;
	BLOCK_DEVICE_CHARACTERISTICS	*bdc;
	uint8_t							*vpd;
	uint16_t						lps;

	ext	= (SIM_MMC_EXT *)hba->ext;

	vpd	= (uint8_t *)ccb->cam_data.cam_data_ptr;
	vh	= (VPD_HEADER *)vpd;
	bl	= (BLOCK_LIMITS *)(vh+1);
	bdc	= (BLOCK_DEVICE_CHARACTERISTICS *)(vh+1);
	memset( vpd, 0, ccb->cam_dxfer_len );

	switch( ccb->cam_cdb_io.cam_cdb_bytes[2] ) {
		case INQ_PC_VPD_SUPPORTED: {
			uint8_t		evp_sup[] = { D_DIR_ACC, INQ_PC_VPD_SUPPORTED, 0, 3,
							INQ_PC_VPD_SUPPORTED, INQ_PC_BLOCK_LIMITS,
							INQ_PC_BLOCK_DEVICE_CHARACTERISTICS };

			if( ccb->cam_dxfer_len < sizeof( evp_sup ) ) {
				return( mmc_sense( hba, SK_ILLEGAL, ASC_INVALID_FIELD, 0 ) );
			}
			memcpy( vpd, evp_sup, sizeof( evp_sup ) );
			break;
		}

		case INQ_PC_BLOCK_LIMITS:
			if( ccb->cam_dxfer_len < ( sizeof( *vh ) + sizeof( *bl ) ) ) {
				return( mmc_sense( hba, SK_ILLEGAL, ASC_INVALID_FIELD, 0 ) );
			}
			vh->peripheral		= D_DIR_ACC;
			vh->page_code		= INQ_PC_BLOCK_LIMITS;
			vh->page_length		= ENDIAN_BE16( 60 );

			lps = 1;
			bl->optimal_xfer_len_granularity	= ENDIAN_BE16( lps );

			if( ext->dcap & MMC_DCAP_TRIM ) {
				bl->max_unmap_lba_count			= ENDIAN_BE32( MMC_TRIM_MAX_LBA );
				bl->max_unmap_desc_count		= ENDIAN_BE32( 1 );
				bl->optimal_unmap_granularity	= ENDIAN_BE32( 1 );
			}
			break;

		case INQ_PC_BLOCK_DEVICE_CHARACTERISTICS:
			if( ccb->cam_dxfer_len < ( sizeof( *vh ) + sizeof( *bdc ) ) ) {
				return( mmc_sense( hba, SK_ILLEGAL, ASC_INVALID_FIELD, 0 ) );
			}
			vh->peripheral				= D_DIR_ACC;
			vh->page_code				= INQ_PC_BLOCK_DEVICE_CHARACTERISTICS;
			vh->page_length				= ENDIAN_BE16( 60 );

			bdc->medium_rotation_rate	= ENDIAN_BE16( 1 );		// 1 indicates non rotating
			bdc->nominal_form_factor	= 0;

			break;

		default:
			return( mmc_sense( hba, SK_ILLEGAL, ASC_INVALID_FIELD, 0 ) );
			break;
	}

	ccb->cam_ch.cam_status	= CAM_REQ_CMP;
	ccb->cam_scsi_status	= SCS_GOOD;
	return( ccb->cam_ch.cam_status );
}

int mmc_std_inquiry( SIM_HBA *hba, CCB_SCSIIO *ccb )
{
	SIM_MMC_EXT		*ext;
	SCSI_INQUIRY	*iptr;
	char			buf[8];

	ext		= (SIM_MMC_EXT *)hba->ext;
	iptr	= (SCSI_INQUIRY *)ccb->cam_data.cam_data_ptr;

	memset(iptr, 0, sizeof(*iptr));

	iptr->peripheral	= D_DIR_ACC;
	iptr->rmb			= CAM_TRUE;
	iptr->version		= INQ_VER_SPC3;			// SPC-3
	iptr->adlen			= 96 - 4; // 36 - 4;

	if (ext->version < MMC_VERSION_1) {
		/* Vendor ID */
		strcpy((char *)&iptr->vend_id[0], "SD:");
		ultoa(ext->cid.sd_cid.mid, buf, 10);
		iptr->vend_id[3] = buf[0];
		iptr->vend_id[4] = buf[1];
		iptr->vend_id[5] = buf[2];

		/* Product ID */
		strcpy((char *)&iptr->prod_id[0], (char *)ext->cid.sd_cid.pnm);

		/* Product revision level, BCD code */
		iptr->prod_rev[0] = (ext->cid.sd_cid.prv >> 4) + '0';
		iptr->prod_rev[1] = '.';
		iptr->prod_rev[2] = (ext->cid.sd_cid.prv & 0x0F) + '0';
	} else {
		strcpy((char *)&iptr->vend_id[0], "MMC:");

		if (ext->csd.mmc_csd.mmc_prot < 2) {
			ultoa(ext->cid.mmc_cid.mid >> 16, buf, 10);

			/* Product revision level, BCD code */
			iptr->prod_rev[0] = ext->cid.mmc_cid.hwr + '0';
			iptr->prod_rev[1] = '.';
			iptr->prod_rev[2] = ext->cid.mmc_cid.fwr + '0';
		} else {
			ultoa(ext->cid.mmc_cid.mid, buf, 10);
		}
		iptr->vend_id[4] = buf[0];
		iptr->vend_id[5] = buf[1];
		iptr->vend_id[6] = buf[2];
		/* Product ID */
		strcpy((char *)&iptr->prod_id[0], (char *)ext->cid.mmc_cid.pnm);

		// Vendor Specific
		ultoa(ext->cid.mmc_cid.psn, (char *)&iptr->vend_spc[0], 16);
	}

	if( ccb->cam_dxfer_len > 56 ) {
		strcpy( (char *)iptr->vend_spc, ext->targets[ccb->cam_ch.cam_target_id].partitions[ccb->cam_ch.cam_target_lun].name );
	}

	ccb->cam_ch.cam_status = CAM_REQ_CMP;
	ccb->cam_scsi_status   = SCS_GOOD;
	return( ccb->cam_ch.cam_status );
}

int mmc_fake_inquiry( SIM_HBA *hba, CCB_SCSIIO *ccb )
{
	SCSI_INQUIRY	*iptr;

	iptr	= (SCSI_INQUIRY *)ccb->cam_data.cam_data_ptr;

	memset(iptr, 0, sizeof(*iptr));
	iptr->peripheral	= D_DIR_ACC;
	iptr->rmb			= CAM_TRUE;
	iptr->version		= INQ_VER_SPC3;			// SPC-3
	iptr->adlen			= 32;
	strcpy((char *)&iptr->vend_id[0], "MMCSD");
	ccb->cam_ch.cam_status = CAM_REQ_CMP;
	ccb->cam_scsi_status   = SCS_GOOD;
	return( ccb->cam_ch.cam_status );
}

int mmc_inquiry( SIM_HBA *hba, CCB_SCSIIO *ccb )
{
	SIM_MMC_EXT		*ext;

	ext		= (SIM_MMC_EXT *)hba->ext;

	if( ext->state == MMCSD_STATE_IDENT ) {
		return( ccb->cam_ch.cam_status );
	}

	if( ( ext->eflags & MMC_EFLAG_READY ) ) {
		if( ccb->cam_cdb_io.cam_cdb_bytes[1] & INQ_OPT_EVPD ) {
			mmc_evpd_inquiry( hba, ccb );
		}
		else {
			mmc_std_inquiry( hba, ccb );
		}
		return( ccb->cam_ch.cam_status );
	}
	if(!ext->normval ){
		return( mmc_sense( hba, SK_NOT_RDY, ASC_MEDIA_NOT_PRESENT, 0 ) );
	}else{
		return( mmc_fake_inquiry( hba, ccb ) );
	}
}

// this routine is only needed if the driver reports a not ready condition
static void mmc_spindle(SIM_HBA *hba)
{
	SIM_MMC_EXT	*ext;
	CCB_SCSIIO	*ccb;

	ext = (SIM_MMC_EXT *)hba->ext;
	ccb = ext->nexus;

	if (!(ext->eflags & MMC_EFLAG_READY)) {
		mmc_sense(hba, SK_NOT_RDY, ASC_MEDIA_NOT_PRESENT, 0);
		return;
	}

	switch (ccb->cam_cdb_io.cam_cdb_bytes[4]) {
		case LD_CMD_STOP:
		case LD_CMD_START:
			ccb->cam_ch.cam_status	= CAM_REQ_CMP;
			ccb->cam_scsi_status	= SCS_GOOD;
			break;
		case LD_CMD_EJECT:
		case LD_CMD_LOAD:
		default:
			mmc_sense(hba, SK_ILLEGAL, 0x24, 0);
			break;
	}
}

static void mmc_capacity( SIM_HBA *hba )
{
	SIM_MMC_EXT		*ext;
	READ_CAPACITY	*cptr;
	CCB_SCSIIO		*ccb;
	uint32_t		elba;

	ext		= (SIM_MMC_EXT *)hba->ext;
	ccb		= ext->nexus;
	cptr	= (READ_CAPACITY *)ccb->cam_data.cam_data_ptr;
	elba	= ext->targets[ccb->cam_ch.cam_target_id].partitions[ccb->cam_ch.cam_target_lun].nlba;

	if( ccb->cam_ch.cam_flags & CAM_DATA_PHYS ) {
		mmc_error( hba, CAM_PROVIDE_FAIL );
		return;
	}

	if( !( ext->eflags & MMC_EFLAG_READY ) ) {
		mmc_sense( hba, SK_NOT_RDY, ASC_MEDIA_NOT_PRESENT, 0 );
		return;
	}

	if( ext->state == MMCSD_STATE_IDENT ) {
		return;
	}

	if( !elba ) {
		mmc_sense( hba, SK_NOT_RDY, ASC_MEDIA_NOT_PRESENT, 0 );
		return;
	}

	elba--;
	cptr->lba				= ENDIAN_BE32( elba );
	cptr->blk_size			= ENDIAN_BE32( ext->mmc_blksize );
	ccb->cam_ch.cam_status	= CAM_REQ_CMP;
	ccb->cam_scsi_status	= SCS_GOOD;
}

static int mmc_get_serialnumber( SIM_HBA *hba, CCB_DEVCTL *ccb )
{
	SIM_MMC_EXT *ext;
	char		*dsn;

	ext 	= (SIM_MMC_EXT *)hba->ext;

	if (!(ext->eflags & MMC_EFLAG_READY)) {
		mmc_sense(hba, SK_NOT_RDY, ASC_MEDIA_NOT_PRESENT, 0);
		return( CAM_REQ_CMP_ERR );
	}

	dsn = (char *)(ccb->cam_devctl_data);
	memset( dsn, 0, sizeof( ccb->cam_devctl_size ) );
	if (ext->version < MMC_VERSION_1) { //SD
		snprintf( dsn, ccb->cam_devctl_size, "%08x", ext->cid.sd_cid.psn);
	}else{ // MMC
		snprintf( dsn, ccb->cam_devctl_size, "%08x", ext->cid.mmc_cid.psn);
	}
	return( CAM_REQ_CMP );
}

static int mmc_get_cid( SIM_HBA *hba, CCB_DEVCTL *ccb )
{
	SIM_MMC_EXT *ext;
	MMCSD_CID	*cid;

	ext 	= (SIM_MMC_EXT *)hba->ext;
	cid= (MMCSD_CID *)ccb->cam_devctl_data;
	
	cid->pid = getpid();
	cid->flags &= MMCSD_FULL_CID;
	cid->media_change = ext->media_change;
	strncpy(cid->slotname, ext->slotname, MMCSD_MAX_SLOTNAME_LEN);
	if(!(ext->eflags & MMC_EFLAG_INSERTED)){
		cid->flags |= MMCSD_DEV_NO_MEDIA;
	}else{
		if(ext->eflags & MMC_EFLAG_CARD_PRELOAD)
			cid->flags |= MMCSD_DEV_PRELOAD;
		
		if( ext->targets[0].partitions[0].pflags & MMC_PFLAG_WP ) {
			cid->flags |= MMCSD_DEV_RDONLY;
		}

		if(ext->eflags & MMC_EFLAG_INVALID_CARD){
			cid->flags |= MMCSD_DEV_MEDIA_ERROR;
			return( CAM_REQ_CMP );
		}
		if(ext->eflags & MMC_EFLAG_READY){
			cid->flags |= MMCSD_DEV_RDY;
			cid->speed = ext->speed;
			cid->hwspec_version = ext->version;
			
			if(cid->flags & MMCSD_FULL_CID){
				memcpy(cid->cid.full_cid.cid, ext->raw_cid, 16);
			}else{
				if (ext->version < MMC_VERSION_1) { //SD
					cid->type = MMCSD_CARD_TYPE_SD;
					cid->csd_version = ext->csd.sd_csd.csd_structure;
					cid->mmcprot_version = 0;
					memcpy((void *)&cid->cid.parsed_cid.sd_cid, (void *)&ext->cid.sd_cid, sizeof(sd_cid_t));
				}else{ //MMC
					cid->type = MMCSD_CARD_TYPE_MMC;
					cid->csd_version = ext->csd.mmc_csd.csd_structure;
					cid->mmcprot_version = ext->csd.mmc_csd.mmc_prot;
					memcpy((void *)&cid->cid.parsed_cid.mmc_cid, (void *)&ext->cid.mmc_cid, sizeof(mmc_cid_t));
				}
			}
		}
	}

	return( CAM_REQ_CMP );
}

uint64_t mmc_wp_grp_size( SIM_HBA *hba )
{
	SIM_MMC_EXT		*ext;
	mmc_csd_t		*csd;
	uint64_t		wp_grp_size;

	ext	= (SIM_MMC_EXT *)hba->ext;
	csd = &ext->csd.mmc_csd;

	if( csd->ext_csd.erase_grp_def ) {
		wp_grp_size =	(uint64_t )csd->ext_csd.hc_wp_grp_size *
						(uint64_t )csd->ext_csd.hc_erase_group_size *
						MMC_ERASE_GRP_512K;
	}
	else {
		wp_grp_size =	( csd->erase.mmc_v31.erase_grp_size + 1 ) *
						( csd->erase.mmc_v31.erase_grp_mult + 1 ) *
						( csd->wp_grp_size + 1 );
	}

	return( wp_grp_size );
}

uint64_t mmc_erase_grp_size( SIM_HBA *hba )
{
	SIM_MMC_EXT		*ext;
	mmc_csd_t		*csd;
	uint64_t		erase_grp_size;

	ext	= (SIM_MMC_EXT *)hba->ext;
	csd = &ext->csd.mmc_csd;

	if( csd->ext_csd.erase_grp_def ) {
		erase_grp_size =	(uint64_t )csd->ext_csd.hc_erase_group_size *
							MMC_ERASE_GRP_512K;
	}
	else {
		erase_grp_size =	( csd->erase.mmc_v31.erase_grp_size + 1 ) *
							( csd->erase.mmc_v31.erase_grp_mult + 1 );
	}

	return( erase_grp_size );
}

#ifndef BS_MMC_WP
int mmc_wp( SIM_HBA *hba, CCB_DEVCTL *ccb )
{
	SIM_MMC_EXT			*ext;
	MMCSD_WRITE_PROTECT	*wp;
	MMC_PARTITION		*part;
	uint32_t			nlba;
	uint32_t			slba;
	uint32_t			pend;
	uint64_t			wp_grp_size;

	ext			= (SIM_MMC_EXT *)hba->ext;
	wp			= (MMCSD_WRITE_PROTECT *)ccb->cam_devctl_data;
	part		= &ext->targets[ccb->cam_ch.cam_target_id].partitions[ccb->cam_ch.cam_target_lun];
	nlba		= wp->nlba;
	slba		= part->slba + wp->lba;
	pend		= part->slba + part->nlba - 1;
	wp->mode	&= MMCSD_WP_MODE_PWR_WP_EN;	// limit mode to nothing permanent
	wp_grp_size = mmc_wp_grp_size( hba ) / 512;

	if( slba > pend || ( slba + nlba - 1 ) > pend || ( nlba % wp_grp_size ) ) {
		ccb->cam_devctl_status = EINVAL;
	}
	else if( mmc_write_protect( hba, wp->action, part->ptype, wp->mode, slba, nlba ) ) {
		ccb->cam_devctl_status = EIO;
	}

	return( CAM_REQ_CMP );
}
#endif

static int mmc_erase_devctl( SIM_HBA *hba, CCB_DEVCTL *ccb )
{
	SIM_MMC_EXT			*ext;
	MMCSD_ERASE			*erase;
	MMC_PARTITION		*part;
	uint64_t			nlba;
	uint64_t			slba;
	uint64_t			pend;
	uint64_t			erase_grp_size;

	ext				= (SIM_MMC_EXT *)hba->ext;
	erase			= (MMCSD_ERASE *)ccb->cam_devctl_data;
	part			= &ext->targets[ccb->cam_ch.cam_target_id].partitions[ccb->cam_ch.cam_target_lun];
	nlba			= erase->nlba;
	slba			= part->slba + erase->lba;
	pend			= part->slba + part->nlba - 1;
	erase_grp_size	= mmc_erase_grp_size( hba ) / 512;

//	mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s:  erase_grp_size %llx, slba %llx, nlba %llx, %d", __FUNCTION__, erase_grp_size, slba, nlba, nlba % erase_grp_size );

	if( slba > pend || ( slba + nlba - 1 ) > pend || ( nlba % erase_grp_size ) ) {
		ccb->cam_devctl_status = EINVAL;
		return( CAM_REQ_CMP );
	}

	if( mmc_erase( hba, part->config, erase->action ? MMC_ERASE_SECURE : 0, slba, nlba ) ) {
		ccb->cam_devctl_status = EIO;
	}

	return( CAM_REQ_CMP );
}

static int mmc_get_csd( SIM_HBA *hba, CCB_DEVCTL *ccb )
{
	SIM_MMC_EXT *ext;
	MMCSD_CSD   *csd;

	ext = (SIM_MMC_EXT *)hba->ext;
	csd = (MMCSD_CSD *)ccb->cam_devctl_data;

	csd->pid = getpid();
	csd->flags &= MMCSD_FULL_CID;

	if (!(ext->eflags & MMC_EFLAG_INSERTED)) {
		csd->flags |= MMCSD_DEV_NO_MEDIA;
	} else {
		if (ext->eflags & MMC_EFLAG_CARD_PRELOAD)
			csd->flags |= MMCSD_DEV_PRELOAD;

		if (ext->eflags & MMC_EFLAG_WP)
			csd->flags |= MMCSD_DEV_RDONLY;

		if (ext->eflags & MMC_EFLAG_INVALID_CARD) {
			csd->flags |= MMCSD_DEV_MEDIA_ERROR;
			return (CAM_REQ_CMP);
		}

		if (ext->eflags & MMC_EFLAG_READY) {
			csd->flags |= MMCSD_DEV_RDY;
			if (ext->version < MMC_VERSION_1) { //SD
				csd->type = MMCSD_CARD_TYPE_SD;
				csd->csd_version = ext->csd.sd_csd.csd_structure;
				csd->mmcprot_version = 0;
				memcpy((void *)&csd->csd.sd_csd, (void*)&ext->csd.sd_csd, sizeof(sd_csd_t));
			} else { //MMC
				csd->type = MMCSD_CARD_TYPE_MMC;
				csd->csd_version = ext->csd.mmc_csd.csd_structure;
				csd->mmcprot_version = ext->csd.mmc_csd.mmc_prot;
				memcpy((void *)&csd->csd.mmc_csd, (void*)&ext->csd.mmc_csd, sizeof(mmc_csd_t));
			}
		}
	}

	return( CAM_REQ_CMP );
}

static int mmc_card_register( SIM_HBA *hba, CCB_DEVCTL *ccb )
{
	SIM_MMC_EXT				*ext;
	MMCSD_CARD_REGISTER		*cr;
	int						len;
	uint8_t					*dptr;
	uint8_t					data[512];

	ext		= (SIM_MMC_EXT *)hba->ext;
	cr		= (MMCSD_CARD_REGISTER *)ccb->cam_devctl_data;
	len 	= ccb->cam_devctl_size - sizeof( MMCSD_CARD_REGISTER );
	dptr	= (uint8_t *)(cr + 1);

#ifdef MMCSD_DEBUG
	mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: action %d, type %d, address %d, size: %d, len %d", __FUNCTION__, cr->action, cr->type, cr->address, ccb->cam_devctl_size, len );
#endif

	if( !( ext->eflags & MMC_EFLAG_READY ) ) {
		ccb->cam_devctl_status	= EIO;
		return( CAM_REQ_CMP );
	}

	if( cr->action == MMCSD_CR_ACTION_READ ) {
		switch( cr->type ) {
			case MMCSD_REG_TYPE_CID:
				memcpy( dptr, ext->raw_cid, min( len, sizeof( ext->raw_cid ) ) );
				break;

			case MMCSD_REG_TYPE_CSD:
				memcpy( dptr, ext->raw_csd, min( len, sizeof( ext->raw_csd ) ) );
				break;

			case MMCSD_REG_TYPE_EXT_CSD:
				if( ext->version < MMC_VERSION_1 ) {
					ccb->cam_devctl_status	= ENOTSUP;
				}
				else {
					if( mmc_send_ext_csd( hba, data ) == MMC_SUCCESS ) {
						memcpy( dptr, data, min( len, sizeof( data ) ) );
					}
					else {
						ccb->cam_devctl_status	= EIO;
					}
				}
				break;

			case MMCSD_REG_TYPE_SCR:
				if( ext->version >= MMC_VERSION_1 ) {
					ccb->cam_devctl_status	= ENOTSUP;
				}
				else {
					memcpy( dptr, ext->raw_scr, min( len, sizeof( ext->raw_scr ) ) );
				}
				break;
					
			default:
				ccb->cam_devctl_status	= EINVAL;
				break;
		}
		cr->type = ( ext->version < MMC_VERSION_1 ) ? MMCSD_CARD_TYPE_SD : MMCSD_CARD_TYPE_MMC;
	}
	else {
		ccb->cam_devctl_status	= ENOTSUP;
	}

	return( CAM_REQ_CMP );
}

int mmc_verbosity( SIM_HBA *hba, CCB_DEVCTL *ccb )
{
	SIM_MMC_EXT			*ext;
	CAM_VERBOSITY		*ver;

	ext = (SIM_MMC_EXT *)hba->ext;
	ver	= (CAM_VERBOSITY *)ccb->cam_devctl_data;

	if( ver->modules == CAM_MODULE_SIM ) {
		ccb->cam_devctl_status	= EOK;
		hba->verbosity			= ver->verbosity;
		ext->cmdlog				= ver->flags;
	}

	return( CAM_REQ_CMP );
}

static int mmc_rpmb_get_raw_cid(SIM_HBA *hba, CCB_DEVCTL *ccb) {
	SIM_MMC_EXT *ext = (SIM_MMC_EXT *) hba->ext;

	memcpy(ccb->cam_devctl_data, ext->raw_cid, sizeof(ext->raw_cid));
	ccb->cam_devctl_status = EOK;
	return CAM_REQ_CMP;
}

static int mmc_rpmb_erased_val(SIM_HBA *hba, CCB_DEVCTL *ccb) {
	SIM_MMC_EXT *ext = (SIM_MMC_EXT *) hba->ext;
	uint8_t *erasedVal = (uint8_t *) ccb->cam_devctl_data;

	if (erasedVal) {
		*erasedVal = ext->extended_csd[EXT_CSD_ERASED_MEM_CONT];
		ccb->cam_devctl_status = EOK;
	}
	else {
		ccb->cam_devctl_status = EINVAL;
	}
	return CAM_REQ_CMP;
}

static int mmc_rpmb_size(SIM_HBA *hba, CCB_DEVCTL *ccb) {
	SIM_MMC_EXT *ext = (SIM_MMC_EXT *) hba->ext;
	uint8_t *size = (uint8_t *) ccb->cam_devctl_data;

	if (size) {
		*size = (uint8_t) ext->extended_csd[EXT_CSD_RPMB_SIZE_MULT];
		ccb->cam_devctl_status = EOK;
	}
	else {
		ccb->cam_devctl_status = EINVAL;
	}
	return CAM_REQ_CMP;
}

static int mmc_rpmb_rw( SIM_HBA *hba, CCB_DEVCTL *ccb, int direction )
{
	SIM_MMC_EXT		*ext;
	MMC_PARTITION	*part;
	MMCSD_RPMB_REQ	*rpmb_frame_req;
    static MMCSD_RPMB_FRAME *rpmb_frame_virt = NULL;
    static MMCSD_RPMB_FRAME *cached_frame = NULL;
	int reliable = 0;

	ext		= (SIM_MMC_EXT *) hba->ext;
	part	= &ext->targets[ccb->cam_ch.cam_target_id].partitions[ccb->cam_ch.cam_target_lun];
	rpmb_frame_req = (MMCSD_RPMB_REQ *)ccb->cam_devctl_data;

	if (rpmb_frame_req->req_type == 0x100 || rpmb_frame_req->req_type == 0x300) {
		reliable = 1;
	}

    if (!rpmb_frame_virt || cached_frame != rpmb_frame_req->rpmb_frame_paddr) {
		slogf(_SLOGC_SIM_MMC, _SLOG_INFO,
				"RPMB : resolving new shared buffer location 0x%x!",
				(uint32_t) rpmb_frame_req->rpmb_frame_paddr);

		cached_frame = rpmb_frame_req->rpmb_frame_paddr;
		rpmb_frame_virt = (MMCSD_RPMB_FRAME *) mmap(NULL, __PAGESIZE,
				PROT_NOCACHE | PROT_READ | PROT_WRITE, MAP_PHYS | MAP_SHARED,
				NOFD, (off_t) rpmb_frame_req->rpmb_frame_paddr);

		slogf(_SLOGC_SIM_MMC, _SLOG_INFO,
				"RPMB : new virt buffer location: 0x%x!",
				(uint32_t) rpmb_frame_virt);
	}

	if (!(ext->eflags & MMC_EFLAG_READY)) {
		mmc_sense(hba, SK_NOT_RDY, ASC_MEDIA_NOT_PRESENT, 0);
		ccb->cam_devctl_status = EBUSY;
		return CAM_REQ_CMP;
	}

	if (mmc_rpmb_rw_frame(hba, rpmb_frame_virt, part->config, direction, reliable) != MMC_SUCCESS) {
		slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "RPMB: read/write frame failed");
		ccb->cam_devctl_status = EIO;
	}

	// if we're doing a block write, issue the write status read request at the same time
	// as an "atomic" set of transaction.
	if(rpmb_frame_req->req_type == 0x300) {
		*rpmb_frame_virt = (MMCSD_RPMB_FRAME) {{0}};
		rpmb_frame_virt->req_resp = 0x500;

		if (mmc_rpmb_rw_frame(hba, rpmb_frame_virt, part->config, direction, 0) != MMC_SUCCESS) {
			slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "RPMB: read/write frame failed");
			ccb->cam_devctl_status = EIO;
		}
	}
	rpmb_frame_req->req_type = 0;
	return CAM_REQ_CMP;
}

static void mmc_dump_vuc(SIM_MMC_EXT *ext, MMCSD_VUC_CMD *vuc)
{
	slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "vuc flags: %x", vuc->flags);
	slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "vuc arg: %x", vuc->arg);
	slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "vuc cmd: %04x", vuc->opcode);
	if(vuc->flags & MMCSD_VUC_RCA){
		slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "vuc rca: %04x", ext->rca);
	}
	slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "vuc blksz: %d", vuc->blk_sz);
	slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "vuc paddr: %x", vuc->data_ptr);
	slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "vuc data_len: %d", vuc->data_len);
	slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "vuc buf_off: %d", vuc->buf_off);
	slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "vuc timeout: %d", vuc->timeout);
	slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "vuc delay: %x", vuc->postdelay_us);
}

static inline void mmc_restore_ac12(SIM_MMC_EXT *ext, uint32_t ac12)
{
	ext->hccap &= ~MMC_HCCAP_ACMD12;
	ext->hccap |= ac12;
}

static int mmc_vuc_cmd( SIM_HBA *hba, CCB_DEVCTL *ccb )
{
	SIM_MMC_EXT		*ext;
	MMCSD_VUC_CMD	*vuc;
	int				status;
	int				nbytes;
	uint32_t		ac12;
	uint32_t		arg, flags;
	uint8_t			*buf;

	ext = (SIM_MMC_EXT *)hba->ext;
	vuc = (MMCSD_VUC_CMD *)ccb->cam_devctl_data;
	buf = (uint8_t *)(vuc);

	if (!(ext->eflags & MMC_EFLAG_READY)) {
		mmc_sense(hba, SK_NOT_RDY, ASC_MEDIA_NOT_PRESENT, 0);
		ccb->cam_devctl_status = ENODEV;
		return( CAM_REQ_CMP_ERR );
	}

	// Save the ACMD12 flag
	ac12 = ext->hccap & MMC_HCCAP_ACMD12;
	if (vuc->flags & MMCSD_VUC_NOAC12)
		ext->hccap &= ~MMC_HCCAP_ACMD12;
	else
		ext->hccap |= MMC_HCCAP_ACMD12;

	for(; ;){

		if(((vuc->flags & MMCSD_VUC_DATA_PHYS) && ((ext->eflags & MMC_EFLAG_PIO) || ext->setup_dma==NULL))
			|| (!(vuc->flags & MMCSD_VUC_DATA_PHYS) && (ext->setup_pio==NULL))){
			slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "%s: %s not supported", __FUNCTION__, (vuc->flags & MMCSD_VUC_DATA_PHYS)?"DMA":"PIO");
			ccb->cam_devctl_status = EINVAL;
			mmc_dump_vuc(ext, vuc);
			mmc_restore_ac12(ext, ac12);
			return CAM_REQ_CMP_ERR;
		}

		vuc->result = MMC_VUC_SUCCESS;
		if(vuc->flags & (MMC_RSP_PRESENT<<MMCSD_VUC_RESP_OFF)){
			memset((void *)&vuc->resp[0], 0, 4*sizeof(uint32_t));
		}

		//do we need command 55?
		if(vuc->flags & MMCSD_VUC_ACMD){
			if( ( status = mmc_sendcmd( hba, MMC_APP_CMD, MMC_RSP_R1, MMC_CMD_INTR, ext->rca << 16) ) == MMC_SUCCESS ) {
				status = mmc_wait_command( hba, (vuc->timeout) ? vuc->timeout : MMC_TIME_DEFAULT );
			}

			if(status != MMC_SUCCESS){
				ccb->cam_devctl_status = EIO;
				slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "%s(%d): status (%x)", __FUNCTION__, __LINE__, status);
				mmc_dump_vuc(ext, vuc);
				mmc_restore_ac12(ext, ac12);
				return CAM_REQ_CMP_ERR;
			}
		}

		if(vuc->flags & MMCSD_VUC_DATA_MSK){
			flags =MMC_CMD_DATA;
			if(vuc->flags & MMCSD_VUC_DATA_IN){
				ext->dir = MMC_DIR_IN;
				flags |= MMC_CMD_DATA_IN;
			}else{
				ext->dir = MMC_DIR_OUT;
			}
			if(vuc->flags & MMCSD_VUC_DATA_PHYS){
				flags |= MMC_CMD_DATA_DMA;
				nbytes = ext->setup_dma(hba, vuc->data_ptr, vuc->data_len, ext->dir);
			}else{
				nbytes = ext->setup_pio(hba, vuc->data_len, ext->dir);
			}
			if (nbytes < vuc->data_len){
				ccb->cam_devctl_status = EINVAL;
				slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "%s(%d): status (%x)", __FUNCTION__, __LINE__, status);
				mmc_dump_vuc(ext, vuc);
				mmc_restore_ac12(ext, ac12);
				return CAM_REQ_CMP_ERR;
			}
			if(nbytes > ext->mmc_blksize){
				flags |= MMC_CMD_DATA_MULTI;
			}
		}else{
			flags = MMC_CMD_INTR;
		}

		//now issue the command
		arg = vuc->arg;
		if(vuc->flags & MMCSD_VUC_RCA){
			arg |= (ext->rca <<16);
		}

		if( ( status = mmc_sendcmd( hba, vuc->opcode, (vuc->flags & MMCSD_VUC_RESP_MSK)>>MMCSD_VUC_RESP_OFF, flags, arg) ) == MMC_SUCCESS ) {
			status = mmc_wait_command( hba, (vuc->timeout) ? vuc->timeout : MMC_TIME_DEFAULT );
		}

		if(vuc->flags & (MMC_RSP_PRESENT<<MMCSD_VUC_RESP_OFF)){
			memcpy((void *)&vuc->resp[0], (void *)&ext->mmc_resp[0], 4*sizeof(uint32_t));
		}

		if(status != MMC_SUCCESS){
			vuc->result = MMC_VUC_FAILED;
			slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "%s(%d): status (%x)", __FUNCTION__, __LINE__, status);
			mmc_dump_vuc(ext, vuc);
			mmc_restore_ac12(ext, ac12);
			return CAM_REQ_CMP_ERR;
		}

		//check finish state
		if(vuc->flags & MMCSD_VUC_DATA_MSK){
			if( flags & MMC_CMD_DATA_DMA){
				if( ext->dma_done(hba, ext->dir) != MMC_SUCCESS){
					vuc->result = MMC_VUC_FAILED;
					slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "%s(%d): status (%x)", __FUNCTION__, __LINE__, status);
					mmc_dump_vuc(ext, vuc);
					mmc_restore_ac12(ext, ac12);
					return CAM_REQ_CMP_ERR;
				}
			}else{
				ext->offset = 0;
				for(;;) {
					do{
						nbytes = ext->pio_done(hba, (char *)buf + vuc->buf_off + ext->offset,
									vuc->data_len - ext->offset, ext->dir);
						ext->offset += nbytes;
					}while(nbytes > 0 && ext->offset != vuc->data_len);

					if (ext->offset != vuc->data_len){
						if((status = mmc_wait_command( hba, (vuc->timeout) ? vuc->timeout : MMC_TIME_DEFAULT )) != MMC_SUCCESS){
							vuc->result = MMC_VUC_FAILED;
							slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "%s(%d): status (%x)", __FUNCTION__, __LINE__, status);
							mmc_dump_vuc(ext, vuc);
							mmc_restore_ac12(ext, ac12);
							return CAM_REQ_CMP_ERR;
						}
					}else{
						if(vuc->flags & MMCSD_VUC_DATA_OUT){
							mmc_wait_command( hba, (vuc->timeout) ? vuc->timeout : MMC_TIME_DEFAULT );
						}
						break;
					}
				}
			}
		}

		if(vuc->postdelay_us){
			usleep(vuc->postdelay_us);
		}

		if(vuc->flags & MMCSD_VUC_END)
			break;

		vuc++;
	}

	mmc_restore_ac12(ext, ac12);
	return( CAM_REQ_CMP );
}

static void mmc_devctl( SIM_HBA *hba )
{
	SIM_MMC_EXT		*ext;
	CCB_DEVCTL		*ccb;
	int				cstatus;

	ext		= (SIM_MMC_EXT *)hba->ext;
	ccb		= (CCB_DEVCTL *)ext->nexus;
	cstatus	= CAM_REQ_CMP;
	ccb->cam_devctl_status = EOK;

	switch( ccb->cam_devctl_dcmd ) {
		case DCMD_MMCSD_GET_CID:
			cstatus = mmc_get_cid( hba, ccb );
			break;

		case DCMD_MMCSD_WRITE_PROTECT:
			cstatus = mmc_wp( hba, ccb );
			break;

		case DCMD_MMCSD_GET_CSD:
			cstatus = mmc_get_csd( hba, ccb );
			break;

		case DCMD_MMCSD_ERASE:
			cstatus = mmc_erase_devctl( hba, ccb );
			break;

		case DCMD_MMCSD_CARD_REGISTER:
			cstatus = mmc_card_register( hba, ccb );
			break;

		case DCMD_MMCSD_VUC_CMD:
			cstatus = mmc_vuc_cmd( hba, ccb );
			break;

		case DCMD_CAM_VERBOSITY:
			cstatus = mmc_verbosity( hba, ccb );
			break;

		case DCMD_CAM_DEV_SERIAL_NUMBER:
			cstatus = mmc_get_serialnumber( hba, ccb );
			break;

		case DCMD_MMCSD_RPMB_RW_FRAME:
			cstatus = mmc_rpmb_rw( hba, ccb, MMC_DIR_OUT );
			if( ccb->cam_devctl_status == EOK ) {
				cstatus = mmc_rpmb_rw( hba, ccb, MMC_DIR_IN );
			}
			break;

		case DCMD_MMCSD_GET_CID_RAW:
			cstatus = mmc_rpmb_get_raw_cid( hba, ccb );
			break;

		case DCMD_MMCSD_ERASED_VAL:
			cstatus = mmc_rpmb_erased_val( hba, ccb );
			break;

		case DCMD_MMCSD_RPMB_SIZE:
			cstatus = mmc_rpmb_size( hba, ccb );
			break;

		default:
			ccb->cam_devctl_status = _RESMGR_DEFAULT;
			break;
	}

	ccb->cam_ch.cam_status = cstatus;
}

paddr_t mmc_mphys(CCB_SCSIIO *ccb, paddr_t dptr, int sgi)
{
	mdl_t	*mdl;
	int		cnt;
	ioreq_t	*ioreq;
	off64_t	_off;

	if (ccb->cam_ch.cam_flags & CAM_DATA_PHYS)
		return (dptr);

	if ((ioreq = (ioreq_t *)ccb->cam_req_map)) {
		mdl = ioreq->mdl;
		if (mdl[sgi].vaddr == (caddr_t)dptr)
			return (mdl[sgi].paddr);
 
		for (cnt = ioreq->nmdl; cnt; cnt--, mdl++) {
			if (mdl->vaddr == (caddr_t)dptr)
				return (mdl->paddr);
		}
	}

	mem_offset64((void *)(dptr), NOFD, 1, &_off, NULL);

	return ((paddr_t)_off);
}

static int mmc_stop_xfer(SIM_HBA *hba)
{
	return mmc_sendcmd(hba, MMC_STOP_TRANSMISSION, MMC_RSP_R1B, MMC_CMD_INTR, 0);
}

static int mmc_pio_xfer(SIM_HBA *hba, uint32_t nbytes)
{
	SIM_MMC_EXT	*ext;

	ext	= (SIM_MMC_EXT *)hba->ext;

	if (ext->setup_pio(hba, nbytes, ext->dir) == nbytes)
		return mmc_send_rwcmd(hba, nbytes, 0);

	return 0;
}

static int mmc_dma_xfer(SIM_HBA *hba, paddr_t paddr, uint32_t nbytes)
{
	SIM_MMC_EXT	*ext;

	ext	= (SIM_MMC_EXT *)hba->ext;

	if ((nbytes = ext->setup_dma(hba, paddr, nbytes, ext->dir)) > 0)
		return mmc_send_rwcmd(hba, nbytes, MMC_CMD_DATA_DMA);

	return 0;
}

static void mmc_dma_rw(SIM_HBA *hba)
{
	SIM_MMC_EXT	*ext;
	CCB_SCSIIO	*ccb;
//	int			status;
	uint32_t	nbytes;

	ext	= (SIM_MMC_EXT *)hba->ext;
	ccb	= ext->nexus;

#if 0
	if( ext->dir == MMC_DIR_OUT ) {
		InterruptUnmask( hba->cfg.IRQRegisters[0], hba->iid );
		status = mmc_get_card_status( hba );
		InterruptMask( hba->cfg.IRQRegisters[0], hba->iid );
		if( status == MMC_SUCCESS ) {
//			mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: card status resp %x", __FUNCTION__, ext->mmc_resp[0] );
			if( ( ext->mmc_resp[0] & MMC_WP_VIOLATION ) ) {
				mmc_sense( hba, SK_DATA_PROT, ASC_WRITE_PROTECTED, 0 );
				return;
			}
			if( ( ext->mmc_resp[0] & MMC_ERROR ) ) {
				mmc_error( hba, MMC_WRITE_ERROR );
				return;
			}
		}
	}
#endif

	if (ext->nbytes == ext->offset) {	// All done
		ccb->cam_ch.cam_status = CAM_REQ_CMP;
		ccb->cam_scsi_status   = SCS_GOOD;
		return;
	}

	if ((nbytes = mmc_dma_xfer(hba, ext->paddr + ext->offset, ext->nbytes - ext->offset)) > 0) {
		ext->offset += nbytes;
		ext->blkno  += nbytes / ext->mmc_blksize;
	} else
		mmc_error(hba, CAM_PROVIDE_FAIL);
}

static inline void mmc_pio_initiate(SIM_HBA *hba)
{
	SIM_MMC_EXT	*ext;
	CCB_SCSIIO	*ccb;
	uint32_t	nbytes = 0;

	ext		= (SIM_MMC_EXT *)hba->ext;
	ccb		= ext->nexus;

	if (ccb->cam_ch.cam_flags & CAM_DATA_PHYS) {
		mmc_error(hba, CAM_PROVIDE_FAIL);
		return;
	}

	if (ccb->cam_ch.cam_flags & CAM_SCATTER_VALID) {
		SG_ELEM		*sge = ccb->cam_data.cam_sg_ptr;
		uint32_t	sgi;

		for (sgi = 0; sgi < ccb->cam_sglist_cnt; sgi++, sge++)
			nbytes += sge->cam_sg_count;
	} else
		nbytes = ccb->cam_dxfer_len;

	ext->nbytes = nbytes;
	if (mmc_pio_xfer(hba, nbytes) > 0) {
		ext->state = MMCSD_STATE_DATA;
		ccb->cam_ch.cam_status = CAM_REQ_INPROG;
		return;
	}

	mmc_error(hba, CAM_PROVIDE_FAIL);
}

static inline void mmc_dma_initiate(SIM_HBA *hba)
{
	SIM_MMC_EXT	*ext;
	CCB_SCSIIO	*ccb;
	uint32_t	nbytes;

	ext	= (SIM_MMC_EXT *)hba->ext;
	ccb	= ext->nexus;

	if (ccb->cam_ch.cam_flags & CAM_SCATTER_VALID) {
		if (ext->setup_sgdma) {
			if ((nbytes = ext->setup_sgdma(hba, ext->dir))) {
				if (mmc_send_rwcmd(hba, nbytes, MMC_CMD_DATA_DMA) == nbytes) {
					ccb->cam_ch.cam_status = CAM_REQ_INPROG;
					ext->state = MMCSD_STATE_DATA;
					ext->nbytes = ext->offset = nbytes;	// all done

					return;
				}
			}
		}

		mmc_error(hba, CAM_PROVIDE_FAIL);
	} else {
		ccb->cam_ch.cam_status = CAM_REQ_INPROG;

		ext->paddr = mmc_mphys(ccb, ccb->cam_data.cam_data_ptr, 0);
		ext->nbytes = ccb->cam_dxfer_len;

		mmc_dma_rw(hba);

		if (ccb->cam_ch.cam_status == CAM_REQ_INPROG)
			ext->state = MMCSD_STATE_DATA;
	}
}

static void mmc_pio_stop(SIM_HBA *hba, int dir)
{
	SIM_MMC_EXT	*ext;
	CCB_SCSIIO	*ccb;

	ext = (SIM_MMC_EXT *)hba->ext;
	ccb = ext->nexus;

	if (ext->dir == dir) {
		if (ext->stop == MMCSD_STOP_PEND) {
			if (mmc_stop_xfer(hba) != MMC_SUCCESS) {
				mmc_error(hba, CAM_PROVIDE_FAIL);
				ext->stop = MMCSD_STOP_NONE;
				return;
			}

			ext->stop = MMCSD_STOP_ISSUED;
		} else {
			ccb->cam_ch.cam_status = CAM_REQ_CMP;
			ccb->cam_scsi_status   = SCS_GOOD;
		}
	}
}

static void mmc_pio_rw(SIM_HBA *hba, int status)
{
	SIM_MMC_EXT	*ext;
	CCB_SCSIIO	*ccb;
	uint32_t	nbytes;

	ext = (SIM_MMC_EXT *)hba->ext;
	ccb = ext->nexus;

	if (ext->offset == -1)
		return mmc_pio_stop(hba, MMC_DIR_OUT);

	if (ccb->cam_ch.cam_flags & CAM_SCATTER_VALID) {
		SG_ELEM		*sge = ccb->cam_data.cam_sg_ptr;
		uint32_t	sgi = ext->cur_sg;

		do {
			nbytes = ext->pio_done(hba, (char *)sge[sgi].cam_sg_address + ext->offset,
						sge[sgi].cam_sg_count - ext->offset, ext->dir);
			ext->offset += nbytes;
			if (ext->offset == sge[sgi].cam_sg_count) {
				if (++sgi >= ccb->cam_sglist_cnt)		// All done
					ext->offset = -1;
				else
					ext->offset = 0;
			}
		} while (nbytes > 0 && ext->offset != -1);

		ext->cur_sg = sgi;
	} else {
		do {
			nbytes = ext->pio_done(hba, (char *)ccb->cam_data.cam_data_ptr + ext->offset,
						ccb->cam_dxfer_len - ext->offset, ext->dir);

			ext->offset += nbytes;
			if (ext->offset == ext->nbytes)
				ext->offset = -1;
		} while (nbytes > 0 && ext->offset != -1);
	}

	if (ext->offset == -1)
		return mmc_pio_stop(hba, MMC_DIR_IN);
}

int mmc_set_partition( SIM_HBA *hba, uint32_t partition )
{
	SIM_MMC_EXT		*ext;

	ext		= (SIM_MMC_EXT *)hba->ext;

	if( ext->pactive == partition ) {
		return( CAM_SUCCESS );
	}

//mmc_slogf( NULL, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s set part %d, cur part %d", __FUNCTION__, partition, ext->pactive );

	if( mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_PART_CONFIG, partition ) == MMC_SUCCESS ) {
		ext->pactive = partition;
		return( CAM_SUCCESS );
	}

	return( MMC_FAILURE );
}

void mmc_write_same( SIM_HBA *hba, CCB_SCSIIO *ccb )
{
	SIM_MMC_EXT		*ext;
	CDB				*cdb;
	MMC_PARTITION	*part;
	uint64_t		lba;
	uint32_t		nlba;

	ext						= (SIM_MMC_EXT *)hba->ext;
	cdb						= cam_cdb( ccb );
	ccb->cam_ch.cam_status	= CAM_REQ_CMP;
	part					= &ext->targets[ccb->cam_ch.cam_target_id].partitions[ccb->cam_ch.cam_target_lun];

	lba		= ENDIAN_BE64( UNALIGNED_RET64( cdb->write_same16.lba ) );
	nlba	= ENDIAN_BE32( UNALIGNED_RET32( cdb->write_same16.transfer_len ) );
	lba		+= part->slba;

//	mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: tid %d, lun %d, lba %d, nlba %d, pact %d", __FUNCTION__, ccb->cam_ch.cam_target_id, ccb->cam_ch.cam_target_lun, lba, nlba, ext->pactive );

	if( !( ext->dcap & MMC_DCAP_TRIM ) || !( cdb->write_same16.opt & WS_OPT_UNMAP ) ) {
		mmc_sense( hba, SK_ILLEGAL, ASC_INVALID_FIELD, 0 );
		return;
	}

#if 0
	if( ext->devid == MMC_SANDISK_INAND ) {
			// INAND require a vendor unique command for TRIM
		if( mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, 113, 1 ) ) {
			mmc_sense( hba, SK_ILLEGAL, ASC_INVALID_FIELD, 0 );
			return;
		}
	}
#endif

	if( mmc_erase( hba, part->config, MMC_ERASE_TRIM, lba, nlba ) ) {
		mmc_sense( hba, SK_ILLEGAL, ASC_INVALID_FIELD, 0 );
	}

#if 0
	if( ext->devid == MMC_SANDISK_INAND ) {
		if( mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, 113, 0 ) ) {
			mmc_sense( hba, SK_ILLEGAL, ASC_INVALID_FIELD, 0 );
			return;
		}
	}
#endif
}

void mmc_erase12( SIM_HBA *hba, CCB_SCSIIO *ccb )
{
	SIM_MMC_EXT		*ext;
	CDB				*cdb;
	MMC_PARTITION	*part;
	uint32_t		lba;
	uint32_t		nlba;

	ext						= (SIM_MMC_EXT *)hba->ext;
	cdb						= cam_cdb( ccb );
	ccb->cam_ch.cam_status	= CAM_REQ_CMP;
	part					= &ext->targets[ccb->cam_ch.cam_target_id].partitions[ccb->cam_ch.cam_target_lun];

	lba		= ENDIAN_BE32( UNALIGNED_RET32( cdb->erase12.lba ) );
	nlba	= ENDIAN_BE32( UNALIGNED_RET32( cdb->erase12.transfer_len ) );
	lba		+= part->slba;

//	mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: tid %d, lun %d, lba %d, nlba %d, pact %d", __FUNCTION__, ccb->cam_ch.cam_target_id, ccb->cam_ch.cam_target_lun, lba, nlba, ext->pactive );

	if( mmc_erase( hba, part->config, MMC_ERASE_SECURE, lba, nlba ) ) {
		mmc_sense( hba, SK_ILLEGAL, ASC_INVALID_FIELD, 0 );
	}
}

void mmc_sync( SIM_HBA *hba, CCB_SCSIIO *ccb )
{
	ccb->cam_ch.cam_status  = CAM_REQ_CMP;
	ccb->cam_scsi_status    = SCS_GOOD;
}

void mmc_rw( SIM_HBA *hba, CCB_SCSIIO *ccb, int dir )
{
	SIM_MMC_EXT		*ext;
	MMC_PARTITION	*part;

	ext		= (SIM_MMC_EXT *)hba->ext;
	part	= &ext->targets[ccb->cam_ch.cam_target_id].partitions[ccb->cam_ch.cam_target_lun];

	if( !( ext->eflags & MMC_EFLAG_READY ) ) {
		mmc_sense( hba, SK_NOT_RDY, ASC_MEDIA_NOT_PRESENT, 0 );
		return;
	}

	if( dir == MMC_DIR_OUT && ( part->pflags & MMC_PFLAG_WP ) ) {
		mmc_sense( hba, SK_DATA_PROT, ASC_WRITE_PROTECTED, 0 );
		return;
	}

	if( part->ptype == MMC_PTYPE_RPMB ) {		// no read/write to RPMB
		ccb->cam_ch.cam_status = CAM_PROVIDE_FAIL;
		return;
	}

	ext->offset = 0;
	ext->cur_sg = 0;
	ext->dir    = dir;
	ext->blkno  = ENDIAN_BE32( UNALIGNED_RET32( &ccb->cam_cdb_io.cam_cdb_bytes[2] ) );
	ext->blkno	+= part->slba;

	if( mmc_set_partition( hba, part->config ) == MMC_FAILURE ) {
		mmc_error( hba, CAM_PROVIDE_FAIL );
		return;
	}

	if( ext->eflags & MMC_EFLAG_PIO ) {
		if( ext->setup_pio ) {
			mmc_pio_initiate( hba );
		}
		else {
			mmc_error( hba, CAM_PROVIDE_FAIL );
		}
	}
	else {
		if( ext->setup_dma ) {
			mmc_dma_initiate( hba );
		}
		else {
			mmc_error( hba, CAM_PROVIDE_FAIL );
		}
	}
}

// interpret SCSI commands
void mmc_io_cmds( SIM_HBA *hba )
{
	SIM_MMC_EXT	*ext;
	CCB_SCSIIO	*ccb;
	int			cmd;

	ext = (SIM_MMC_EXT *)hba->ext;
	ccb = ext->nexus;
	cmd = ccb->cam_cdb_io.cam_cdb_bytes[0];

	if( ext->state == MMCSD_STATE_IDENT ) {
		return;
	}

	if( hba->verbosity > 3 ) {
		mmc_display_ccb( hba, ccb );
	}

	switch( cmd ) {
		case SC_MSENSE10:
			mmc_msense( hba );
			break;

		case SC_UNIT_RDY:
			mmc_unit_ready( hba );
			break;

		case SC_INQUIRY:
			mmc_inquiry( hba, ccb );
			break;

		case SC_RD_CAP:
			mmc_capacity( hba );
			break;

		case SC_SPINDLE:
			mmc_spindle( hba );
			break;

		case SC_SYNC:
			mmc_sync( hba, ccb );
			break;

		case SC_WRITE10:
			mmc_rw( hba, ccb, MMC_DIR_OUT );
			break;

		case SC_READ10:
			mmc_rw( hba, ccb, MMC_DIR_IN );
			break;

		case SC_ERASE12:
			mmc_erase12( hba, ccb );
			break;

#ifdef MMC_TRIM_SUP
		case SC_WR_SAME16:
			mmc_write_same( hba, ccb );
			break;
#endif

		default:
			mmc_sense( hba, SK_ILLEGAL, 0x24, 0 );
			break;
	}
}

#ifdef MMC_BKOPS_SUP
int mmc_bkops( SIM_HBA *hba, int op )
{
	SIM_MMC_EXT		*ext;
	uint8_t			ecsd[EXT_CSD_SIZE];

	ext			= (SIM_MMC_EXT *)hba->ext;

	if( !( ext->eflags & MMC_EFLAG_BKOPS ) ) {
		return( CAM_SUCCESS );
	}

	mmc_pm( hba, MMC_PM_ACTIVE );

	if( op ) {							// Timer event, poll status
		if( ext->bkops_status ) {
			ext->bkops_ticks++;
		}
		if( ext->nexus ) {
			return( CAM_SUCCESS );
		}
		if( mmc_send_ext_csd( hba, ecsd ) == MMC_SUCCESS ) {
			if( ext->bkops_status != ecsd[EXT_CSD_BKOPS_STATUS] ) {
				if( hba->verbosity ) {
					mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: bkops status change %d to %d, ticks %d", __FUNCTION__, ext->bkops_status, ecsd[EXT_CSD_BKOPS_STATUS] );
				}
				ext->bkops_ticks	= 0;
				ext->bkops_status	= ecsd[EXT_CSD_BKOPS_STATUS];
			}
		}
	}

	switch( ext->bkops_status ) {
		case MMC_BKOPS_STATUS_OPERATIONS_NONE:
			break;
		
		case MMC_BKOPS_STATUS_OPERATIONS_NON_CRITICAL:
			if( ext->bkops_ticks < MMC_BKOPS_NC_TICKS ) {
				break;
			}

			// fall through

		case MMC_BKOPS_STATUS_OPERATIONS_IMPACTED:
			if( ext->bkops_ticks < MMC_BKOPS_IMPACTED_TICKS ) {
				break;
			}

			// fall through

		case MMC_BKOPS_STATUS_OPERATIONS_CRITICAL:
			if( hba->verbosity ) {
				mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: initiate bkops status %d, ticks %d", __FUNCTION__, ext->bkops_status, ext->bkops_ticks );
			}
			if( mmc_switch( hba, 1, MMC_SWITCH_CMDSET_DFLT, MMC_SWITCH_MODE_WRITE, EXT_CSD_BKOPS_START, EXT_CSD_BKOPS_INITIATE ) == MMC_SUCCESS ) {
				ext->bkops_status	= MMC_BKOPS_STATUS_OPERATIONS_NONE;
				ext->bkops_ticks	= 0;
			}
			else {
				mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: BKOPS_START failure", __FUNCTION__ );
			}
			break;
	}
	
	return( CAM_SUCCESS );
}
#endif

// start executing a new CCB
static void mmc_start_ccb(SIM_HBA *hba)
{
	SIM_MMC_EXT		*ext;
	CCB_SCSIIO		*ccb;

	ext = (SIM_MMC_EXT *)hba->ext;

	do{
		if (ext->nexus && !(simq_ccb_state(ext->nexus, SIM_CCB_QUERY) & SIM_CCB_ABORT))
			return;

		if ((ext->nexus = ccb = simq_ccb_dequeue(hba->simq)) == NULL)
			return;

		ext->idle_ticks = 0;

		mmc_pm( hba, MMC_PM_ACTIVE );

#ifdef MMC_BKOPS_SUP
		mmc_bkops( hba, CAM_FALSE );	// Check for urgent background operations
#endif

		switch (ccb->cam_ch.cam_func_code) {
			case XPT_SCSI_IO:
				mmc_io_cmds(hba);
				break;

			case XPT_DEVCTL:
				mmc_devctl(hba);
				break;

			default:
				slogf(_SLOGC_SIM_MMC, _SLOG_ERROR,
						"mmc_start_ccb: unsupported function code %d", ccb->cam_ch.cam_func_code);
				ccb->cam_ch.cam_status	= CAM_REQ_CMP_ERR;
				ccb->cam_scsi_status	= SCS_CHECK;
				break;
		}

		if (ccb->cam_ch.cam_status != CAM_REQ_INPROG) {
			if( hba->verbosity > 3 ) {
				mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s:  %p, cstatus %x", __FUNCTION__, ccb, ccb->cam_ch.cam_status );
			}
#ifdef MMC_TRACE
			mmc_trace_event( MMC_TRACE_EVENT, "mmc post ccb %p", ccb );
#endif
			mmc_post_ccb( hba, ccb );
		}
	} while ( ext->nexus == NULL );
}

static void mmc_dma_irq(SIM_HBA *hba)
{
	SIM_MMC_EXT	*ext = (SIM_MMC_EXT *)hba->ext;

	if (ext->stop == MMCSD_STOP_PEND) {
		if (mmc_stop_xfer(hba) != MMC_SUCCESS) {
			mmc_error(hba, CAM_PROVIDE_FAIL);
			ext->stop = MMCSD_STOP_NONE;
			return;
		}
		ext->stop = MMCSD_STOP_ISSUED;
		return;
	}

	mmc_dma_rw(hba);
}

static void mmc_ident_irq(SIM_HBA *hba, int intr)
{
	SIM_MMC_EXT	*ext = (SIM_MMC_EXT *)hba->ext;
	CCB_SCSIIO	*ccb = ext->nexus;

	if (mmc_card_ident(hba, intr) == MMC_SUCCESS) {
		if (ext->ident == MMCSD_IDENT_NONE) {	// ident complete
			ext->state = MMCSD_STATE_IDLE;
			atomic_clr_value( &ext->eflags, MMC_EFLAG_RESETTING );
			if(!(ext->eflags & MMC_EFLAG_READY)){
				atomic_set(&ext->eflags, MMC_EFLAG_READY);
				MsgSendPulse(mmc_ctrl.coid, ext->mmc_priority, MMCSD_PULSE_CARD_READY, 0);
			}
			if (ccb)
				mmc_io_cmds(hba);
		}
	} else {
		if (ccb)
			mmc_error(hba, CAM_PROVIDE_FAIL);

		if( ext->eflags & MMC_EFLAG_RESETTING ) {	// failure trying to power up/ident, retry via pulse handler
			MsgSendPulse( mmc_ctrl.coid, MMC_PRIORITY, MMCSD_PULSE_RESET, 0 );
			return;
		}

		ext->state = MMCSD_STATE_IDLE;

		if (ext->detect(hba) == MMC_SUCCESS) {
			// unsupported card
			slogf(_SLOGC_SIM_MMC, _SLOG_ERROR, "Unsupported card inserted!");
			atomic_set(&ext->eflags, MMC_EFLAG_INVALID_CARD);
			ext->powerdown(hba);	// powerdown the card
		} else {
			// card removed before ident is done
			mmc_media_check(hba);
		}
	}
}

static void mmc_data_irq(SIM_HBA *hba, int intr)
{
	SIM_MMC_EXT	*ext = (SIM_MMC_EXT *)hba->ext;
	CCB_SCSIIO	*ccb = ext->nexus;
	static int 	stops_retry=0;

#ifdef MMC_BKOPS_SUP
	if( ( ext->mmc_resp[0] & MMC_URGENT_BKOPS ) ) {
		if( hba->verbosity ) {
			mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s: bkops status change %d to %d", __FUNCTION__, ext->bkops_status, EXT_CSD_BS_OPERATIONS_CRITICAL );
		}
		ext->bkops_status = EXT_CSD_BS_OPERATIONS_CRITICAL;
	}
#endif

	if (ext->stop == MMCSD_STOP_ISSUED) {			// stop command issued ?
		if (intr & MMC_INTR_ERROR) {				// error ?
			slogf(_SLOGC_SIM_MMC, _SLOG_ERROR,
					"%s: stop command error %x", __FUNCTION__, intr);

			mmc_media_check(hba);
			if(!(ext->eflags & MMC_EFLAG_INSERTED)){
				 mmc_error(hba, MMC_NOT_PRESENT);
				 stops_retry = 0;
				 ext->stop = MMCSD_STOP_NONE;
				 return;
			}

			if(stops_retry++>5 || (mmc_stop_xfer(hba) != MMC_SUCCESS)){
					// try to reset/reinit, we set ident state so
					// we don't try to start a new request
				ext->state = MMCSD_STATE_IDENT;
				mmc_error(hba, CAM_PROVIDE_FAIL);
				ext->stop = MMCSD_STOP_NONE;
				MsgSendPulse( mmc_ctrl.coid, MMC_PRIORITY, MMCSD_PULSE_RESET, 0 );
			}
		} else if (intr & MMC_INTR_COMMAND) {		// command complete ?
			stops_retry = 0;
			ext->stop = MMCSD_STOP_NONE;
			if (ccb->cam_scsi_status == SCS_CHECK) {
				mmc_error(hba, CAM_PROVIDE_FAIL);
			}
			else if (ext->eflags & MMC_EFLAG_PIO) {
				ccb->cam_ch.cam_status = CAM_REQ_CMP;
				ccb->cam_scsi_status   = SCS_GOOD;
			} else
				mmc_dma_irq(hba);
		}
	} else {
		stops_retry = 0;
		if (intr & MMC_INTR_ERROR) {
			slogf(_SLOGC_SIM_MMC, _SLOG_ERROR,
					"%s: data error %x", __FUNCTION__, intr);

			mmc_media_check(hba);
			if(!(ext->eflags & MMC_EFLAG_INSERTED)){
				mmc_error(hba, MMC_NOT_PRESENT);
				ext->stop = MMCSD_STOP_NONE;
				return;
			}

			if (mmc_stop_xfer(hba) != MMC_SUCCESS) {
				mmc_error(hba, CAM_PROVIDE_FAIL);
				ext->stop = MMCSD_STOP_NONE;
				return;
			}

			ext->stop = MMCSD_STOP_ISSUED;
				// set check condition as a flag for us to mark the
				// request as errored when the stop completes.
			ccb->cam_scsi_status = SCS_CHECK;
		} else if (intr & MMC_INTR_DATA) {
			if (ext->eflags & MMC_EFLAG_PIO)
				return mmc_pio_rw(hba, intr);

			if (ext->dma_done(hba, ext->dir) == MMC_SUCCESS)
				return mmc_dma_irq(hba);

			mmc_error(hba, CAM_PROVIDE_FAIL);
			slogf(_SLOGC_SIM_MMC, _SLOG_ERROR,
					"%s: data error %x", __FUNCTION__, intr);
		} else if (intr & (MMC_INTR_RBRDY | MMC_INTR_WBRDY))
			mmc_pio_rw(hba, intr);
	}
}

static void mmc_media_check(SIM_HBA *hba)
{
	SIM_MMC_EXT	*ext = (SIM_MMC_EXT *)hba->ext;

	if (ext->detect(hba) == MMC_SUCCESS) {
		if (!(atomic_set_value(&ext->eflags, MMC_EFLAG_INSERTED) & MMC_EFLAG_INSERTED)) {
			if(ext->normval)
				atomic_set(&ext->eflags, MMC_EFLAG_MEDIA_CHANGED);
			// Insertion
			if (hba->verbosity)
				slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMCSD : card inserted");
			ext->media_change++;
			MsgSendPulse(mmc_ctrl.coid, ext->mmc_priority, MMCSD_PULSE_CARD_INSERTION, 0);
		}
	} else {
		if (atomic_clr_value(&ext->eflags, MMC_EFLAG_INSERTED) & MMC_EFLAG_INSERTED) {
			if(ext->normval)
				atomic_set(&ext->eflags, MMC_EFLAG_MEDIA_CHANGED);
			// Removal
			if (hba->verbosity)
				slogf(_SLOGC_SIM_MMC, _SLOG_INFO, "MMCSD : card removed");
			atomic_clr(&ext->eflags, MMC_EFLAG_READY | MMC_EFLAG_INVALID_CARD);
			ext->powerdown(hba);	// powerdown the card
			ext->media_change++;
			MsgSendPulse(mmc_ctrl.coid, ext->mmc_priority, MMCSD_PULSE_CARD_REMOVAL, 0);
		}
	}
}

static int mmc_process_interrupt(SIM_HBA *hba, int irq)
{
	SIM_MMC_EXT	*ext = (SIM_MMC_EXT *)hba->ext;
	CCB_SCSIIO	*ccb = ext->nexus;
	uint32_t	intr;

	if ((intr = ext->interrupt(hba, irq, ext->resp_type, ext->mmc_resp)) == 0)
		return 0;

	if (intr & MMC_INTR_CARD) {
		mmc_media_check(hba);

		mmc_error(hba, CAM_PROVIDE_FAIL);
	} else {
		switch (ext->state) {
		case MMCSD_STATE_IDENT:
			mmc_ident_irq(hba, intr);
			break;
		case MMCSD_STATE_DATA:
			mmc_data_irq(hba, intr);
			if(ccb && ccb->cam_ch.cam_status != CAM_REQ_INPROG){
                if((ext->eflags & MMC_EFLAG_AC13)
                    && ccb && (ccb->cam_ch.cam_status == CAM_REQ_CMP)
                    && (ccb->cam_cdb_io.cam_cdb_bytes[0] == SC_READ10)
                    && (ext->nbytes == ext->mmc_blksize)) {

                    if (irq == hba->cfg.IRQRegisters[0])
                        InterruptUnmask(hba->cfg.IRQRegisters[0], hba->iid);

                    if (mmc_get_card_status(hba)!=MMC_SUCCESS)
                        mmc_error(hba, CAM_PROVIDE_FAIL);

                    if (irq == hba->cfg.IRQRegisters[0])
                        InterruptMask(hba->cfg.IRQRegisters[0], hba->iid);
				}

                /*
                 * NOTE: mmc_wait_card_ready() assumes the hba->cfg.IRQRegisters[0] is unmasked,
                 * but this could not be always be true.
                 */
				if(mmc_wait_card_ready(hba)!=MMC_SUCCESS){
					mmc_error(hba, CAM_PROVIDE_FAIL);
				}
			}

			break;
		default:
			slogf(_SLOGC_SIM_MMC, _SLOG_INFO,
				"MMCSD : unexpected interrupt, state = %d, intr status = %x, cmd %x", ext->state, intr, ext->cmd.opcode);
			break;
		}
	}

	if (ccb) {
		if (ccb->cam_ch.cam_status != CAM_REQ_INPROG) {
			if( hba->verbosity > 3 ) {
				mmc_slogf( hba, _SLOGC_SIM_MMC, _SLOG_ERROR, 1, "%s:  %p, cstatus %x", __FUNCTION__, ccb, ccb->cam_ch.cam_status );
			}
#ifdef MMC_TRACE
			mmc_trace_event( MMC_TRACE_EVENT, "mmc post ccb %p", ccb );
#endif
			if( ext->state != MMCSD_STATE_IDENT ) {
				ext->state = MMCSD_STATE_IDLE;
			}

			mmc_post_ccb( hba, ccb );
		}
	}

	return 0;
}

// the driver thread starts here
static inline void mmc_pulse_handler(SIM_HBA *hba)
{
	SIM_MMC_EXT		*ext;
	struct _pulse	pulse;
	int				rid;

	ext = (SIM_MMC_EXT *)hba->ext;

	while (1) {
		if( ( rid = MsgReceivePulse( hba->chid, &pulse, sizeof( pulse ), NULL ) ) == -1 ) {
			continue;
		}

		switch (pulse.code) {
			case SIM_ENQUEUE:
				mmc_start_ccb(hba);
				break;

			case SIM_INTERRUPT:
				mmc_process_interrupt(hba, hba->cfg.IRQRegisters[0]);
				InterruptUnmask(hba->cfg.IRQRegisters[0], hba->iid);
				break;

			case SIM_DMA_INTERRUPT:
				mmc_process_interrupt(hba, hba->cfg.IRQRegisters[1]);
				if(hba->cfg.NumIRQs > 1) {
					InterruptUnmask(hba->cfg.IRQRegisters[1], ext->iid);
				}
				break;

			case SIM_TIMER:
#ifdef MMC_BKOPS_SUP
				mmc_bkops( hba, CAM_TRUE );
#endif

#define MMC_PM_IDLE_TICKS	1//2
				if( ext->nexus == NULL && ++ext->idle_ticks == MMC_PM_IDLE_TICKS ) {
					mmc_pm( hba, MMC_PM_IDLE );
				}

				// Only check media if card detection interrupt is not supported
				if( !( ext->hccap & MMC_HCCAP_CD_INTR ) ) {
					mmc_media_check( hba );
					if( !( ext->eflags & MMC_EFLAG_INSERTED ) && ext->nexus ) {
						mmc_error( hba, MMC_NOT_PRESENT );
						mmc_post_ccb( hba, ext->nexus );
					}
				}
				break;

			default:
				break;
		}

		if (ext->nexus == NULL)
			mmc_start_ccb(hba);
	}
}

void *mmc_driver_thread(void *data)
{ 
	SIM_HBA			*hba;
	SIM_MMC_EXT		*ext;
	struct sigevent	event;

	hba = (SIM_HBA *)data;
	ext = (SIM_MMC_EXT *)hba->ext;

#if (defined(_NTO_VERSION) && (_NTO_VERSION >= 632))
		/* set the interrupt handler thread name */
		pthread_setname_np( NULL, "mmc_pulse_handler" );
#endif

	if (hba->cfg.NumIRQs) {
		SIGEV_PULSE_INIT(&event, hba->coid, ext->mmc_irq_priority, SIM_INTERRUPT, NULL);

		if ((hba->iid = InterruptAttachEvent(hba->cfg.IRQRegisters[0], &event, _NTO_INTR_FLAGS_TRK_MSK)) == -1)
			perror("Interrupt attach failed:");

		if (hba->cfg.NumIRQs > 1) {
			SIGEV_PULSE_INIT(&event, hba->coid, ext->mmc_irq_priority, SIM_DMA_INTERRUPT, NULL);

			if ((ext->iid = InterruptAttachEvent(hba->cfg.IRQRegisters[1], &event, _NTO_INTR_FLAGS_TRK_MSK)) == -1)
				perror("Interrupt attach failed:");
		}
	}

	atomic_set(&ext->eflags, MMC_EFLAG_DRIVER_READY);

	while (1) {
		if (ext->eflags & MMC_EFLAG_MONITOR_READY)
			break;
		delay(1);
	}

	mmc_pulse_handler(hba);

	return (CAM_SUCCESS);
}

#if defined(__QNXNTO__) && defined(__USESRCVERSION)
#include <sys/srcversion.h>
__SRCVERSION("$URL: http://svn/product/branches/6.6.0/trunk/hardware/devb/mmcsd/sim_mmc.c $ $Rev: 736700 $")
#endif
