/*
 * $QNXLicenseC: 
 * Copyright 2010, QNX Software Systems.  
 *  
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in  
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,  
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied. 
 * 
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at  
 * http://licensing.qnx.com/license-guide/ for other information. 
 * $
 */


#ifndef	__ARM_MX51_IOMUX_H_INCLUDED
#define	__ARM_MX51_IOMUX_H_INCLUDED
#include <stdint.h>
#include <sys/srcversion.h>

/*
 * offsets of IOMUXC registers from MX51_IOMUX_SWMUX
 * where MX51_IOMUX_SWMUX = MX51_IOMUXC_BASE + 0x001C
 */

#define	SWMUX_EIM_DA0	0
#define	SWMUX_EIM_DA1	1
#define	SWMUX_EIM_DA2	2
#define	SWMUX_EIM_DA3	3
#define	SWMUX_EIM_DA4	4
#define	SWMUX_EIM_DA5	5
#define	SWMUX_EIM_DA6	6
#define	SWMUX_EIM_DA7	7
#define	SWMUX_EIM_DA8	8
#define	SWMUX_EIM_DA9	9
#define	SWMUX_EIM_DA10	10
#define	SWMUX_EIM_DA11	11
#define	SWMUX_EIM_DA12	12
#define	SWMUX_EIM_DA13	13
#define	SWMUX_EIM_DA14	14
#define	SWMUX_EIM_DA15	15
#define	SWMUX_EIM_D16	16
#define	SWMUX_EIM_D17	17
#define	SWMUX_EIM_D18	18
#define	SWMUX_EIM_D19	19
#define	SWMUX_EIM_D20	20
#define	SWMUX_EIM_D21	21
#define	SWMUX_EIM_D22	22
#define	SWMUX_EIM_D23	23
#define	SWMUX_EIM_D24	24
#define	SWMUX_EIM_D25	25
#define	SWMUX_EIM_D26	26
#define	SWMUX_EIM_D27	27
#define	SWMUX_EIM_D28	28
#define	SWMUX_EIM_D29	29
#define	SWMUX_EIM_D30	30
#define	SWMUX_EIM_D31	31
#define	SWMUX_EIM_A16	32
#define	SWMUX_EIM_A17	33
#define	SWMUX_EIM_A18	34
#define	SWMUX_EIM_A19	35
#define	SWMUX_EIM_A20	36
#define	SWMUX_EIM_A21	37
#define	SWMUX_EIM_A22	38
#define	SWMUX_EIM_A23	39
#define	SWMUX_EIM_A24	40
#define	SWMUX_EIM_A25	41
#define	SWMUX_EIM_A26	42
#define	SWMUX_EIM_A27	43
#define	SWMUX_EIM_EB0	44
#define	SWMUX_EIM_EB1	45
#define	SWMUX_EIM_EB2	46
#define	SWMUX_EIM_EB3	47
#define	SWMUX_EIM_OE	48
#define	SWMUX_EIM_CS0	49
#define	SWMUX_EIM_CS1	50
#define	SWMUX_EIM_CS2	51
#define	SWMUX_EIM_CS3	52
#define	SWMUX_EIM_CS4	53
#define	SWMUX_EIM_CS5	54
#define	SWMUX_EIM_DTACK	55
#define	SWMUX_EIM_LBA	56
#define	SWMUX_EIM_CRE	57
#define	SWMUX_DRAM_CS1	58
#define	SWMUX_NANDF_WE_B	59
#define	SWMUX_NANDF_RE_B	60
#define	SWMUX_NANDF_ALE	61
#define	SWMUX_NANDF_CLE	62
#define	SWMUX_NANDF_WP_B	63
#define	SWMUX_NANDF_RB0	64
#define	SWMUX_NANDF_RB1	65
#define	SWMUX_NANDF_RB2	66
#define	SWMUX_NANDF_RB3	67
#define	SWMUX_GPIO_NAND	68
#define	SWMUX_NANDF_CS0	69
#define	SWMUX_NANDF_CS1	70
#define	SWMUX_NANDF_CS2	71
#define	SWMUX_NANDF_CS3	72
#define	SWMUX_NANDF_CS4	73
#define	SWMUX_NANDF_CS5	74
#define	SWMUX_NANDF_CS6	75
#define	SWMUX_NANDF_CS7	76
#define	SWMUX_NANDF_RDY_INT	77
#define	SWMUX_NANDF_D15	78
#define	SWMUX_NANDF_D14	79
#define	SWMUX_NANDF_D13	80
#define	SWMUX_NANDF_D12	81
#define	SWMUX_NANDF_D11	82
#define	SWMUX_NANDF_D10	83
#define	SWMUX_NANDF_D9	84
#define	SWMUX_NANDF_D8	85
#define	SWMUX_NANDF_D7	86
#define	SWMUX_NANDF_D6	87
#define	SWMUX_NANDF_D5	88
#define	SWMUX_NANDF_D4	89
#define	SWMUX_NANDF_D3	90
#define	SWMUX_NANDF_D2	91
#define	SWMUX_NANDF_D1	92
#define	SWMUX_NANDF_D0	93
#define	SWMUX_CSI1_D8	94
#define	SWMUX_CSI1_D9	95
#define	SWMUX_CSI1_D10	96
#define	SWMUX_CSI1_D11	97
#define	SWMUX_CSI1_D12	98
#define	SWMUX_CSI1_D13	99
#define	SWMUX_CSI1_D14	100
#define	SWMUX_CSI1_D15	101
#define	SWMUX_CSI1_D16	102
#define	SWMUX_CSI1_D17	103
#define	SWMUX_CSI1_D18	104
#define	SWMUX_CSI1_D19	105
#define	SWMUX_CSI1_VSYNC	106
#define	SWMUX_CSI1_HSYNC	107
#define	SWMUX_CSI2_D12	108
#define	SWMUX_CSI2_D13	109
#define	SWMUX_CSI2_D14	110
#define	SWMUX_CSI2_D15	111
#define	SWMUX_CSI2_D16	112
#define	SWMUX_CSI2_D17	113
#define	SWMUX_CSI2_D18	114
#define	SWMUX_CSI2_D19	115
#define	SWMUX_CSI2_VSYNC	116
#define	SWMUX_CSI2_HSYNC	117
#define	SWMUX_CSI2_PIXCLK	118
#define	SWMUX_I2C1_CLK	119
#define	SWMUX_I2C1_DAT	120
#define	SWMUX_AUD3_BB_TXD	121
#define	SWMUX_AUD3_BB_RXD	122
#define	SWMUX_AUD3_BB_CK	123
#define	SWMUX_AUD3_BB_FS	124
#define	SWMUX_CSPI1_MOSI	125
#define	SWMUX_CSPI1_MISO	126
#define	SWMUX_CSPI1_SS0	127
#define	SWMUX_CSPI1_SS1	128
#define	SWMUX_CSPI1_RDY	129
#define	SWMUX_CSPI1_SCLK	130
#define	SWMUX_UART1_RXD	131
#define	SWMUX_UART1_TXD	132
#define	SWMUX_UART1_RTS	133
#define	SWMUX_UART1_CTS	134
#define	SWMUX_UART2_RXD	135
#define	SWMUX_UART2_TXD	136
#define	SWMUX_UART3_RXD	137
#define	SWMUX_UART3_TXD	138
#define	SWMUX_OWIRE_LINE	139
#define	SWMUX_KEY_ROW0	140
#define	SWMUX_KEY_ROW1	141
#define	SWMUX_KEY_ROW2	142
#define	SWMUX_KEY_ROW3	143
#define	SWMUX_KEY_COL0	144
#define	SWMUX_KEY_COL1	145
#define	SWMUX_KEY_COL2	146
#define	SWMUX_KEY_COL3	147
#define	SWMUX_KEY_COL4	148
#define	SWMUX_KEY_COL5	149
#define	SWMUX_JTAG_DE_B	150
#define	SWMUX_USBH1_CLK	151
#define	SWMUX_USBH1_DIR	152
#define	SWMUX_USBH1_STP	153
#define	SWMUX_USBH1_NXT	154
#define	SWMUX_USBH1_DATA0	155
#define	SWMUX_USBH1_DATA1	156
#define	SWMUX_USBH1_DATA2	157
#define	SWMUX_USBH1_DATA3	158
#define	SWMUX_USBH1_DATA4	159
#define	SWMUX_USBH1_DATA5	160
#define	SWMUX_USBH1_DATA6	161
#define	SWMUX_USBH1_DATA7	162
#define	SWMUX_DI1_PIN11	163
#define	SWMUX_DI1_PIN12	164
#define	SWMUX_DI1_PIN13	165
#define	SWMUX_DI1_D0_CS	166
#define	SWMUX_DI1_D1_CS	167
#define	SWMUX_DISPB2_SER_DIN	168
#define	SWMUX_DISPB2_SER_DIO	169
#define	SWMUX_DISPB2_SER_CLK	170
#define	SWMUX_DISPB2_SER_RS	171
#define	SWMUX_DISP1_DAT0	172
#define	SWMUX_DISP1_DAT1	173
#define	SWMUX_DISP1_DAT2	174
#define	SWMUX_DISP1_DAT3	175
#define	SWMUX_DISP1_DAT4	176
#define	SWMUX_DISP1_DAT5	177
#define	SWMUX_DISP1_DAT6	178
#define	SWMUX_DISP1_DAT7	179
#define	SWMUX_DISP1_DAT8	180
#define	SWMUX_DISP1_DAT9	181
#define	SWMUX_DISP1_DAT10	182
#define	SWMUX_DISP1_DAT11	183
#define	SWMUX_DISP1_DAT12	184
#define	SWMUX_DISP1_DAT13	185
#define	SWMUX_DISP1_DAT14	186
#define	SWMUX_DISP1_DAT15	187
#define	SWMUX_DISP1_DAT16	188
#define	SWMUX_DISP1_DAT17	189
#define	SWMUX_DISP1_DAT18	190
#define	SWMUX_DISP1_DAT19	191
#define	SWMUX_DISP1_DAT20	192
#define	SWMUX_DISP1_DAT21	193
#define	SWMUX_DISP1_DAT22	194
#define	SWMUX_DISP1_DAT23	195
#define	SWMUX_DI1_PIN3		196
#define	SWMUX_DI1_PIN2		197
#define	SWMUX_DI_GP1		198
#define	SWMUX_DI_GP2		199
#define	SWMUX_DI_GP3		200
#define	SWMUX_DI2_PIN4		201
#define	SWMUX_DI2_PIN2		202
#define	SWMUX_DI2_PIN3		203
#define	SWMUX_DI2_DISP_CLK	204
#define	SWMUX_DI_GP4		205
#define	SWMUX_DISP2_DAT0	206
#define	SWMUX_DISP2_DAT1	207
#define	SWMUX_DISP2_DAT2	208
#define	SWMUX_DISP2_DAT3	209
#define	SWMUX_DISP2_DAT4	210
#define	SWMUX_DISP2_DAT5	211
#define	SWMUX_DISP2_DAT6	212
#define	SWMUX_DISP2_DAT7	213
#define	SWMUX_DISP2_DAT8	214
#define	SWMUX_DISP2_DAT9	215
#define	SWMUX_DISP2_DAT10	216
#define	SWMUX_DISP2_DAT11	217
#define	SWMUX_DISP2_DAT12	218
#define	SWMUX_DISP2_DAT13	219
#define	SWMUX_DISP2_DAT14	220
#define	SWMUX_DISP2_DAT15	221
#define	SWMUX_SD1_CMD		222
#define	SWMUX_SD1_CLK		223
#define	SWMUX_SD1_DATA0		224
#define	SWMUX_SD1_DATA1		225
#define	SWMUX_SD1_DATA2		226
#define	SWMUX_SD1_DATA3		227
#define	SWMUX_GPIO1_0		228
#define	SWMUX_GPIO1_1		229
#define	SWMUX_SD2_CMD		230
#define	SWMUX_SD2_CLK		231
#define	SWMUX_SD2_DATA0		232
#define	SWMUX_SD2_DATA1		233
#define	SWMUX_SD2_DATA2		234
#define	SWMUX_SD2_DATA3		235
#define	SWMUX_GPIO1_2		236
#define	SWMUX_GPIO1_3		237
#define	SWMUX_PMIC_INT_REQ	238
#define	SWMUX_GPIO1_4		239
#define	SWMUX_GPIO1_5		240
#define	SWMUX_GPIO1_6		241
#define	SWMUX_GPIO1_7		242
#define	SWMUX_GPIO1_8		243
#define	SWMUX_GPIO1_9		244

/*
 * offsets of IOMUXC registers from MX51_IOMUX_SWPAD
 * where MX51_IOMUX_SWPAD = MX51_IOMUXC_BASE + 0x03F0
 */

#define	SWPAD_EIM_D16		0
#define	SWPAD_EIM_D17		1
#define	SWPAD_EIM_D18		2
#define	SWPAD_EIM_D19		3
#define	SWPAD_EIM_D20		4
#define	SWPAD_EIM_D21		5
#define	SWPAD_EIM_D22		6
#define	SWPAD_EIM_D23		7
#define	SWPAD_EIM_D24		8
#define	SWPAD_EIM_D25		9
#define	SWPAD_EIM_D26		10
#define	SWPAD_EIM_D27		11
#define	SWPAD_EIM_D28		12
#define	SWPAD_EIM_D29		13
#define	SWPAD_EIM_D30		14
#define	SWPAD_EIM_D31		15
#define	SWPAD_EIM_A16		16
#define	SWPAD_EIM_A17		17
#define	SWPAD_EIM_A18		18
#define	SWPAD_EIM_A19		19
#define	SWPAD_EIM_A20		20
#define	SWPAD_EIM_A21		21
#define	SWPAD_EIM_A22		22
#define	SWPAD_EIM_A23		23
#define	SWPAD_EIM_A24		24
#define	SWPAD_EIM_A25		25
#define	SWPAD_EIM_A26		26
#define	SWPAD_EIM_A27		27
#define	SWPAD_EIM_EB0		28
#define	SWPAD_EIM_EB1		29
#define	SWPAD_EIM_EB2		30
#define	SWPAD_EIM_EB3		31
#define	SWPAD_EIM_OE		32
#define	SWPAD_EIM_CS0		33
#define	SWPAD_EIM_CS1		34
#define	SWPAD_EIM_CS2		35
#define	SWPAD_EIM_CS3		36
#define	SWPAD_EIM_CS4		37
#define	SWPAD_EIM_CS5		38
#define	SWPAD_EIM_DTACK		39
#define	SWPAD_EIM_WAIT		40
#define	SWPAD_EIM_LBA		41
#define	SWPAD_EIM_BCLK		42
#define	SWPAD_EIM_RW		43
#define	SWPAD_EIM_CRE		44
#define	SWPAD_DRAM_RAS		45
#define	SWPAD_DRAM_CAS		46
#define	SWPAD_DRAM_SDWE		47
#define	SWPAD_DRAM_SDCKE0	48
#define	SWPAD_DRAM_SDCKE1	49
#define	SWPAD_DRAM_SDCLK	50
#define	SWPAD_DRAM_SDQS0	51
#define	SWPAD_DRAM_SDQS1	52
#define	SWPAD_DRAM_SDQS2	53
#define	SWPAD_DRAM_SDQS3	54
#define	SWPAD_DRAM_CS0		55
#define	SWPAD_DRAM_CS1		56
#define	SWPAD_DRAM_DQM0		57
#define	SWPAD_DRAM_DQM1		58
#define	SWPAD_DRAM_DQM2		59
#define	SWPAD_DRAM_DQM3		60
#define	SWPAD_NANDF_WE_B	61
#define	SWPAD_NANDF_RE_B	62
#define	SWPAD_NANDF_ALE		63
#define	SWPAD_NANDF_CLE		64
#define	SWPAD_NANDF_WP_B	65
#define	SWPAD_NANDF_RB0		66
#define	SWPAD_NANDF_RB1		67
#define	SWPAD_NANDF_RB2		68
#define	SWPAD_NANDF_RB3		69
#define	SWPAD_EIM_SDBA2		70
#define	SWPAD_EIM_SDODT1	71
#define	SWPAD_EIM_SDODT0	72
#define	SWPAD_GPIO_NAND		73
#define	SWPAD_NANDF_CS0		74
#define	SWPAD_NANDF_CS1		75
#define	SWPAD_NANDF_CS2		76
#define	SWPAD_NANDF_CS3		77
#define	SWPAD_NANDF_CS4		78
#define	SWPAD_NANDF_CS5		79
#define	SWPAD_NANDF_CS6		80
#define	SWPAD_NANDF_CS7		81
#define	SWPAD_NANDF_RDY_INT	82
#define	SWPAD_NANDF_D15		83
#define	SWPAD_NANDF_D14		84
#define	SWPAD_NANDF_D13		85
#define	SWPAD_NANDF_D12		86
#define	SWPAD_NANDF_D11		87
#define	SWPAD_NANDF_D10		88
#define	SWPAD_NANDF_D9		89
#define	SWPAD_NANDF_D8		90
#define	SWPAD_NANDF_D7		91
#define	SWPAD_NANDF_D6		92
#define	SWPAD_NANDF_D5		93
#define	SWPAD_NANDF_D4		94
#define	SWPAD_NANDF_D3		95
#define	SWPAD_NANDF_D2		96
#define	SWPAD_NANDF_D1		97
#define	SWPAD_NANDF_D0		98
#define	SWPAD_CSI1_D8		99
#define	SWPAD_CSI1_D9		100
#define	SWPAD_CSI1_D10		101
#define	SWPAD_CSI1_D11		102
#define	SWPAD_CSI1_D12		103
#define	SWPAD_CSI1_D13		104
#define	SWPAD_CSI1_D14		105
#define	SWPAD_CSI1_D15		106
#define	SWPAD_CSI1_D16		107
#define	SWPAD_CSI1_D17		108
#define	SWPAD_CSI1_D18		109
#define	SWPAD_CSI1_D19		110
#define	SWPAD_CSI1_VSYNC	111
#define	SWPAD_CSI1_HSYNC	112
#define	SWPAD_CSI1_PIXCLK	113
#define	SWPAD_CSI1_MCLK		114
#define	SWPAD_CSI2_D12		115
#define	SWPAD_CSI2_D13		116
#define	SWPAD_CSI2_D14		117
#define	SWPAD_CSI2_D15		118
#define	SWPAD_CSI2_D16		119
#define	SWPAD_CSI2_D17		120
#define	SWPAD_CSI2_D18		121
#define	SWPAD_CSI2_D19		122
#define	SWPAD_CSI2_VSYNC	123
#define	SWPAD_CSI2_HSYNC	124
#define	SWPAD_CSI2_PIXCLK	125
#define	SWPAD_I2C1_CLK		126
#define	SWPAD_I2C1_DAT		127
#define	SWPAD_AUD3_BB_TXD	128
#define	SWPAD_AUD3_BB_RXD	129
#define	SWPAD_AUD3_BB_CK	130
#define	SWPAD_AUD3_BB_FS	131
#define	SWPAD_CSPI1_MOSI	132
#define	SWPAD_CSPI1_MISO	133
#define	SWPAD_CSPI1_SS0		134
#define	SWPAD_CSPI1_SS1		135
#define	SWPAD_CSPI1_RDY		136
#define	SWPAD_CSPI1_SCLK	137
#define	SWPAD_UART1_RXD		138
#define	SWPAD_UART1_TXD		139
#define	SWPAD_UART1_RTS		140
#define	SWPAD_UART1_CTS		141
#define	SWPAD_UART2_RXD		142
#define	SWPAD_UART2_TXD		143
#define	SWPAD_UART3_RXD		144
#define	SWPAD_UART3_TXD		145
#define	SWPAD_OWIRE_LINE	146
#define	SWPAD_KEY_ROW0		147
#define	SWPAD_KEY_ROW1		148
#define	SWPAD_KEY_ROW2		149
#define	SWPAD_KEY_ROW3		150
#define	SWPAD_KEY_COL0		151
#define	SWPAD_KEY_COL1		152
#define	SWPAD_KEY_COL2		153
#define	SWPAD_KEY_COL3		154
#define	SWPAD_KEY_COL4		155
#define	SWPAD_KEY_COL5		156
#define	SWPAD_JTAG_TCK		157
#define	SWPAD_JTAG_TMS		158
#define	SWPAD_JTAG_TDI		159
#define	SWPAD_JTAG_TRSTB	160
#define	SWPAD_JTAG_MOD		161
#define	SWPAD_USBH1_CLK		162
#define	SWPAD_USBH1_DIR		163
#define	SWPAD_USBH1_STP		164
#define	SWPAD_USBH1_NXT		165
#define	SWPAD_USBH1_DATA0		166
#define	SWPAD_USBH1_DATA1		167
#define	SWPAD_USBH1_DATA2		168
#define	SWPAD_USBH1_DATA3		169
#define	SWPAD_USBH1_DATA4		170
#define	SWPAD_USBH1_DATA5		171
#define	SWPAD_USBH1_DATA6		172
#define	SWPAD_USBH1_DATA7		173
#define	SWPAD_DI1_PIN11		174
#define	SWPAD_DI1_PIN12		175
#define	SWPAD_DI1_PIN13		176
#define	SWPAD_DI1_D0_CS		177
#define	SWPAD_DI1_D1_CS		178
#define	SWPAD_DISPB2_SER_DIN		179
#define	SWPAD_DISPB2_SER_DIO		180
#define	SWPAD_DISPB2_SER_CLK		181
#define	SWPAD_DISPB2_SER_RS		182
#define	SWPAD_DISP1_DAT0		183
#define	SWPAD_DISP1_DAT1		184
#define	SWPAD_DISP1_DAT2		185
#define	SWPAD_DISP1_DAT3		186
#define	SWPAD_DISP1_DAT4		187
#define	SWPAD_DISP1_DAT5		188
#define	SWPAD_DISP1_DAT6		189
#define	SWPAD_DISP1_DAT7		190
#define	SWPAD_DISP1_DAT8		191
#define	SWPAD_DISP1_DAT9		192
#define	SWPAD_DISP1_DAT10		193
#define	SWPAD_DISP1_DAT11		194
#define	SWPAD_DISP1_DAT12		195
#define	SWPAD_DISP1_DAT13		196
#define	SWPAD_DISP1_DAT14		197
#define	SWPAD_DISP1_DAT15		198
#define	SWPAD_DISP1_DAT16		199
#define	SWPAD_DISP1_DAT17		200
#define	SWPAD_DISP1_DAT18		201
#define	SWPAD_DISP1_DAT19		202
#define	SWPAD_DISP1_DAT20		203
#define	SWPAD_DISP1_DAT21		204
#define	SWPAD_DISP1_DAT22		205
#define	SWPAD_DISP1_DAT23		206
#define	SWPAD_DI1_PIN3		207
#define	SWPAD_DI1_DISP_CLK		208
#define	SWPAD_DI1_PIN2		209
#define	SWPAD_DI1_PIN15		210
#define	SWPAD_DI_GP1		211
#define	SWPAD_DI_GP2		212
#define	SWPAD_DI_GP3		213
#define	SWPAD_DI2_PIN4		214
#define	SWPAD_DI2_PIN2		215
#define	SWPAD_DI2_PIN3		216
#define	SWPAD_DI2_DISP_CLK		217
#define	SWPAD_DI_GP4		218
#define	SWPAD_DISP2_DAT0		219
#define	SWPAD_DISP2_DAT1		220
#define	SWPAD_DISP2_DAT2		221
#define	SWPAD_DISP2_DAT3		222
#define	SWPAD_DISP2_DAT4		223
#define	SWPAD_DISP2_DAT5		224
#define	SWPAD_DISP2_DAT6		225
#define	SWPAD_DISP2_DAT7		226
#define	SWPAD_DISP2_DAT8		227
#define	SWPAD_DISP2_DAT9		228
#define	SWPAD_DISP2_DAT10		229
#define	SWPAD_DISP2_DAT11		230
#define	SWPAD_DISP2_DAT12		231
#define	SWPAD_DISP2_DAT13		232
#define	SWPAD_DISP2_DAT14		233
#define	SWPAD_DISP2_DAT15		234
#define	SWPAD_SD1_CMD		235
#define	SWPAD_SD1_CLK		236
#define	SWPAD_SD1_DATA0		237
#define	SWPAD_SD1_DATA1		238
#define	SWPAD_SD1_DATA2		239
#define	SWPAD_SD1_DATA3		240
#define	SWPAD_GPIO1_0		241
#define	SWPAD_GPIO1_1		242
#define	SWPAD_SD2_CMD		243
#define	SWPAD_SD2_CLK		244
#define	SWPAD_SD2_DATA0		245
#define	SWPAD_SD2_DATA1		246
#define	SWPAD_SD2_DATA2		247
#define	SWPAD_SD2_DATA3		248
#define	SWPAD_GPIO1_2		249
#define	SWPAD_GPIO1_3		250
#define	SWPAD_RESET_IN_B		251
#define	SWPAD_POR_B		252
#define	SWPAD_BOOT_MODE1		253
#define	SWPAD_BOOT_MODE0		254
#define	SWPAD_PMIC_RDY		255
#define	SWPAD_CKIL		256
#define	SWPAD_PMIC_STBY_REQ		257
#define	SWPAD_PMIC_ON_REQ		258
#define	SWPAD_PMIC_INT_REQ		259
#define	SWPAD_CLK_SS		260
#define	SWPAD_GPIO1_4		261
#define	SWPAD_GPIO1_5		262
#define	SWPAD_GPIO1_6		263
#define	SWPAD_GPIO1_7		264
#define	SWPAD_GPIO1_8		265
#define	SWPAD_GPIO1_9		266
#define	PAD_GRP_CSI2_PKE0		267
#define	PAD_GRP_DDRPKS		268
#define	PAD_GRP_EIM_SR1		269
#define	PAD_GRP_DISP2_PKE0		270
#define	PAD_GRP_DRAM_B4		271
#define	PAD_GRP_INDDR		272
#define	PAD_GRP_EIM_SR2		273
#define	PAD_GRP_PKEDDR		274
#define	PAD_GRP_DDR_A0		275
#define	PAD_GRP_EMI_PKE0		276
#define	PAD_GRP_EIM_SR3		277
#define	PAD_GRP_DDR_A1		278
#define	PAD_GRP_DDRAPUS		279
#define	PAD_GRP_EIM_SR4		280
#define	PAD_GRP_EMI_SR5		281
#define	PAD_GRP_EMI_SR6		282
#define	PAD_GRP_HYSDDR0		283
#define	PAD_GRP_CSI1_PKE0		284
#define	PAD_GRP_HYSDD1		285
#define	PAD_GRP_DISP1_PKE0		286
#define	PAD_GRP_HYSDDR2		287
#define	PAD_GRP_HVDDR		288
#define	PAD_GRP_HYSDDR3		289
#define	PAD_GRP_DRAM_SR_B0		290
#define	PAD_GRP_DDRAPKS		291
#define	PAD_GRP_DRAM_SR_B1		292
#define	PAD_GRP_DDRPUS		293
#define	PAD_GRP_EIM_DS1		294
#define	PAD_GRP_DRAM_SR_B2		295
#define	PAD_GRP_PKEADDR		296
#define	PAD_GRP_EIM_DS2		297
#define	PAD_GRP_EIM_DS3		298
#define	PAD_GRP_DRAM_SR_B4		299
#define	PAD_GRP_INMODE1		300
#define	PAD_GRP_DRAM_B0		301
#define	PAD_GRP_EIM_DS4		302
#define	PAD_GRP_DRAM_B1		303
#define	PAD_GRP_DDR_SR_A0		304
#define	PAD_GRP_EMI_DS5		305
#define	PAD_GRP_DRAM_B2		306
#define	PAD_GRP_DDR_SR_A1		307
#define	PAD_GRP_EMI_DS6		308

/*
 * offsets of IOMUXC registers from MX51_IOMUX_INPUT
 * where MX51_IOMUX_INPUT = MX51_IOMUXC_BASE + 0x08C4
 */

#define	SWINPUT_AUDMUX_P4_DA_AMX	0
#define	SWINPUT_AUDMUX_P4_DB_AMX	1
#define	SWINPUT_AUDMUX_P4_TXCLK_AMX	2
#define	SWINPUT_AUDMUX_P4_TXFS_AMX	3
#define	SWINPUT_AUDMUX_P5_DA_AMX	4
#define	SWINPUT_AUDMUX_P5_DB_AMX	5
#define	SWINPUT_AUDMUX_P5_RXCLK_AMX	6
#define	SWINPUT_AUDMUX_P5_RXFS_AMX	7
#define	SWINPUT_AUDMUX_P5_TXCLK_AMX	8
#define	SWINPUT_AUDMUX_P5_TXFS_AMX	9
#define	SWINPUT_AUDMUX_P6_DA_AMX	10
#define	SWINPUT_AUDMUX_P6_DB_AMX	11
#define	SWINPUT_AUDMUX_P6_RXCLK_AMX	12
#define	SWINPUT_AUDMUX_P6_RXFS_AMX	13
#define	SWINPUT_AUDMUX_P6_TXCLK_AMX	14
#define	SWINPUT_AUDMUX_P6_TXFS_AMX	15
#define	SWINPUT_CCM_IPP_DI0_CLK	16
#define	SWINPUT_CCM_IPP_DI1_CLK	17
#define	SWINPUT_CCM_PLL1_BYPASS_CLK	18
#define	SWINPUT_CCM_PLL2_BYPASS_CLK	19
#define	SWINPUT_CSPI_IPP_CSPI_CLK_IN	20
#define	SWINPUT_CSPI_IPP_IND_MISO	21
#define	SWINPUT_CSPI_IPP_IND_MOSI	22
#define	SWINPUT_CSPI_IPP_IND_SS1_B	23
#define	SWINPUT_CSPI_IPP_IND_SS2_B	24
#define	SWINPUT_CSPI_IPP_IND_SS3_B	25
#define	SWINPUT_DPLLIP1_L1T_TOG_EN	26
#define	SWINPUT_ECSPI2_IPP_IND_SS_B_1	27
#define	SWINPUT_ECSPI2_IPP_IND_SS_B_3	28
#define	SWINPUT_EMI_IPP_IND_RDY_INT	29
#define	SWINPUT_ESDHC3_IPP_DAT0_IN	30
#define	SWINPUT_ESDHC3_IPP_DAT1_IN	31
#define	SWINPUT_ESDHC3_IPP_DAT2_IN	32
#define	SWINPUT_ESDHC3_IPP_DAT3_IN	33
#define	SWINPUT_FEC_FEC_COL	34
#define	SWINPUT_FEC_FEC_CRS	35
#define	SWINPUT_FEC_FEC_MDI	36
#define	SWINPUT_FEC_FEC_RDATA_0	37
#define	SWINPUT_FEC_FEC_RDATA_1	38
#define	SWINPUT_FEC_FEC_RDATA_2	39
#define	SWINPUT_FEC_FEC_RDATA_3	40
#define	SWINPUT_FEC_FEC_RX_CLK	41
#define	SWINPUT_FEC_FEC_RX_DV	42
#define	SWINPUT_FEC_FEC_RX_ER	43
#define	SWINPUT_FEC_FEC_TX_CLK	44
#define	SWINPUT_GPIO3_IPP_IND_G_IN_1	45
#define	SWINPUT_GPIO3_IPP_IND_G_IN_2	46
#define	SWINPUT_GPIO3_IPP_IND_G_IN_3	47
#define	SWINPUT_GPIO3_IPP_IND_G_IN_4	48
#define	SWINPUT_GPIO3_IPP_IND_G_IN_5	49
#define	SWINPUT_GPIO3_IPP_IND_G_IN_6	50
#define	SWINPUT_GPIO3_IPP_IND_G_IN_7	51
#define	SWINPUT_GPIO3_IPP_IND_G_IN_8	52
#define	SWINPUT_GPIO3_IPP_IND_G_IN_12	53
#define	SWINPUT_HSC_MIPI_MIX_IPP_IND_SENS1_DATA_EN	54
#define	SWINPUT_HSC_MIPI_MIX_IPP_IND_SENS2_DATA_EN	55
#define	SWINPUT_HSC_MIPI_MIX_PAR0_VSYNC	56
#define	SWINPUT_HSC_MIPI_MIX_PAR1_DI_WAIT	57
#define	SWINPUT_HSC_MIPI_MIX_PAR_SISG_TRIG	58
#define	SWINPUT_I2C1_IPP_SCL_IN	59
#define	SWINPUT_I2C1_IPP_SDA_IN	60
#define	SWINPUT_I2C2_IPP_SCL_IN	61
#define	SWINPUT_I2C2_IPP_SDA_IN	62
#define	SWINPUT_IPU_IPP_DI_0_IND_DISPB_SD_D	63
#define	SWINPUT_IPU_IPP_DI_1_IND_DISPB_SD_D	64
#define	SWINPUT_KPP_IPP_IND_COL_6	65
#define	SWINPUT_KPP_IPP_IND_COL_7	66
#define	SWINPUT_KPP_IPP_IND_ROW_4	67
#define	SWINPUT_KPP_IPP_IND_ROW_5	68
#define	SWINPUT_KPP_IPP_IND_ROW_6	69
#define	SWINPUT_KPP_IPP_IND_ROW_7	70
#define	SWINPUT_UART1_IPP_UART_RTS_B	71
#define	SWINPUT_UART1_IPP_UART_RXD_MUX	72
#define	SWINPUT_UART2_IPP_UART_RTS_B	73
#define	SWINPUT_UART2_IPP_UART_RXD_MUX	74
#define	SWINPUT_UART3_IPP_UART_RTS_B	75
#define	SWINPUT_UART3_IPP_UART_RXD_MUX	76
#define	SWINPUT_USBOH3_IPP_IND_UH3_CLK	77
#define	SWINPUT_USBOH3_IPP_IND_UH3_DATA_0	78
#define	SWINPUT_USBOH3_IPP_IND_UH3_DATA_1	79
#define	SWINPUT_USBOH3_IPP_IND_UH3_DATA_2	80
#define	SWINPUT_USBOH3_IPP_IND_UH3_DATA_3	81
#define	SWINPUT_USBOH3_IPP_IND_UH3_DATA_4	82
#define	SWINPUT_USBOH3_IPP_IND_UH3_DATA_5	83
#define	SWINPUT_USBOH3_IPP_IND_UH3_DATA_6	84
#define	SWINPUT_USBOH3_IPP_IND_UH3_DATA_7	85
#define	SWINPUT_USBOH3_IPP_IND_UH3_DIR	86
#define	SWINPUT_USBOH3_IPP_IND_UH3_NXT	87
#define	SWINPUT_USBOH3_IPP_IND_UH3_STP	88


/*
 * Bit definitions for SW_MUX_CTL registers
 */
#define MUX_CTL_SION			(0x1 << 4)
#define	MUX_CTL_MUX_MODE_ALT0		0
#define	MUX_CTL_MUX_MODE_ALT1		1
#define MUX_CTL_MUX_MODE_ALT2		2
#define	MUX_CTL_MUX_MODE_ALT3		3
#define	MUX_CTL_MUX_MODE_ALT4		4
#define	MUX_CTL_MUX_MODE_ALT5		5
#define	MUX_CTL_MUX_MODE_ALT6		6
#define	MUX_CTL_MUX_MODE_ALT7		7


/*
 * Bit definitions for SW_PAD_CTL registers
 */

/* GPIO type pads */
#define PAD_CTL_GPIO_DRV_3_3_V          (0x0 << 13)
#define PAD_CTL_GPIO_DRV_1_8_V          (0x1 << 13)
#define PAD_CTL_GPIO_HYS_DISABLE        (0x0 << 8)
#define PAD_CTL_GPIO_HYS_ENABLE         (0x1 << 8)
#define PAD_CTL_GPIO_PKE_DISABLE        (0x0 << 7)
#define PAD_CTL_GPIO_PKE_ENABLE         (0x1 << 7)
#define PAD_CTL_GPIO_PUE_PULL           (0x1 << 6)
#define PAD_CTL_GPIO_PUE_KEEP           (0x0 << 6)
#define PAD_CTL_GPIO_PUS_100K_PD        (0x0 << 4)
#define PAD_CTL_GPIO_PUS_47K_PU         (0x1 << 4)
#define PAD_CTL_GPIO_PUS_100K_PU        (0x2 << 4)
#define PAD_CTL_GPIO_PUS_22K_PU         (0x3 << 4)
#define PAD_CTL_GPIO_ODE_DISABLE        (0x0 << 3)
#define PAD_CTL_GPIO_ODE_ENABLE         (0x1 << 3)
#define PAD_CTL_GPIO_DSE_LOW            (0x0 << 1)
#define PAD_CTL_GPIO_DSE_MID            (0x1 << 1)
#define PAD_CTL_GPIO_DSE_HIGH           (0x2 << 1)
#define PAD_CTL_GPIO_DSE_MAX            (0x3 << 1)
#define PAD_CTL_GPIO_SRE_SLOW           (0x0 << 0)
#define PAD_CTL_GPIO_SRE_FAST           (0x1 << 0)

/* DDR type pads */
#define PAD_CTL_DDR_DRV_3_3_V		(0x0 << 13)
#define PAD_CTL_DDR_DRV_1_8_V		(0x1 << 13)
#define PAD_CTL_DDR_TYPE_MOBILE		(0x0 << 11)
#define PAD_CTL_DDR_TYPE_SDRAM		(0x1 << 11)
#define PAD_CTL_DDR_TYPE_DDR2		(0x2 << 11)
#define PAD_CTL_DDR_PKE_DISABLE		(0x0 << 7)
#define PAD_CTL_DDR_PKE_ENABLE		(0x1 << 7)
#define PAD_CTL_DDR_ODE_DISABLE		(0x0 << 3)
#define PAD_CTL_DDR_ODE_ENABLE		(0x1 << 3)
#define PAD_CTL_DDR_DSE_STD		(0x0 << 1)
#define PAD_CTL_DDR_DSE_HIGH		(0x1 << 1)
#define PAD_CTL_DDR_DSE_MAX		(0x2 << 1)

/*
 * Function prototypes
 */
void pinmux_set_swmux(uintptr_t iomuxc_base, int pin, int mux_config);
void pinmux_set_padcfg(uintptr_t iomuxc_base, int pin, int pad_config);
void pinmux_set_input(uintptr_t iomuxc_base, int pin, int input_config);


#endif	/* __ARM_MX51_IOMUX_H_INCLUDED */

#if defined(__QNXNTO__) && defined(__USESRCVERSION)
#include <sys/srcversion.h>
__SRCVERSION("$URL: http://svn/product/branches/6.6.0/trunk/hardware/spi/mx51ecspi/mx51_iomux.h $ $Rev: 680332 $")
#endif
